@@ -20,6 +20,7 @@ import org.gradle.api.Project;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.resources.TextResource;
 
+import javax.annotation.Nullable;
 import java.util.Arrays;
 import java.util.List;
 
@@ -82,7 +83,7 @@ public class PmdExtension extends CodeQualityExtension {
     /**
      * Sets the target jdk used with pmd.
      *
-     * @value The target jdk
+     * @param targetJdk The target jdk
      * @since 4.0
      */
     public void setTargetJdk(TargetJdk targetJdk) {
@@ -92,7 +93,7 @@ public class PmdExtension extends CodeQualityExtension {
     /**
      * Sets the target jdk used with pmd.
      *
-     * @value The value for the target jdk as defined by {@link TargetJdk#toVersion(Object)}
+     * @param value The value for the target jdk as defined by {@link TargetJdk#toVersion(Object)}
      */
     public void setTargetJdk(Object value) {
         targetJdk = TargetJdk.toVersion(value);
@@ -131,6 +132,7 @@ public class PmdExtension extends CodeQualityExtension {
      * @since 2.2
      */
     @Incubating
+    @Nullable
     public TextResource getRuleSetConfig() {
         return ruleSetConfig;
     }
@@ -145,7 +147,7 @@ public class PmdExtension extends CodeQualityExtension {
      * @since 2.2
      */
     @Incubating
-    public void setRuleSetConfig(TextResource ruleSetConfig) {
+    public void setRuleSetConfig(@Nullable TextResource ruleSetConfig) {
         this.ruleSetConfig = ruleSetConfig;
     }
 
