@@ -24,8 +24,10 @@ import org.gradle.api.reporting.internal.CustomizableHtmlReportImpl;
 import org.gradle.api.reporting.internal.TaskGeneratedSingleFileReport;
 import org.gradle.api.reporting.internal.TaskReportContainer;
 
-public class FindBugsReportsImpl extends TaskReportContainer<SingleFileReport> implements FindBugsReportsInternal {
+import javax.inject.Inject;
 
+public class FindBugsReportsImpl extends TaskReportContainer<SingleFileReport> implements FindBugsReportsInternal {
+    @Inject
     public FindBugsReportsImpl(Task task) {
         super(SingleFileReport.class, task);
 
@@ -50,10 +52,4 @@ public class FindBugsReportsImpl extends TaskReportContainer<SingleFileReport> i
     public SingleFileReport getEmacs() {
         return getByName("emacs");
     }
-
-    @Override
-    public Boolean getWithMessagesFlag() {
-        FindBugsXmlReport report = (FindBugsXmlReport)getEnabled().findByName("xml");
-        return report != null ? report.isWithMessages() : Boolean.FALSE;
-    }
 }
