@@ -19,21 +19,17 @@ package org.gradle.composite.internal;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
 import org.gradle.api.GradleException;
-import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.BuildIdentifier;
 import org.gradle.api.artifacts.component.ProjectComponentIdentifier;
-import org.gradle.api.initialization.ConfigurableIncludedBuild;
 import org.gradle.api.initialization.IncludedBuild;
 import org.gradle.api.internal.BuildDefinition;
 import org.gradle.api.internal.SettingsInternal;
-import org.gradle.api.internal.artifacts.component.DefaultBuildIdentifier;
+import org.gradle.api.internal.artifacts.DefaultBuildIdentifier;
 import org.gradle.api.internal.composite.CompositeBuildContext;
 import org.gradle.api.internal.project.ProjectRegistry;
 import org.gradle.api.specs.Spec;
 import org.gradle.initialization.DefaultProjectDescriptor;
 import org.gradle.initialization.NestedBuildFactory;
-import org.gradle.internal.Cast;
-import org.gradle.internal.Pair;
 import org.gradle.internal.component.local.model.DefaultProjectComponentIdentifier;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.concurrent.Stoppable;
@@ -41,8 +37,10 @@ import org.gradle.util.CollectionUtils;
 import org.gradle.util.Path;
 
 import java.io.File;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
@@ -62,14 +60,13 @@ public class DefaultIncludedBuildRegistry implements IncludedBuildRegistry, Stop
         compositeBuildContext.setIncludedBuildRegistry(this);
     }
 
-    @Override
     public boolean hasIncludedBuilds() {
         return !includedBuilds.isEmpty();
     }
 
     @Override
-    public Collection<IncludedBuild> getIncludedBuilds() {
-        return Cast.uncheckedCast(includedBuilds.values());
+    public Collection<IncludedBuildInternal> getIncludedBuilds() {
+        return includedBuilds.values();
     }
 
     @Override
@@ -78,35 +75,34 @@ public class DefaultIncludedBuildRegistry implements IncludedBuildRegistry, Stop
     }
 
     @Override
-    public IncludedBuild getBuild(final BuildIdentifier buildIdentifier) {
-        return CollectionUtils.findFirst(includedBuilds.values(), new Spec<IncludedBuild>() {
+    public IncludedBuildInternal getBuild(final BuildIdentifier buildIdentifier) {
+        return CollectionUtils.findFirst(includedBuilds.values(), new Spec<IncludedBuildInternal>() {
             @Override
-            public boolean isSatisfiedBy(IncludedBuild includedBuild) {
+            public boolean isSatisfiedBy(IncludedBuildInternal includedBuild) {
                 return includedBuild.getName().equals(buildIdentifier.getName());
             }
         });
     }
 
-    @Override
-    public Collection<Pair<ModuleVersionIdentifier, ProjectComponentIdentifier>> getModuleToProjectMapping(IncludedBuild includedBuild) {
-        return ((IncludedBuildInternal)includedBuild).getAvailableModules();
-    }
-
     @Override
     public void validateExplicitIncludedBuilds(SettingsInternal settings) {
         validateIncludedBuilds(settings);
 
         registerRootBuildProjects(settings);
-        Collection<IncludedBuild> includedBuilds = getIncludedBuilds();
+        Collection<IncludedBuildInternal> includedBuilds = getIncludedBuilds();
+        List<IncludedBuild> modelElements = new ArrayList<IncludedBuild>(includedBuilds.size());
+        for (IncludedBuildInternal includedBuild : includedBuilds) {
+            modelElements.add(includedBuild.getModel());
+        }
         // Set the only visible included builds from the root build
-        settings.getGradle().setIncludedBuilds(includedBuilds);
+        settings.getGradle().setIncludedBuilds(modelElements);
         registerProjects(includedBuilds, false);
         registerSubstitutions(includedBuilds);
     }
 
     private void validateIncludedBuilds(SettingsInternal settings) {
         Set<String> names = Sets.newHashSet();
-        for (IncludedBuild build : includedBuilds.values()) {
+        for (IncludedBuildInternal build : includedBuilds.values()) {
             String buildName = build.getName();
             if (!names.add(buildName)) {
                 throw new GradleException("Included build '" + buildName + "' is not unique in composite.");
@@ -120,19 +116,19 @@ public class DefaultIncludedBuildRegistry implements IncludedBuildRegistry, Stop
         }
     }
 
-    private void registerSubstitutions(Iterable<IncludedBuild> includedBuilds) {
-        for (IncludedBuild includedBuild : includedBuilds) {
-            dependencySubstitutionsBuilder.build((IncludedBuildInternal)includedBuild);
+    private void registerSubstitutions(Iterable<IncludedBuildInternal> includedBuilds) {
+        for (IncludedBuildInternal includedBuild : includedBuilds) {
+            dependencySubstitutionsBuilder.build(includedBuild);
         }
     }
 
     @Override
-    public ConfigurableIncludedBuild addImplicitBuild(BuildDefinition buildDefinition, NestedBuildFactory nestedBuildFactory) {
+    public IncludedBuildInternal addImplicitBuild(BuildDefinition buildDefinition, NestedBuildFactory nestedBuildFactory) {
         // TODO: synchronization
-        ConfigurableIncludedBuild includedBuild = includedBuilds.get(buildDefinition.getBuildRootDir());
+        IncludedBuildInternal includedBuild = includedBuilds.get(buildDefinition.getBuildRootDir());
         if (includedBuild == null) {
             includedBuild = registerBuild(buildDefinition, nestedBuildFactory);
-            registerProjects(Collections.<IncludedBuild>singletonList(includedBuild), true);
+            registerProjects(Collections.singletonList(includedBuild), true);
         }
         // TODO: else, verify that the build definition is the same
         return includedBuild;
@@ -151,16 +147,15 @@ public class DefaultIncludedBuildRegistry implements IncludedBuildRegistry, Stop
 
     private void registerRootBuildProjects(SettingsInternal settings) {
         ProjectRegistry<DefaultProjectDescriptor> settingsProjectRegistry = settings.getProjectRegistry();
-        String rootName = settingsProjectRegistry.getRootProject().getName();
-        DefaultBuildIdentifier buildIdentifier = new DefaultBuildIdentifier(rootName, true);
+        BuildIdentifier buildIdentifier = DefaultBuildIdentifier.ROOT;
         registerProjects(Path.ROOT, buildIdentifier, settingsProjectRegistry.getAllProjects(), false);
     }
 
-    private void registerProjects(Iterable<IncludedBuild> includedBuilds, boolean isImplicitBuild) {
-        for (IncludedBuild includedBuild : includedBuilds) {
+    private void registerProjects(Iterable<IncludedBuildInternal> includedBuilds, boolean isImplicitBuild) {
+        for (IncludedBuildInternal includedBuild : includedBuilds) {
             Path rootProjectPath = Path.ROOT.child(includedBuild.getName());
             BuildIdentifier buildIdentifier = new DefaultBuildIdentifier(includedBuild.getName());
-            Set<DefaultProjectDescriptor> allProjects = ((IncludedBuildInternal) includedBuild).getLoadedSettings().getProjectRegistry().getAllProjects();
+            Set<DefaultProjectDescriptor> allProjects = includedBuild.getLoadedSettings().getProjectRegistry().getAllProjects();
             registerProjects(rootProjectPath, buildIdentifier, allProjects, isImplicitBuild);
         }
     }
@@ -169,7 +164,7 @@ public class DefaultIncludedBuildRegistry implements IncludedBuildRegistry, Stop
         for (DefaultProjectDescriptor project : allProjects) {
             Path projectIdentityPath = rootPath.append(project.path());
             ProjectComponentIdentifier projectComponentIdentifier = DefaultProjectComponentIdentifier.newProjectId(buildIdentifier, project.getPath());
-            projectRegistry.add(projectIdentityPath, projectComponentIdentifier, isImplicitBuild);
+            projectRegistry.add(projectIdentityPath, project.getName(), projectComponentIdentifier, isImplicitBuild);
         }
     }
 
