@@ -16,8 +16,9 @@
 
 package org.gradle.api.internal.changedetection.state;
 
-import com.google.common.hash.HashCode;
-import org.gradle.api.Nullable;
+import org.gradle.internal.hash.HashCode;
+
+import javax.annotation.Nullable;
 
 public interface RegularFileHasher {
     /**
