@@ -17,10 +17,12 @@
 package org.gradle.api.internal.component;
 
 import org.gradle.api.Named;
+import org.gradle.api.artifacts.DependencyConstraint;
 import org.gradle.api.artifacts.ModuleDependency;
 import org.gradle.api.artifacts.PublishArtifact;
 import org.gradle.api.attributes.HasAttributes;
 import org.gradle.api.attributes.Usage;
+import org.gradle.api.capabilities.Capability;
 
 import java.util.Set;
 
@@ -29,4 +31,6 @@ public interface UsageContext extends HasAttributes, Named {
     Usage getUsage();
     Set<? extends PublishArtifact> getArtifacts();
     Set<? extends ModuleDependency> getDependencies();
+    Set<? extends DependencyConstraint> getDependencyConstraints();
+    Set<? extends Capability> getCapabilities();
 }
