@@ -22,15 +22,23 @@ import org.gradle.api.Transformer;
 import org.gradle.api.file.FileVisitDetails;
 import org.gradle.api.file.FileVisitor;
 import org.gradle.api.file.RelativePath;
-import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.file.AbstractFileTreeElement;
 import org.gradle.api.internal.file.FileSystemSubset;
 import org.gradle.internal.Factory;
+import org.gradle.internal.io.StreamByteBuffer;
 import org.gradle.internal.nativeintegration.filesystem.Chmod;
 import org.gradle.util.CollectionUtils;
+import org.gradle.util.ConfigureUtil;
 
-import java.io.*;
-import java.util.*;
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.util.Arrays;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 /**
@@ -40,18 +48,20 @@ public class MapFileTree implements MinimalFileTree, FileSystemMirroringFileTree
     private final Map<RelativePath, Action<OutputStream>> elements = new LinkedHashMap<RelativePath, Action<OutputStream>>();
     private final Factory<File> tmpDirSource;
     private final Chmod chmod;
+    private final DirectoryFileTreeFactory directoryFileTreeFactory;
 
-    public MapFileTree(final File tmpDir, Chmod chmod) {
+    public MapFileTree(final File tmpDir, Chmod chmod, DirectoryFileTreeFactory directoryFileTreeFactory) {
         this(new Factory<File>() {
                 public File create() {
                     return tmpDir;
                 }
-        }, chmod);
+        }, chmod, directoryFileTreeFactory);
     }
 
-    public MapFileTree(Factory<File> tmpDirSource, Chmod chmod) {
+    public MapFileTree(Factory<File> tmpDirSource, Chmod chmod, DirectoryFileTreeFactory directoryFileTreeFactory) {
         this.tmpDirSource = tmpDirSource;
         this.chmod = chmod;
+        this.directoryFileTreeFactory = directoryFileTreeFactory;
     }
 
     private File getTmpDir() {
@@ -63,7 +73,7 @@ public class MapFileTree implements MinimalFileTree, FileSystemMirroringFileTree
     }
 
     public DirectoryFileTree getMirror() {
-        return new DirectoryFileTree(getTmpDir());
+        return directoryFileTreeFactory.create(getTmpDir());
     }
 
     public void visit(FileVisitor visitor) {
@@ -93,7 +103,7 @@ public class MapFileTree implements MinimalFileTree, FileSystemMirroringFileTree
      * of the element to.
      */
     public void add(String path, Closure contentClosure) {
-        Action<OutputStream> action = new ClosureBackedAction<OutputStream>(contentClosure);
+        Action<OutputStream> action = ConfigureUtil.configureUsing(contentClosure);
         add(path, action);
     }
 
@@ -178,7 +188,7 @@ public class MapFileTree implements MinimalFileTree, FileSystemMirroringFileTree
         // remains the same as the content in the existing file
         private void updateFileOnlyWhenGeneratedContentChanges() {
             byte[] generatedContent = generateContent();
-            if (hasGeneratedContentChanged(generatedContent)) {
+            if (!hasContent(generatedContent, file)) {
                 try {
                     Files.write(generatedContent, file);
                 } catch (IOException e) {
@@ -188,27 +198,25 @@ public class MapFileTree implements MinimalFileTree, FileSystemMirroringFileTree
         }
 
         private byte[] generateContent() {
-            ByteArrayOutputStream buffer = new ByteArrayOutputStream(Math.max(((int) file.length()) + 64, 256));
-            copyTo(buffer);
-            return buffer.toByteArray();
+            StreamByteBuffer buffer = new StreamByteBuffer();
+            copyTo(buffer.getOutputStream());
+            return buffer.readAsByteArray();
         }
 
-        private boolean hasGeneratedContentChanged(byte[] generatedContent) {
-            boolean hasChanged = false;
-            if (generatedContent.length == file.length()) {
-                try {
-                    byte[] existingContent = Files.toByteArray(file);
-                    if (!Arrays.equals(generatedContent, existingContent)) {
-                        hasChanged = true;
-                    }
-                } catch (IOException e) {
-                    // attempt to write new file if reading old file fails
-                    hasChanged = true;
-                }
-            } else {
-                hasChanged = true;
+        private boolean hasContent(byte[] generatedContent, File file) {
+            if (generatedContent.length != file.length()) {
+                return false;
             }
-            return hasChanged;
+
+            byte[] existingContent;
+            try {
+                existingContent = Files.toByteArray(this.file);
+            } catch (IOException e) {
+                // Assume changed if reading old file fails
+                return false;
+            }
+
+            return Arrays.equals(generatedContent, existingContent);
         }
 
         public boolean isDirectory() {
