@@ -36,8 +36,8 @@ import org.gradle.internal.exceptions.MultiCauseException;
 import org.gradle.internal.operations.BuildOperationContext;
 import org.gradle.internal.operations.BuildOperationExecutor;
 import org.gradle.internal.operations.RunnableBuildOperation;
-import org.gradle.internal.progress.BuildOperationDescriptor;
-import org.gradle.internal.progress.BuildOperationState;
+import org.gradle.internal.operations.BuildOperationDescriptor;
+import org.gradle.internal.operations.BuildOperationRef;
 import org.gradle.internal.scopeids.id.BuildInvocationScopeId;
 import org.gradle.internal.work.AsyncWorkTracker;
 
@@ -65,7 +65,7 @@ public class ExecuteActionsTaskExecuter implements TaskExecuter {
 
     public void execute(TaskInternal task, TaskStateInternal state, TaskExecutionContext context) {
         listener.beforeActions(task);
-        if (!task.getTaskActions().isEmpty()) {
+        if (task.hasTaskActions()) {
             outputsGenerationListener.beforeTaskOutputChanged();
         }
         state.setExecuting(true);
@@ -118,7 +118,7 @@ public class ExecuteActionsTaskExecuter implements TaskExecuter {
 
             @Override
             public void run(BuildOperationContext context) {
-                BuildOperationState currentOperation = buildOperationExecutor.getCurrentOperation();
+                BuildOperationRef currentOperation = buildOperationExecutor.getCurrentOperation();
                 Throwable actionFailure = null;
                 try {
                     action.execute(task);
