@@ -17,6 +17,7 @@
 package org.gradle.caching.internal.tasks;
 
 import org.gradle.StartParameter;
+import org.gradle.api.NonNullApi;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.InstantiatorFactory;
 import org.gradle.api.internal.cache.StringInterner;
@@ -41,7 +42,6 @@ import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.time.Clock;
 import org.gradle.util.GradleVersion;
 import org.gradle.util.Path;
-import org.gradle.util.SingleMessageLogger;
 
 import java.io.File;
 
@@ -50,6 +50,7 @@ import static org.gradle.caching.internal.controller.BuildCacheControllerFactory
 import static org.gradle.caching.internal.controller.BuildCacheControllerFactory.RemoteAccessMode.OFFLINE;
 import static org.gradle.caching.internal.controller.BuildCacheControllerFactory.RemoteAccessMode.ONLINE;
 
+@NonNullApi
 public class BuildCacheTaskServices {
 
     private static final Path ROOT_BUILD_SRC_PATH = Path.path(":" + BuildSourceBuilder.BUILD_SRC);
@@ -118,10 +119,6 @@ public class BuildCacheTaskServices {
         RemoteAccessMode remoteAccessMode = startParameter.isOffline() ? OFFLINE : ONLINE;
         boolean logStackTraces = startParameter.getShowStacktrace() != ShowStacktrace.INTERNAL_EXCEPTIONS;
 
-        if (buildCacheMode == ENABLED) {
-            SingleMessageLogger.incubatingFeatureUsed("Build cache");
-        }
-
         return BuildCacheControllerFactory.create(
             buildOperationExecutor,
             buildIdentityPath,
