@@ -20,19 +20,18 @@ import org.gradle.api.Action;
 import org.gradle.api.internal.TaskInternal;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
-import org.gradle.internal.concurrent.ExecutorFactory;
 import org.gradle.concurrent.ParallelismConfiguration;
+import org.gradle.internal.concurrent.ExecutorFactory;
 import org.gradle.internal.concurrent.ManagedExecutor;
+import org.gradle.internal.time.Time;
+import org.gradle.internal.time.TimeFormatting;
 import org.gradle.internal.time.Timer;
-import org.gradle.internal.time.Timers;
 import org.gradle.internal.work.WorkerLeaseRegistry.WorkerLease;
 import org.gradle.internal.work.WorkerLeaseService;
 
 import java.util.concurrent.Executor;
 import java.util.concurrent.atomic.AtomicLong;
 
-import static org.gradle.internal.time.Clock.prettyTime;
-
 class DefaultTaskPlanExecutor implements TaskPlanExecutor {
     private static final Logger LOGGER = Logging.getLogger(DefaultTaskPlanExecutor.class);
     private final int executorCount;
@@ -90,23 +89,23 @@ class DefaultTaskPlanExecutor implements TaskPlanExecutor {
 
         public void run() {
             final AtomicLong busy = new AtomicLong(0);
-            Timer totalTimer = Timers.startTimer();
-            final Timer taskTimer = Timers.startTimer();
+            Timer totalTimer = Time.startTimer();
+            final Timer taskTimer = Time.startTimer();
 
             WorkerLease childLease = parentWorkerLease.createChild();
             boolean moreTasksToExecute = true;
             while (moreTasksToExecute) {
-                moreTasksToExecute = taskExecutionPlan.executeWithTask(childLease, new Action<TaskInfo>() {
+                moreTasksToExecute = taskExecutionPlan.executeWithTask(childLease, new Action<TaskInternal>() {
                     @Override
-                    public void execute(TaskInfo task) {
-                        final String taskPath = task.getTask().getPath();
+                    public void execute(TaskInternal task) {
+                        final String taskPath = task.getPath();
                         LOGGER.info("{} ({}) started.", taskPath, Thread.currentThread());
                         taskTimer.reset();
-                        processTask(task);
+                        taskWorker.execute(task);
                         long taskDuration = taskTimer.getElapsedMillis();
                         busy.addAndGet(taskDuration);
                         if (LOGGER.isInfoEnabled()) {
-                            LOGGER.info("{} ({}) completed. Took {}.", taskPath, Thread.currentThread(), prettyTime(taskDuration));
+                            LOGGER.info("{} ({}) completed. Took {}.", taskPath, Thread.currentThread(), TimeFormatting.formatDurationVerbose(taskDuration));
                         }
                     }
                 });
@@ -115,17 +114,7 @@ class DefaultTaskPlanExecutor implements TaskPlanExecutor {
             long total = totalTimer.getElapsedMillis();
 
             if (LOGGER.isDebugEnabled()) {
-                LOGGER.debug("Task worker [{}] finished, busy: {}, idle: {}", Thread.currentThread(), prettyTime(busy.get()), prettyTime(total - busy.get()));
-            }
-        }
-
-        private void processTask(TaskInfo taskInfo) {
-            try {
-                taskWorker.execute(taskInfo.getTask());
-            } catch (Throwable e) {
-                taskInfo.setExecutionFailure(e);
-            } finally {
-                taskExecutionPlan.taskComplete(taskInfo);
+                LOGGER.debug("Task worker [{}] finished, busy: {}, idle: {}", Thread.currentThread(), TimeFormatting.formatDurationVerbose(busy.get()), TimeFormatting.formatDurationVerbose(total - busy.get()));
             }
         }
     }
