@@ -17,8 +17,13 @@
 package org.gradle.initialization.buildsrc;
 
 import org.gradle.StartParameter;
+import org.gradle.api.internal.BuildDefinition;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
+import org.gradle.cache.FileLock;
+import org.gradle.cache.FileLockManager;
+import org.gradle.cache.LockOptions;
+import org.gradle.initialization.DefaultSettings;
 import org.gradle.initialization.GradleLauncher;
 import org.gradle.initialization.NestedBuildFactory;
 import org.gradle.internal.classpath.CachedClasspathTransformer;
@@ -28,53 +33,64 @@ import org.gradle.internal.invocation.GradleBuildController;
 import org.gradle.internal.operations.BuildOperationContext;
 import org.gradle.internal.operations.BuildOperationExecutor;
 import org.gradle.internal.operations.CallableBuildOperation;
-import org.gradle.internal.progress.BuildOperationDescriptor;
+import org.gradle.internal.operations.BuildOperationDescriptor;
+import org.gradle.plugin.management.internal.DefaultPluginRequests;
 import org.gradle.util.Path;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.File;
+
+import static org.gradle.cache.internal.filelock.LockOptionsBuilder.mode;
+
 public class BuildSourceBuilder {
     private static final Logger LOGGER = LoggerFactory.getLogger(BuildSourceBuilder.class);
-    public static final BuildBuildSrcBuildOperationType.Details BUILD_BUILDSRC_DETAILS = new BuildBuildSrcBuildOperationType.Details() {
-    };
-    public static final BuildBuildSrcBuildOperationType.Result BUILD_BUILDSRC_RESULT = new BuildBuildSrcBuildOperationType.Result() {
+    private static final BuildBuildSrcBuildOperationType.Result BUILD_BUILDSRC_RESULT = new BuildBuildSrcBuildOperationType.Result() {
     };
     public static final String BUILD_SRC = "buildSrc";
 
     private final NestedBuildFactory nestedBuildFactory;
     private final ClassLoaderScope classLoaderScope;
+    private final FileLockManager fileLockManager;
     private final BuildOperationExecutor buildOperationExecutor;
     private final CachedClasspathTransformer cachedClasspathTransformer;
     private final BuildSrcBuildListenerFactory buildSrcBuildListenerFactory;
 
-    public BuildSourceBuilder(NestedBuildFactory nestedBuildFactory, ClassLoaderScope classLoaderScope, BuildOperationExecutor buildOperationExecutor, CachedClasspathTransformer cachedClasspathTransformer, BuildSrcBuildListenerFactory buildSrcBuildListenerFactory) {
+    public BuildSourceBuilder(NestedBuildFactory nestedBuildFactory, ClassLoaderScope classLoaderScope, FileLockManager fileLockManager, BuildOperationExecutor buildOperationExecutor, CachedClasspathTransformer cachedClasspathTransformer, BuildSrcBuildListenerFactory buildSrcBuildListenerFactory) {
         this.nestedBuildFactory = nestedBuildFactory;
         this.classLoaderScope = classLoaderScope;
+        this.fileLockManager = fileLockManager;
         this.buildOperationExecutor = buildOperationExecutor;
         this.cachedClasspathTransformer = cachedClasspathTransformer;
         this.buildSrcBuildListenerFactory = buildSrcBuildListenerFactory;
     }
 
-    public ClassLoaderScope buildAndCreateClassLoader(StartParameter startParameter) {
-        ClassPath classpath = createBuildSourceClasspath(startParameter);
-        return classLoaderScope.createChild(startParameter.getCurrentDir().getAbsolutePath())
-            .export(cachedClasspathTransformer.transform(classpath))
+    public ClassLoaderScope buildAndCreateClassLoader(GradleInternal gradle, File rootDir, StartParameter containingBuildParameters) {
+        File buildSrcDir = new File(rootDir, DefaultSettings.DEFAULT_BUILD_SRC_DIR);
+        ClassPath classpath = createBuildSourceClasspath(gradle, buildSrcDir, containingBuildParameters);
+        return classLoaderScope.createChild(buildSrcDir.getAbsolutePath())
+            .export(classpath)
             .lock();
     }
 
-    ClassPath createBuildSourceClasspath(final StartParameter startParameter) {
-        assert startParameter.getCurrentDir() != null && startParameter.getBuildFile() == null;
-
-        LOGGER.debug("Starting to build the build sources.");
-        if (!startParameter.getCurrentDir().isDirectory()) {
+    private ClassPath createBuildSourceClasspath(final GradleInternal gradle, File buildSrcDir, final StartParameter containingBuildParameters) {
+        if (!buildSrcDir.isDirectory()) {
             LOGGER.debug("Gradle source dir does not exist. We leave.");
             return ClassPath.EMPTY;
         }
 
+        final StartParameter buildSrcStartParameter = containingBuildParameters.newBuild();
+        buildSrcStartParameter.setCurrentDir(buildSrcDir);
+        buildSrcStartParameter.setProjectProperties(containingBuildParameters.getProjectProperties());
+        buildSrcStartParameter.setSearchUpwards(false);
+        buildSrcStartParameter.setProfile(containingBuildParameters.isProfile());
+        final BuildDefinition buildDefinition = BuildDefinition.fromStartParameterForBuild(buildSrcStartParameter, buildSrcDir, DefaultPluginRequests.EMPTY);
+        assert buildSrcStartParameter.getBuildFile() == null;
+
         return buildOperationExecutor.call(new CallableBuildOperation<ClassPath>() {
             @Override
             public ClassPath call(BuildOperationContext context) {
-                ClassPath classPath = buildBuildSrc(startParameter);
+                ClassPath classPath = buildBuildSrc(buildDefinition);
                 context.setResult(BUILD_BUILDSRC_RESULT);
                 return classPath;
             }
@@ -83,31 +99,43 @@ public class BuildSourceBuilder {
             public BuildOperationDescriptor.Builder description() {
                 return BuildOperationDescriptor.displayName("Build buildSrc").
                     progressDisplayName("Building buildSrc").
-                    details(BUILD_BUILDSRC_DETAILS);
+                    details(new BuildBuildSrcBuildOperationType.Details(){
+
+                        @Override
+                        public String getBuildPath() {
+                            return gradle.getIdentityPath().toString();
+                        }
+                    });
             }
         });
     }
 
-    private ClassPath buildBuildSrc(StartParameter startParameter) {
-        BuildController buildController = createBuildController(startParameter);
+    private ClassPath buildBuildSrc(BuildDefinition buildDefinition) {
+        BuildController buildController = createBuildController(buildDefinition);
         try {
-            return new BuildSrcUpdateFactory(buildController, buildSrcBuildListenerFactory).create();
+            File lockTarget = new File(buildDefinition.getBuildRootDir(), ".gradle/noVersion/buildSrc");
+            FileLock lock = fileLockManager.lock(lockTarget, LOCK_OPTIONS, "buildSrc build lock");
+            try {
+                return new BuildSrcUpdateFactory(buildController, buildSrcBuildListenerFactory, cachedClasspathTransformer).create();
+            } finally {
+                lock.close();
+            }
         } finally {
             buildController.stop();
         }
     }
 
-    private BuildController createBuildController(StartParameter startParameter) {
-        GradleLauncher gradleLauncher = buildGradleLauncher(startParameter);
+    private static final LockOptions LOCK_OPTIONS = mode(FileLockManager.LockMode.Exclusive).useCrossVersionImplementation();
+
+    private BuildController createBuildController(BuildDefinition buildDefinition) {
+        GradleLauncher gradleLauncher = buildGradleLauncher(buildDefinition);
         return new GradleBuildController(gradleLauncher);
     }
 
-    private GradleLauncher buildGradleLauncher(StartParameter startParameter) {
-        StartParameter startParameterArg = startParameter.newInstance();
-        startParameterArg.setProjectProperties(startParameter.getProjectProperties());
-        startParameterArg.setSearchUpwards(false);
-        startParameterArg.setProfile(startParameter.isProfile());
-        GradleLauncher gradleLauncher = nestedBuildFactory.nestedInstance(startParameterArg);
+    private GradleLauncher buildGradleLauncher(BuildDefinition buildDefinition) {
+        // TODO: Migrate buildSrc builds to be more similar to other "included" builds.
+        GradleLauncher gradleLauncher = nestedBuildFactory.nestedInstance(buildDefinition);
+        StartParameter startParameter = buildDefinition.getStartParameter();
         GradleInternal build = gradleLauncher.getGradle();
         if (build.getParent().findIdentityPath() == null) {
             // When nested inside a nested build, we need to synthesize a path for this build, as the root project is not yet known for the parent build
