@@ -17,6 +17,7 @@
 package org.gradle.internal.operations.trace;
 
 import com.google.common.base.Charsets;
+import com.google.common.base.StandardSystemProperty;
 import com.google.common.io.Files;
 import com.google.common.io.LineProcessor;
 import groovy.json.JsonOutput;
@@ -28,11 +29,13 @@ import org.gradle.api.invocation.Gradle;
 import org.gradle.internal.UncheckedException;
 import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.event.ListenerManager;
-import org.gradle.internal.progress.BuildOperationDescriptor;
-import org.gradle.internal.progress.BuildOperationListener;
-import org.gradle.internal.progress.OperationFinishEvent;
-import org.gradle.internal.progress.OperationProgressEvent;
-import org.gradle.internal.progress.OperationStartEvent;
+import org.gradle.internal.operations.BuildOperationDescriptor;
+import org.gradle.internal.operations.BuildOperationListener;
+import org.gradle.internal.operations.BuildOperationListenerManager;
+import org.gradle.internal.operations.OperationFinishEvent;
+import org.gradle.internal.operations.OperationIdentifier;
+import org.gradle.internal.operations.OperationProgressEvent;
+import org.gradle.internal.operations.OperationStartEvent;
 import org.gradle.util.GFileUtils;
 
 import javax.annotation.Nonnull;
@@ -51,6 +54,7 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Queue;
+import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 
@@ -90,11 +94,14 @@ public class BuildOperationTrace implements Stoppable {
 
     private final String basePath;
     private final OutputStream logOutputStream;
+
+    private final BuildOperationListenerManager buildOperationListenerManager;
     private final ListenerManager listenerManager;
 
     private final BuildOperationListener listener = new LoggingListener();
 
-    public BuildOperationTrace(StartParameter startParameter, ListenerManager listenerManager) {
+    public BuildOperationTrace(StartParameter startParameter, BuildOperationListenerManager buildOperationListenerManager, ListenerManager listenerManager) {
+        this.buildOperationListenerManager = buildOperationListenerManager;
         this.listenerManager = listenerManager;
 
         Map<String, String> sysProps = startParameter.getSystemPropertiesArgs();
@@ -123,16 +130,20 @@ public class BuildOperationTrace implements Stoppable {
             throw UncheckedException.throwAsUncheckedException(e);
         }
 
+        buildOperationListenerManager.addListener(listener);
         listenerManager.addListener(listener);
     }
 
     @Override
     public void stop() {
+        buildOperationListenerManager.removeListener(listener);
         listenerManager.removeListener(listener);
 
         if (logOutputStream != null) {
             try {
-                logOutputStream.close();
+                synchronized (logOutputStream) {
+                    logOutputStream.close();
+                }
 
                 final List<BuildOperationRecord> roots = readLogToTreeRoots(logFile(basePath));
                 writeDetailTree(roots);
@@ -144,9 +155,13 @@ public class BuildOperationTrace implements Stoppable {
     }
 
     private void writeDetailTree(List<BuildOperationRecord> roots) throws IOException {
-        String rawJson = JsonOutput.toJson(BuildOperationTree.serialize(roots));
-        String prettyJson = JsonOutput.prettyPrint(rawJson);
-        Files.asCharSink(file(basePath, "-tree.json"), Charsets.UTF_8).write(prettyJson);
+        try {
+            String rawJson = JsonOutput.toJson(BuildOperationTree.serialize(roots));
+            String prettyJson = JsonOutput.prettyPrint(rawJson);
+            Files.asCharSink(file(basePath, "-tree.json"), Charsets.UTF_8).write(prettyJson);
+        } catch (OutOfMemoryError e) {
+            System.err.println("Failed to write build operation trace JSON due to out of memory.");
+        }
     }
 
     private void writeSummaryTree(final List<BuildOperationRecord> roots) throws IOException {
@@ -178,7 +193,9 @@ public class BuildOperationTrace implements Stoppable {
 
                         stringBuilder.setLength(0);
 
-                        for (int i = 0; i < stack.size() - 1; ++i) {
+                        int indents = stack.size() - 1;
+
+                        for (int i = 0; i < indents; ++i) {
                             stringBuilder.append("  ");
                         }
 
@@ -206,6 +223,18 @@ public class BuildOperationTrace implements Stoppable {
                         stringBuilder.append(record.id);
                         stringBuilder.append(")");
 
+                        if (!record.progress.isEmpty()) {
+                            for (BuildOperationRecord.Progress progress : record.progress) {
+                                stringBuilder.append(StandardSystemProperty.LINE_SEPARATOR.value());
+                                for (int i = 0; i < indents; ++i) {
+                                    stringBuilder.append("  ");
+                                }
+                                stringBuilder.append("- ")
+                                    .append(progress.details).append(" [")
+                                    .append(progress.time - record.startTime)
+                                    .append("]");
+                            }
+                        }
 
                         return stringBuilder.toString();
                     }
@@ -243,7 +272,7 @@ public class BuildOperationTrace implements Stoppable {
                     } else if (map.containsKey("time")) {
                         SerializedOperationProgress serialized = new SerializedOperationProgress(map);
                         PendingOperation pending = pendings.get(serialized.id);
-                        assert pending != null;
+                        assert pending != null : "did not find owner of progress event with ID " + serialized.id;
                         pending.progress.add(serialized);
                     } else {
                         SerializedOperationFinish finish = new SerializedOperationFinish(map);
@@ -344,7 +373,7 @@ public class BuildOperationTrace implements Stoppable {
         // truly immutable and convey values known at the time.
         private boolean buffering = true;
         private final Lock bufferLock = new ReentrantLock();
-        private final Queue<Entry> buffer = new ArrayDeque<Entry>();
+        private final Queue<Entry> buffer = new ConcurrentLinkedQueue<Entry>();
 
         @Override
         public void projectsLoaded(@SuppressWarnings("NullableProblems") Gradle gradle) {
@@ -361,12 +390,15 @@ public class BuildOperationTrace implements Stoppable {
 
         @Override
         public void started(BuildOperationDescriptor buildOperation, OperationStartEvent startEvent) {
+            if (buildOperation.getId().getId() == 11) {
+                int i = 1;
+            }
             new Entry(new SerializedOperationStart(buildOperation, startEvent), false).add();
         }
 
         @Override
-        public void progress(BuildOperationDescriptor buildOperation, OperationProgressEvent progressEvent) {
-            new Entry(new SerializedOperationProgress(buildOperation, progressEvent), false).add();
+        public void progress(OperationIdentifier buildOperationId, OperationProgressEvent progressEvent) {
+            new Entry(new SerializedOperationProgress(buildOperationId, progressEvent), false).add();
         }
 
         @Override
@@ -405,6 +437,11 @@ public class BuildOperationTrace implements Stoppable {
                     bufferLock.lock();
                     try {
                         if (buffering) {
+                            if (operation instanceof SerializedOperationStart) {
+                                if (((SerializedOperationStart) operation).id == 11) {
+                                    int i = 1;
+                                }
+                            }
                             buffer.add(this);
                         } else {
                             write();
@@ -421,11 +458,14 @@ public class BuildOperationTrace implements Stoppable {
             private void write() {
                 String json = JsonOutput.toJson(operation.toMap());
                 try {
-                    if (indent) {
-                        logOutputStream.write(INDENT);
+                    synchronized (logOutputStream) {
+                        if (indent) {
+                            logOutputStream.write(INDENT);
+                        }
+                        logOutputStream.write(json.getBytes("UTF-8"));
+                        logOutputStream.write(NEWLINE);
+                        logOutputStream.flush();
                     }
-                    logOutputStream.write(json.getBytes("UTF-8"));
-                    logOutputStream.write(NEWLINE);
                 } catch (IOException e) {
                     throw UncheckedException.throwAsUncheckedException(e);
                 }
