@@ -17,6 +17,7 @@
 package org.gradle.internal.service.scopes;
 
 import org.gradle.api.Action;
+import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.InstantiatorFactory;
 import org.gradle.api.internal.SettingsInternal;
 import org.gradle.api.internal.file.BaseDirFileResolver;
@@ -46,9 +47,7 @@ public class SettingsScopeServices extends DefaultServiceRegistry {
         register(new Action<ServiceRegistration>() {
             public void execute(ServiceRegistration registration) {
                 for (PluginServiceRegistry pluginServiceRegistry : parent.getAll(PluginServiceRegistry.class)) {
-                    if (pluginServiceRegistry instanceof SettingScopePluginServiceRegistry) {
-                        ((SettingScopePluginServiceRegistry) pluginServiceRegistry).registerSettingsServices(registration);
-                    }
+                    pluginServiceRegistry.registerSettingsServices(registration);
                 }
             }
         });
@@ -75,4 +74,7 @@ public class SettingsScopeServices extends DefaultServiceRegistry {
         return ConfigurationTargetIdentifier.of(settings);
     }
 
+    protected GradleInternal createGradleInternal() {
+        return settings.getGradle();
+    }
 }
