@@ -28,6 +28,7 @@ import org.gradle.api.ProjectEvaluationListener;
 import org.gradle.api.initialization.IncludedBuild;
 import org.gradle.api.initialization.Settings;
 import org.gradle.api.internal.GradleInternal;
+import org.gradle.api.internal.SettingsInternal;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.api.internal.initialization.ScriptHandlerFactory;
@@ -45,7 +46,7 @@ import org.gradle.internal.event.ListenerBroadcast;
 import org.gradle.internal.event.ListenerManager;
 import org.gradle.internal.installation.CurrentGradleInstallation;
 import org.gradle.internal.installation.GradleInstallation;
-import org.gradle.internal.progress.BuildOperationState;
+import org.gradle.internal.operations.BuildOperationRef;
 import org.gradle.internal.resource.TextResourceLoader;
 import org.gradle.internal.scan.config.BuildScanConfigInit;
 import org.gradle.internal.service.ServiceRegistry;
@@ -62,6 +63,7 @@ import java.util.Collections;
 import java.util.NoSuchElementException;
 
 public class DefaultGradle extends AbstractPluginAware implements GradleInternal {
+    private SettingsInternal settings;
     private ProjectInternal rootProject;
     private ProjectInternal defaultProject;
     private final GradleInternal parent;
@@ -74,7 +76,7 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
     private boolean projectsLoaded;
     private Path identityPath;
     private final ClassLoaderScope classLoaderScope;
-    private BuildOperationState operation;
+    private BuildOperationRef operation;
 
     public DefaultGradle(GradleInternal parent, StartParameter startParameter, ServiceRegistryFactory parentRegistry) {
         this.parent = parent;
@@ -145,7 +147,7 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
     }
 
     @Override
-    public BuildOperationState getBuildOperation() {
+    public BuildOperationRef getBuildOperation() {
         if (operation != null) {
             return operation;
         }
@@ -156,7 +158,7 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
     }
 
     @Override
-    public void setBuildOperation(BuildOperationState operation) {
+    public void setBuildOperation(BuildOperationRef operation) {
         this.operation = operation;
     }
 
@@ -195,6 +197,19 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
         return startParameter;
     }
 
+    @Override
+    public SettingsInternal getSettings() {
+        if (settings == null) {
+            throw new IllegalStateException("The settings are not yet available for " + this + ".");
+        }
+        return settings;
+    }
+
+    @Override
+    public void setSettings(SettingsInternal settings) {
+        this.settings = settings;
+    }
+
     @Override
     public ProjectInternal getRootProject() {
         if (rootProject == null) {
@@ -365,7 +380,7 @@ public class DefaultGradle extends AbstractPluginAware implements GradleInternal
     }
 
     @Override
-    public void setIncludedBuilds(Collection<IncludedBuild> includedBuilds) {
+    public void setIncludedBuilds(Collection<? extends IncludedBuild> includedBuilds) {
         this.includedBuilds.addAll(includedBuilds);
     }
 
