@@ -18,7 +18,6 @@ package org.gradle.api.internal.artifacts.configurations;
 import org.gradle.api.Action;
 import org.gradle.api.artifacts.DependencySubstitution;
 import org.gradle.api.artifacts.ResolutionStrategy;
-import org.gradle.api.artifacts.cache.ResolutionRules;
 import org.gradle.api.internal.artifacts.ComponentSelectionRulesInternal;
 import org.gradle.api.internal.artifacts.configurations.dynamicversion.CachePolicy;
 import org.gradle.api.internal.artifacts.ivyservice.dependencysubstitution.DependencySubstitutionsInternal;
@@ -39,12 +38,6 @@ public interface ResolutionStrategyInternal extends ResolutionStrategy {
      */
     ConflictResolution getConflictResolution();
 
-    /**
-     * The nascent DSL for cache control, and possibly other per-module resolution overrides
-     * @return the resolution rules
-     */
-    ResolutionRules getResolutionRules();
-
     /**
      * @return the dependency substitution rule (may aggregate multiple rules)
      */
