@@ -87,16 +87,12 @@ public class DynamicVersionResolver {
 
         final RepositoryChainModuleResolution latestResolved = findLatestModule(resolveStates, errors);
         if (latestResolved != null) {
-            LOGGER.debug("Using {} from {}", latestResolved.module.getId(), latestResolved.repository);
+            LOGGER.debug("Using {} from {}", latestResolved.module.getModuleVersionId(), latestResolved.repository);
             for (Throwable error : errors) {
                 LOGGER.debug("Discarding resolve failure.", error);
             }
 
             result.resolved(metaDataFactory.transform(latestResolved));
-            String reason = dependency.getReason();
-            if (reason != null) {
-                result.setSelectionDescription(result.getSelectionDescription().withReason(reason));
-            }
             return;
         }
         if (!errors.isEmpty()) {
