@@ -15,46 +15,39 @@
  */
 package org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser;
 
-import com.google.common.collect.ArrayListMultimap;
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.ListMultimap;
 import com.google.common.collect.Lists;
 import org.gradle.api.artifacts.ModuleIdentifier;
-import org.gradle.api.artifacts.ModuleVersionSelector;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
-import org.gradle.api.internal.artifacts.DefaultModuleVersionSelector;
-import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
+import org.gradle.api.artifacts.component.ModuleComponentSelector;
+import org.gradle.api.internal.artifacts.dependencies.DefaultImmutableVersionConstraint;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.PomReader.PomDependencyData;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.data.PomDependencyMgt;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelectorScheme;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.PatternMatchers;
-import org.gradle.internal.component.external.descriptor.Artifact;
 import org.gradle.internal.component.external.descriptor.Configuration;
 import org.gradle.internal.component.external.descriptor.DefaultExclude;
 import org.gradle.internal.component.external.descriptor.MavenScope;
 import org.gradle.internal.component.external.model.DefaultModuleComponentIdentifier;
-import org.gradle.internal.component.external.model.IvyDependencyMetadata;
-import org.gradle.internal.component.external.model.MavenDependencyMetadata;
+import org.gradle.internal.component.external.model.DefaultModuleComponentSelector;
+import org.gradle.internal.component.external.model.MavenDependencyDescriptor;
 import org.gradle.internal.component.model.DefaultIvyArtifactName;
-import org.gradle.internal.component.model.DependencyMetadata;
-import org.gradle.internal.component.model.Exclude;
+import org.gradle.internal.component.model.ExcludeMetadata;
 import org.gradle.internal.component.model.IvyArtifactName;
 
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
 /**
  * This a straight copy of org.apache.ivy.plugins.parser.m2.PomModuleDescriptorBuilder, with minor changes: 1) Do not create artifact for empty classifier. (Previously did so for all non-null
  * classifiers)
  */
 public class GradlePomModuleDescriptorBuilder {
-    public static final Map<String, Configuration> MAVEN2_CONFIGURATIONS = ImmutableMap.<String, Configuration>builder()
+    public static final ImmutableMap<String, Configuration> MAVEN2_CONFIGURATIONS = ImmutableMap.<String, Configuration>builder()
         .put("default", new Configuration("default", true, true, ImmutableSet.of("runtime", "master")))
         .put("master", new Configuration("master", true, true, ImmutableSet.<String>of()))
         .put("compile", new Configuration("compile", true, true, ImmutableSet.<String>of()))
@@ -73,26 +66,22 @@ public class GradlePomModuleDescriptorBuilder {
         .put("test", MavenScope.Test)
         .put("system", MavenScope.System)
         .build();
-    private static final Pattern TIMESTAMP_PATTERN = Pattern.compile("(.+)-\\d{8}\\.\\d{6}-\\d+");
-    private static final String[] WILDCARD = new String[]{"*"};
 
     private final VersionSelectorScheme defaultVersionSelectorScheme;
     private final VersionSelectorScheme mavenVersionSelectorScheme;
 
-    private List<DependencyMetadata> dependencies = Lists.newArrayList();
+    private List<MavenDependencyDescriptor> dependencies = Lists.newArrayList();
     private final PomReader pomReader;
-    private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
     private String status;
     private ModuleComponentIdentifier componentIdentifier;
 
-    public GradlePomModuleDescriptorBuilder(PomReader pomReader, VersionSelectorScheme gradleVersionSelectorScheme, VersionSelectorScheme mavenVersionSelectorScheme, ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
+    public GradlePomModuleDescriptorBuilder(PomReader pomReader, VersionSelectorScheme gradleVersionSelectorScheme, VersionSelectorScheme mavenVersionSelectorScheme) {
         this.defaultVersionSelectorScheme = gradleVersionSelectorScheme;
         this.mavenVersionSelectorScheme = mavenVersionSelectorScheme;
         this.pomReader = pomReader;
-        this.moduleIdentifierFactory = moduleIdentifierFactory;
     }
 
-    public List<DependencyMetadata> getDependencies() {
+    public List<MavenDependencyDescriptor> getDependencies() {
         return dependencies;
     }
 
@@ -105,46 +94,49 @@ public class GradlePomModuleDescriptorBuilder {
     }
 
     public void setModuleRevId(String group, String module, String version) {
-        String effectiveVersion = version;
-        if (version != null) {
-            Matcher matcher = TIMESTAMP_PATTERN.matcher(version);
-            if (matcher.matches()) {
-                effectiveVersion = matcher.group(1) + "-SNAPSHOT";
-            }
-        }
-
-        status = effectiveVersion != null && effectiveVersion.endsWith("SNAPSHOT") ? "integration" : "release";
+        String effectiveVersion = MavenVersionUtils.toEffectiveVersion(version);
+        status = MavenVersionUtils.inferStatusFromEffectiveVersion(version);
         componentIdentifier = DefaultModuleComponentIdentifier.newId(group, module, effectiveVersion);
     }
 
     public void addDependency(PomDependencyData dep) {
-        String scopeString = dep.getScope();
-        if (scopeString == null || scopeString.length() == 0) {
-            scopeString = getDefaultScope(dep);
-        }
+        doAddDependency(dep, dep.isOptional(), false);
+    }
 
+    public void addOptionalDependency(PomDependencyMgt dep) {
+        doAddDependency(dep, true, true);
+    }
+
+    private void doAddDependency(PomDependencyMgt dep, boolean optional, boolean useCompileScope) {
         MavenScope scope;
-        if (SCOPES.containsKey(scopeString)) {
-            scope = SCOPES.get(scopeString);
-        } else {
-            // unknown scope, defaulting to 'compile'
+        if (useCompileScope) {
             scope = MavenScope.Compile;
+        } else {
+            String scopeString = dep.getScope();
+            if (scopeString == null || scopeString.length() == 0) {
+                scopeString = getDefaultScope(dep);
+            }
+
+            if (SCOPES.containsKey(scopeString)) {
+                scope = SCOPES.get(scopeString);
+            } else {
+                // unknown scope, defaulting to 'compile'
+                scope = MavenScope.Compile;
+            }
         }
 
         String version = determineVersion(dep);
         String mappedVersion = convertVersionFromMavenSyntax(version);
-        ModuleVersionSelector selector = DefaultModuleVersionSelector.newSelector(dep.getGroupId(), dep.getArtifactId(), mappedVersion);
+        ModuleComponentSelector selector = DefaultModuleComponentSelector.newSelector(dep.getGroupId(), dep.getArtifactId(), new DefaultImmutableVersionConstraint(mappedVersion));
 
         // Some POMs depend on themselves, don't add this dependency: Ivy doesn't allow this!
         // Example: http://repo2.maven.org/maven2/net/jini/jsk-platform/2.1/jsk-platform-2.1.pom
         if (selector.getGroup().equals(componentIdentifier.getGroup())
-            && selector.getName().equals(componentIdentifier.getModule())) {
+            && selector.getModule().equals(componentIdentifier.getModule())) {
             return;
         }
 
-        boolean optional = dep.isOptional();
-
-        List<Artifact> artifacts = Lists.newArrayList();
+        IvyArtifactName dependencyArtifact = null;
         boolean hasClassifier = dep.getClassifier() != null && dep.getClassifier().length() > 0;
         boolean hasNonJarType = dep.getType() != null && !"jar".equals(dep.getType());
         if (hasClassifier || hasNonJarType) {
@@ -155,32 +147,24 @@ public class GradlePomModuleDescriptorBuilder {
             String ext = determineExtension(type);
             String classifier = hasClassifier ? dep.getClassifier() : getClassifierForType(type);
 
-            // here we have to assume a type and ext for the artifact, so this is a limitation
-            // compared to how m2 behave with classifiers
-            String optionalizedScope = optional ? "optional" : scope.toString().toLowerCase();
-
-            IvyArtifactName artifactName = new DefaultIvyArtifactName(selector.getName(), type, ext, classifier);
-            artifacts.add(new Artifact(artifactName, Collections.singleton(optionalizedScope)));
+            dependencyArtifact = new DefaultIvyArtifactName(selector.getModule(), type, ext, classifier);
         }
 
         // experimentation shows the following, excluded modules are
         // inherited from parent POMs if either of the following is true:
         // the <exclusions> element is missing or the <exclusions> element
         // is present, but empty.
-        List<Exclude> excludes = Lists.newArrayList();
+        List<ExcludeMetadata> excludes = Lists.newArrayList();
         List<ModuleIdentifier> excluded = dep.getExcludedModules();
         if (excluded.isEmpty()) {
             excluded = getDependencyMgtExclusions(dep);
         }
         for (ModuleIdentifier excludedModule : excluded) {
-            DefaultExclude rule = new DefaultExclude(
-                moduleIdentifierFactory.module(excludedModule.getGroup(), excludedModule.getName()),
-                WILDCARD,
-                PatternMatchers.EXACT);
+            DefaultExclude rule = new DefaultExclude(excludedModule);
             excludes.add(rule);
         }
 
-        dependencies.add(new MavenDependencyMetadata(scope, optional, selector, artifacts, excludes));
+        dependencies.add(new MavenDependencyDescriptor(scope, optional, selector, dependencyArtifact, excludes));
     }
 
     private String convertVersionFromMavenSyntax(String version) {
@@ -245,45 +229,32 @@ public class GradlePomModuleDescriptorBuilder {
 
     /**
      * Determines the version of a dependency. Uses the specified version if declared for the as coordinate. If the version is not declared, try to resolve it from the dependency management section.
-     * In case the version cannot be resolved with any of these methods, throw an exception of type {@see org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.UnresolvedDependencyVersionException}.
+     * In case the version cannot be resolved with any of these methods:
+     * - If this is a direct dependency: throw an exception of type {@see org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.UnresolvedDependencyVersionException}.
+     * - If this is an optional dependency: return the empty version
      *
      * @param dependency Dependency
      * @return Resolved dependency version
      */
-    private String determineVersion(PomDependencyData dependency) {
+    private String determineVersion(PomDependencyMgt dependency) {
         String version = dependency.getVersion();
         version = (version == null || version.length() == 0) ? getDefaultVersion(dependency) : version;
-
-        if (version == null) {
-            throw new UnresolvedDependencyVersionException(dependency.getId());
-        }
-
-        return version;
+        return version == null ? "" : version;
     }
 
-    public void addDependencyForRelocation(ModuleVersionSelector selector) {
-
+    public void addDependencyForRelocation(ModuleComponentSelector selector) {
         // Some POMs depend on themselves through their parent POM, don't add this dependency
         // since Ivy doesn't allow this!
         // Example: http://repo2.maven.org/maven2/com/atomikos/atomikos-util/3.6.4/atomikos-util-3.6.4.pom
         if (selector.getGroup().equals(componentIdentifier.getGroup())
-            && selector.getName().equals(componentIdentifier.getModule())) {
+            && selector.getModule().equals(componentIdentifier.getModule())) {
             return;
         }
 
-        // TODO - this is a constant
-        ListMultimap<String, String> confMappings = ArrayListMultimap.create();
-        // Map dependency on all public configurations
-        for (Configuration m2Conf : GradlePomModuleDescriptorBuilder.MAVEN2_CONFIGURATIONS.values()) {
-            if (m2Conf.isVisible()) {
-                confMappings.put(m2Conf.getName(), m2Conf.getName());
-            }
-        }
-
-        dependencies.add(new IvyDependencyMetadata(selector, confMappings));
+        dependencies.add(new MavenDependencyDescriptor(MavenScope.Runtime, false, selector, null, ImmutableList.<ExcludeMetadata>of()));
     }
 
-    private String getDefaultVersion(PomDependencyData dep) {
+    private String getDefaultVersion(PomDependencyMgt dep) {
         PomDependencyMgt pomDependencyMgt = findDependencyDefault(dep);
         if (pomDependencyMgt != null) {
             return pomDependencyMgt.getVersion();
@@ -291,7 +262,7 @@ public class GradlePomModuleDescriptorBuilder {
         return null;
     }
 
-    private String getDefaultScope(PomDependencyData dep) {
+    private String getDefaultScope(PomDependencyMgt dep) {
         PomDependencyMgt pomDependencyMgt = findDependencyDefault(dep);
         String result = null;
         if (pomDependencyMgt != null) {
@@ -303,7 +274,7 @@ public class GradlePomModuleDescriptorBuilder {
         return result;
     }
 
-    private List<ModuleIdentifier> getDependencyMgtExclusions(PomDependencyData dep) {
+    private List<ModuleIdentifier> getDependencyMgtExclusions(PomDependencyMgt dep) {
         PomDependencyMgt pomDependencyMgt = findDependencyDefault(dep);
         if (pomDependencyMgt != null) {
             return pomDependencyMgt.getExcludedModules();
@@ -312,7 +283,7 @@ public class GradlePomModuleDescriptorBuilder {
         return Collections.emptyList();
     }
 
-    private PomDependencyMgt findDependencyDefault(PomDependencyData dependency) {
+    private PomDependencyMgt findDependencyDefault(PomDependencyMgt dependency) {
         return pomReader.findDependencyDefaults(dependency.getId());
     }
 }
