@@ -23,6 +23,7 @@ import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
 import org.gradle.api.attributes.AttributeContainer;
+import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ModuleComponentSelectorSerializer;
 import org.gradle.api.internal.artifacts.ivyservice.NamespaceId;
@@ -126,6 +127,7 @@ public class ModuleMetadataSerializer {
                 writeVariantDependencies(variant.getDependencies());
                 writeVariantConstraints(variant.getDependencyConstraints());
                 writeVariantFiles(variant.getFiles());
+                writeVariantCapabilities(variant.getCapabilities().getCapabilities());
             }
         }
 
@@ -166,6 +168,16 @@ public class ModuleMetadataSerializer {
             }
         }
 
+        private void writeVariantCapabilities(List<? extends Capability> capabilities) throws IOException {
+            encoder.writeSmallInt(capabilities.size());
+            for (Capability capability : capabilities) {
+                encoder.writeString(capability.getGroup());
+                encoder.writeString(capability.getName());
+                encoder.writeString(capability.getVersion());
+            }
+        }
+
+
         private void write(IvyModuleResolveMetadata metadata) throws IOException {
             encoder.writeByte(TYPE_IVY);
             writeInfoSection(metadata);
@@ -192,7 +204,7 @@ public class ModuleMetadataSerializer {
         }
 
         private void writeInfoSection(ModuleComponentResolveMetadata metadata) throws IOException {
-            writeId(metadata.getComponentId());
+            writeId(metadata.getId());
             writeAttributes(metadata.getAttributes());
         }
 
@@ -413,6 +425,7 @@ public class ModuleMetadataSerializer {
                 readVariantDependencies(variant);
                 readVariantConstraints(variant);
                 readVariantFiles(variant);
+                readVariantCapabilities(variant);
             }
         }
 
@@ -457,6 +470,13 @@ public class ModuleMetadataSerializer {
             }
         }
 
+        private void readVariantCapabilities(MutableComponentVariant variant) throws IOException {
+            int count = decoder.readSmallInt();
+            for (int i = 0; i < count; i++) {
+                variant.addCapability(decoder.readString(), decoder.readString(), decoder.readString());
+            }
+        }
+
         private MutableModuleComponentResolveMetadata readIvy() throws IOException {
             readInfoSection();
             Map<NamespaceId, String> extraAttributes = readExtraInfo();
