@@ -20,13 +20,12 @@ import org.gradle.api.artifacts.ArtifactIdentifier;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.ResolvedModuleVersion;
-import org.gradle.api.artifacts.cache.ArtifactResolutionControl;
-import org.gradle.api.artifacts.cache.DependencyResolutionControl;
-import org.gradle.api.artifacts.cache.ModuleResolutionControl;
-import org.gradle.api.artifacts.cache.ResolutionControl;
-import org.gradle.api.artifacts.cache.ResolutionRules;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
+import org.gradle.api.internal.artifacts.cache.ArtifactResolutionControl;
+import org.gradle.api.internal.artifacts.cache.DependencyResolutionControl;
+import org.gradle.api.internal.artifacts.cache.ModuleResolutionControl;
+import org.gradle.api.internal.artifacts.cache.ResolutionControl;
 import org.gradle.api.internal.artifacts.configurations.MutationValidator;
 import org.gradle.api.internal.artifacts.configurations.dynamicversion.CachePolicy;
 import org.gradle.api.internal.artifacts.ivyservice.modulecache.dynamicversions.DefaultResolvedModuleVersion;
@@ -39,7 +38,7 @@ import java.util.concurrent.TimeUnit;
 
 import static org.gradle.api.internal.artifacts.configurations.MutationValidator.MutationType.STRATEGY;
 
-public class DefaultCachePolicy implements CachePolicy, ResolutionRules {
+public class DefaultCachePolicy implements CachePolicy {
     private static final int SECONDS_IN_DAY = 24 * 60 * 60;
 
     final List<Action<? super DependencyResolutionControl>> dependencyCacheRules;
@@ -73,25 +72,50 @@ public class DefaultCachePolicy implements CachePolicy, ResolutionRules {
         this.mutationValidator = validator;
     }
 
-    public void eachDependency(Action<? super DependencyResolutionControl> rule) {
-        mutationValidator.validateMutation(STRATEGY);
-        dependencyCacheRules.add(0, rule);
+    @Override
+    public void setOffline() {
+        eachDependency(new Action<DependencyResolutionControl>() {
+            public void execute(DependencyResolutionControl dependencyResolutionControl) {
+                dependencyResolutionControl.useCachedResult();
+            }
+        });
+        eachModule(new Action<ModuleResolutionControl>() {
+            public void execute(ModuleResolutionControl moduleResolutionControl) {
+                moduleResolutionControl.useCachedResult();
+            }
+        });
+        eachArtifact(new Action<ArtifactResolutionControl>() {
+            public void execute(ArtifactResolutionControl artifactResolutionControl) {
+                artifactResolutionControl.useCachedResult();
+            }
+        });
     }
 
-    public void eachModule(Action<? super ModuleResolutionControl> rule) {
-        mutationValidator.validateMutation(STRATEGY);
-        moduleCacheRules.add(0, rule);
-    }
-
-    public void eachArtifact(Action<? super ArtifactResolutionControl> rule) {
-        mutationValidator.validateMutation(STRATEGY);
-        artifactCacheRules.add(0, rule);
+    @Override
+    public void setRefreshDependencies() {
+        eachDependency(new Action<DependencyResolutionControl>() {
+            public void execute(DependencyResolutionControl dependencyResolutionControl) {
+                dependencyResolutionControl.cacheFor(0, TimeUnit.SECONDS);
+            }
+        });
+        eachModule(new Action<ModuleResolutionControl>() {
+            public void execute(ModuleResolutionControl moduleResolutionControl) {
+                moduleResolutionControl.cacheFor(0, TimeUnit.SECONDS);
+            }
+        });
+        eachArtifact(new Action<ArtifactResolutionControl>() {
+            public void execute(ArtifactResolutionControl artifactResolutionControl) {
+                artifactResolutionControl.cacheFor(0, TimeUnit.SECONDS);
+            }
+        });
     }
 
     public void cacheDynamicVersionsFor(final int value, final TimeUnit unit) {
         eachDependency(new Action<DependencyResolutionControl>() {
             public void execute(DependencyResolutionControl dependencyResolutionControl) {
-                dependencyResolutionControl.cacheFor(value, unit);
+                if (!dependencyResolutionControl.getCachedResult().isEmpty()) {
+                    dependencyResolutionControl.cacheFor(value, unit);
+                }
             }
         });
     }
@@ -123,6 +147,33 @@ public class DefaultCachePolicy implements CachePolicy, ResolutionRules {
         });
     }
 
+    /**
+     * Apply a rule to control resolution of dependencies.
+     * @param rule the rule to apply
+     */
+    private void eachDependency(Action<? super DependencyResolutionControl> rule) {
+        mutationValidator.validateMutation(STRATEGY);
+        dependencyCacheRules.add(0, rule);
+    }
+
+    /**
+     * Apply a rule to control resolution of modules.
+     * @param rule the rule to apply
+     */
+    private void eachModule(Action<? super ModuleResolutionControl> rule) {
+        mutationValidator.validateMutation(STRATEGY);
+        moduleCacheRules.add(0, rule);
+    }
+
+    /**
+     * Apply a rule to control resolution of artifacts.
+     * @param rule the rule to apply
+     */
+    private void eachArtifact(Action<? super ArtifactResolutionControl> rule) {
+        mutationValidator.validateMutation(STRATEGY);
+        artifactCacheRules.add(0, rule);
+    }
+
     public boolean mustRefreshVersionList(final ModuleIdentifier moduleIdentifier, Set<ModuleVersionIdentifier> matchingVersions, long ageMillis) {
         CachedDependencyResolutionControl dependencyResolutionControl = new CachedDependencyResolutionControl(moduleIdentifier, matchingVersions, ageMillis);
 
