@@ -16,8 +16,8 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine;
 
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
-import org.gradle.api.artifacts.result.ComponentSelectionReason;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.StringVersioned;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionDescriptorInternal;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
 
 import javax.annotation.Nullable;
@@ -31,9 +31,12 @@ public interface ComponentResolutionState extends StringVersioned {
      * @return null if the meta-data is not available due to some failure.
      */
     @Nullable
-    ComponentResolveMetadata getMetaData();
+    ComponentResolveMetadata getMetadata();
 
-    ComponentSelectionReason getSelectionReason();
+    void addCause(ComponentSelectionDescriptorInternal componentSelectionDescription);
+
+    void reject();
+
+    boolean isRejected();
 
-    void setSelectionReason(ComponentSelectionReason componentSelectionReason);
 }
