@@ -17,25 +17,25 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder;
 
 import com.google.common.collect.Lists;
+import org.gradle.api.Action;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionDescriptorInternal;
 import org.gradle.api.attributes.Attribute;
 import org.gradle.api.attributes.AttributeContainer;
-import org.gradle.api.internal.artifacts.ResolvedVersionConstraint;
+import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.ComponentResolutionState;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.ComponentResult;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphComponent;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionDescriptorInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasonInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.VersionSelectionReasons;
 import org.gradle.api.internal.attributes.AttributeContainerInternal;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.internal.Cast;
+import org.gradle.internal.component.external.model.ImmutableCapability;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
 import org.gradle.internal.component.model.DefaultComponentOverrideMetadata;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
 import org.gradle.internal.resolve.resolver.ComponentMetaDataResolver;
-import org.gradle.internal.resolve.result.ComponentIdResolveResult;
 import org.gradle.internal.resolve.result.DefaultBuildableComponentResolveResult;
 
 import java.util.List;
@@ -44,26 +44,35 @@ import java.util.Set;
 /**
  * Resolution state for a given component
  */
-public class ComponentState implements ComponentResolutionState, ComponentResult, DependencyGraphComponent, ComponentStateWithDependents<ComponentState> {
+public class ComponentState implements ComponentResolutionState, DependencyGraphComponent {
+    private final ComponentIdentifier componentIdentifier;
     private final ModuleVersionIdentifier id;
     private final ComponentMetaDataResolver resolver;
+    private final VariantNameBuilder variantNameBuilder;
     private final List<NodeState> nodes = Lists.newLinkedList();
     private final Long resultId;
     private final ModuleResolveState module;
     private final ComponentSelectionReasonInternal selectionReason = VersionSelectionReasons.empty();
-    private volatile ComponentResolveMetadata metaData;
+    private final ImmutableCapability implicitCapability;
+    private volatile ComponentResolveMetadata metadata;
 
-    private ModuleState state = ModuleState.Selectable;
+    private ComponentSelectionState state = ComponentSelectionState.Selectable;
     private ModuleVersionResolveException failure;
-    private SelectorState selectedBy;
+    // The first selector that resolved this component
+    private SelectorState firstSelectedBy;
+    private List<SelectorState> selectedBy;
     private DependencyGraphBuilder.VisitState visitState = DependencyGraphBuilder.VisitState.NotSeen;
-    List<SelectorState> allResolvers;
 
-    ComponentState(Long resultId, ModuleResolveState module, ModuleVersionIdentifier id, ComponentMetaDataResolver resolver) {
+    private boolean rejected;
+
+    ComponentState(Long resultId, ModuleResolveState module, ModuleVersionIdentifier id, ComponentIdentifier componentIdentifier, ComponentMetaDataResolver resolver, VariantNameBuilder variantNameBuilder) {
         this.resultId = resultId;
         this.module = module;
         this.id = id;
+        this.componentIdentifier = componentIdentifier;
         this.resolver = resolver;
+        this.variantNameBuilder = variantNameBuilder;
+        this.implicitCapability = new ImmutableCapability(id.getGroup(), id.getName(), id.getVersion());
     }
 
     @Override
@@ -113,7 +122,17 @@ public class ComponentState implements ComponentResolutionState, ComponentResult
 
     @Override
     public ComponentResolveMetadata getMetadata() {
-        return metaData;
+        resolve();
+        return metadata;
+    }
+
+    @Override
+    public ComponentIdentifier getComponentId() {
+        // Use the resolved component id if available: this ensures that Maven Snapshot ids are correctly reported
+        if (metadata != null) {
+            return metadata.getId();
+        }
+        return componentIdentifier;
     }
 
     public void restart(ComponentState selected) {
@@ -123,11 +142,15 @@ public class ComponentState implements ComponentResolutionState, ComponentResult
     }
 
     public void selectedBy(SelectorState resolver) {
-        if (selectedBy == null) {
-            selectedBy = resolver;
-            allResolvers = Lists.newLinkedList();
+        if (firstSelectedBy == null) {
+            firstSelectedBy = resolver;
+            selectedBy = Lists.newLinkedList();
         }
-        allResolvers.add(resolver);
+        selectedBy.add(resolver);
+    }
+
+    public List<SelectorState> getSelectedBy() {
+        return selectedBy;
     }
 
     /**
@@ -135,58 +158,26 @@ public class ComponentState implements ComponentResolutionState, ComponentResult
      *
      * @return true if it has been resolved in a cheap way
      */
-    public boolean fastResolve() {
-        if (metaData != null || failure != null) {
-            return true;
-        }
-
-        ComponentIdResolveResult idResolveResult = selectedBy.getResolveResult();
-        if (idResolveResult.getFailure() != null) {
-            failure = idResolveResult.getFailure();
-            return true;
-        }
-        if (idResolveResult.getMetaData() != null) {
-            metaData = idResolveResult.getMetaData();
-            return true;
-        }
-        return false;
+    public boolean alreadyResolved() {
+        return metadata != null || failure != null;
     }
 
     public void resolve() {
-        if (fastResolve()) {
+        if (alreadyResolved()) {
             return;
         }
 
-        ComponentIdResolveResult idResolveResult = selectedBy.getResolveResult();
-
         DefaultBuildableComponentResolveResult result = new DefaultBuildableComponentResolveResult();
-        resolver.resolve(idResolveResult.getId(), DefaultComponentOverrideMetadata.forDependency(selectedBy.getDependencyMetadata()), result);
+        resolver.resolve(componentIdentifier, DefaultComponentOverrideMetadata.forDependency(firstSelectedBy.getDependencyMetadata()), result);
         if (result.getFailure() != null) {
             failure = result.getFailure();
             return;
         }
-        metaData = result.getMetaData();
+        metadata = result.getMetadata();
     }
 
-    @Override
-    public ComponentResolveMetadata getMetaData() {
-        if (metaData == null) {
-            resolve();
-        }
-        return metaData;
-    }
-
-    public ResolvedVersionConstraint getVersionConstraint() {
-        return selectedBy == null ? null : selectedBy.getVersionConstraint();
-    }
-
-    @Override
-    public boolean isResolved() {
-        return metaData != null;
-    }
-
-    public void setMetaData(ComponentResolveMetadata metaData) {
-        this.metaData = metaData;
+    public void setMetadata(ComponentResolveMetadata metaData) {
+        this.metadata = metaData;
         this.failure = null;
     }
 
@@ -210,16 +201,25 @@ public class ComponentState implements ComponentResolutionState, ComponentResult
     }
 
     @Override
-    public ComponentIdentifier getComponentId() {
-        return getMetaData().getComponentId();
+    public String getVariantName() {
+        String name = null;
+        for (NodeState node : nodes) {
+            if (node.isSelected()) {
+                name = variantNameBuilder.getVariantName(name, node.getMetadata().getName());
+            }
+        }
+        return name == null ? "unknown" : name;
     }
 
     @Override
-    public String getVariantName() {
+    public AttributeContainer getVariantAttributes() {
         NodeState selected = getSelectedNode();
-        return selected == null ? "unknown" : selected.getMetadata().getName();
+        return selected == null ? ImmutableAttributes.EMPTY : desugarAttributes(selected);
     }
 
+    /**
+     * Returns the _first_ selected node. There may be multiple.
+     */
     private NodeState getSelectedNode() {
         for (NodeState node : nodes) {
             if (node.isSelected()) {
@@ -229,12 +229,6 @@ public class ComponentState implements ComponentResolutionState, ComponentResult
         return null;
     }
 
-    @Override
-    public AttributeContainer getVariantAttributes() {
-        NodeState selected = getSelectedNode();
-        return selected == null ? ImmutableAttributes.EMPTY : desugarAttributes(selected);
-    }
-
     /**
      * Desugars attributes so that what we're going to serialize consists only of String or Boolean attributes,
      * and not their original types.
@@ -272,37 +266,117 @@ public class ComponentState implements ComponentResolutionState, ComponentResult
         return incoming;
     }
 
-    public List<ComponentState> getUnattachedDependencies() {
-        return module.getUnattachedEdgesTo(this);
-    }
-
-    @Override
-    public boolean isFromPendingNode() {
-        return selectedBy != null && selectedBy.getDependencyMetadata().isPending();
-    }
-
-    boolean isSelected() {
-        return state == ModuleState.Selected;
+    public boolean isSelected() {
+        return state == ComponentSelectionState.Selected;
     }
 
     boolean isSelectable() {
         return state.isSelectable();
     }
 
-    boolean isCandidateForConflictResolution() {
+    public boolean isCandidateForConflictResolution() {
         return state.isCandidateForConflictResolution();
     }
 
     void evict() {
-        state = ModuleState.Evicted;
+        state = ComponentSelectionState.Evicted;
     }
 
     void select() {
-        state = ModuleState.Selected;
+        state = ComponentSelectionState.Selected;
     }
 
     void makeSelectable() {
-        state = ModuleState.Selectable;
+        state = ComponentSelectionState.Selectable;
+    }
+
+    @Override
+    public void reject() {
+        this.rejected = true;
+
+    }
+
+    @Override
+    public boolean isRejected() {
+        return rejected;
+    }
+
+
+    /**
+     * Describes the possible states of a component in the graph.
+     */
+    enum ComponentSelectionState {
+        /**
+         * A selectable component is either new to the graph, or has been visited before,
+         * but wasn't selected because another compatible version was used.
+         */
+        Selectable(true, true),
+
+        /**
+         * A selected component has been chosen, at some point, as the version to use.
+         * This is not for a lifetime: a component can later be evicted through conflict resolution,
+         * or another compatible component can be chosen instead if more constraints arise.
+         */
+        Selected(true, false),
+
+        /**
+         * An evicted component has been evicted and will never, ever be chosen starting from the moment it is evicted.
+         * Either because it has been excluded, or because conflict resolution selected a different version.
+         */
+        Evicted(false, false);
+
+        private final boolean candidateForConflictResolution;
+        private final boolean canSelect;
+
+        ComponentSelectionState(boolean candidateForConflictResolution, boolean canSelect) {
+            this.candidateForConflictResolution = candidateForConflictResolution;
+            this.canSelect = canSelect;
+        }
+
+        boolean isCandidateForConflictResolution() {
+            return candidateForConflictResolution;
+        }
+
+        public boolean isSelectable() {
+            return canSelect;
+        }
+    }
+
+
+    public void forEachCapability(Action<? super Capability> action) {
+        // check conflict for each target node
+        for (NodeState target : nodes) {
+            List<? extends Capability> capabilities = target.getMetadata().getCapabilities().getCapabilities();
+            // The isEmpty check is not required, might look innocent, but Guava's performance bad for an empty immutable list
+            // because it still creates an inner class for an iterator, which delegates to an Array iterator, which does... nothing.
+            // so just adding this check has a significant impact because most components do not declare any capability
+            if (!capabilities.isEmpty()) {
+                for (Capability capability : capabilities) {
+                    action.execute(capability);
+                }
+            }
+        }
+    }
+
+    public Capability findCapability(String group, String name) {
+        if (id.getGroup().equals(group) && id.getName().equals(name)) {
+            return implicitCapability;
+        }
+        return findCapabilityOnTarget(group, name);
+    }
+
+    private Capability findCapabilityOnTarget(String group, String name) {
+        for (NodeState target : nodes) {
+            List<? extends Capability> capabilities = target.getMetadata().getCapabilities().getCapabilities();
+            if (!capabilities.isEmpty()) { // Not required, but Guava's performance bad for an empty immutable list
+                for (Capability capability : capabilities) {
+                    if (capability.getGroup().equals(group) && capability.getName().equals(name)) {
+                        return capability;
+                    }
+                }
+            }
+        }
+        return null;
     }
 
 }
