@@ -24,8 +24,6 @@ import org.gradle.api.artifacts.component.ComponentSelector;
 import org.gradle.api.artifacts.result.ComponentSelectionReason;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusion;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphEdge;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphNode;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphSelector;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.internal.component.local.model.DslOriginDependencyMetadata;
 import org.gradle.internal.component.model.ComponentArtifactMetadata;
@@ -43,6 +41,7 @@ import java.util.List;
  * Represents the edges in the dependency graph.
  */
 class EdgeState implements DependencyGraphEdge {
+    private final DependencyState dependencyState;
     private final DependencyMetadata dependencyMetadata;
     private final NodeState from;
     private final SelectorState selector;
@@ -55,11 +54,12 @@ class EdgeState implements DependencyGraphEdge {
 
     EdgeState(NodeState from, DependencyState dependencyState, ModuleExclusion transitiveExclusions, ResolveState resolveState) {
         this.from = from;
-        this.dependencyMetadata = dependencyState.getDependencyMetadata();
+        this.dependencyState = dependencyState;
+        this.dependencyMetadata = dependencyState.getDependency();
         // The accumulated exclusions that apply to this edge based on the path from the root
         this.transitiveExclusions = transitiveExclusions;
         this.resolveState = resolveState;
-        this.selector = resolveState.getSelector(dependencyMetadata, dependencyState.getModuleIdentifier());
+        this.selector = resolveState.getSelector(dependencyState, dependencyState.getModuleIdentifier());
     }
 
     @Override
@@ -81,21 +81,10 @@ class EdgeState implements DependencyGraphEdge {
     }
 
     @Override
-    public DependencyGraphSelector getSelector() {
+    public SelectorState getSelector() {
         return selector;
     }
 
-    /**
-     * @return The resolved module version
-     */
-    public ComponentState resolveModuleRevisionId() {
-        if (targetModuleRevision == null) {
-            targetModuleRevision = selector.resolveModuleRevisionId();
-            selector.getSelectedModule().addUnattachedDependency(this);
-        }
-        return targetModuleRevision;
-    }
-
     public boolean isTransitive() {
         return from.isTransitive() && dependencyMetadata.isTransitive();
     }
@@ -109,7 +98,7 @@ class EdgeState implements DependencyGraphEdge {
             targetConfiguration.addIncomingEdge(this);
         }
         if (!targetNodes.isEmpty()) {
-            selector.getSelectedModule().removeUnattachedDependency(this);
+            selector.getTargetModule().removeUnattachedDependency(this);
         }
     }
 
@@ -120,7 +109,7 @@ class EdgeState implements DependencyGraphEdge {
         targetNodes.clear();
         targetNodeSelectionFailure = null;
         if (targetModuleRevision != null) {
-            selector.getSelectedModule().removeUnattachedDependency(this);
+            selector.getTargetModule().removeUnattachedDependency(this);
         }
     }
 
@@ -130,10 +119,14 @@ class EdgeState implements DependencyGraphEdge {
         attachToTargetConfigurations();
     }
 
+    public void start(ComponentState selected) {
+        targetModuleRevision = selected;
+    }
+
     private void calculateTargetConfigurations() {
         targetNodes.clear();
         targetNodeSelectionFailure = null;
-        ComponentResolveMetadata targetModuleVersion = targetModuleRevision.getMetaData();
+        ComponentResolveMetadata targetModuleVersion = targetModuleRevision.getMetadata();
         if (targetModuleVersion == null) {
             // Broken version
             return;
@@ -144,8 +137,8 @@ class EdgeState implements DependencyGraphEdge {
         try {
             targetConfigurations = dependencyMetadata.selectConfigurations(attributes, targetModuleVersion, resolveState.getAttributesSchema());
         } catch (Throwable t) {
-//                 Broken selector
-            targetNodeSelectionFailure = new ModuleVersionResolveException(dependencyMetadata.getSelector(), t);
+            // Failure to select the target variant/configurations from this component, given the dependency attributes/metadata.
+            targetNodeSelectionFailure = new ModuleVersionResolveException(dependencyState.getRequested(), t);
             return;
         }
         for (ConfigurationMetadata targetConfiguration : targetConfigurations) {
@@ -164,9 +157,14 @@ class EdgeState implements DependencyGraphEdge {
         return resolveState.getModuleExclusions().intersect(edgeExclusions, transitiveExclusions);
     }
 
+    @Override
+    public boolean contributesArtifacts() {
+        return !dependencyMetadata.isPending();
+    }
+
     @Override
     public ComponentSelector getRequested() {
-        return dependencyMetadata.getSelector();
+        return dependencyState.getRequested();
     }
 
     @Override
@@ -195,11 +193,6 @@ class EdgeState implements DependencyGraphEdge {
         return null;
     }
 
-    @Override
-    public Iterable<? extends DependencyGraphNode> getTargets() {
-        return targetNodes;
-    }
-
     @Override
     public List<ComponentArtifactMetadata> getArtifacts(final ConfigurationMetadata targetConfiguration) {
         return CollectionUtils.collect(dependencyMetadata.getArtifacts(), new Transformer<ComponentArtifactMetadata, IvyArtifactName>() {
@@ -209,4 +202,5 @@ class EdgeState implements DependencyGraphEdge {
             }
         });
     }
+
 }
