@@ -19,6 +19,7 @@ package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder
 import com.google.common.collect.Lists;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
+import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflicts.CandidateModule;
 import org.gradle.internal.id.IdGenerator;
 import org.gradle.internal.resolve.resolver.ComponentMetaDataResolver;
@@ -41,12 +42,14 @@ class ModuleResolveState implements CandidateModule {
     private final List<EdgeState> unattachedDependencies = new LinkedList<EdgeState>();
     private final Map<ModuleVersionIdentifier, ComponentState> versions = new LinkedHashMap<ModuleVersionIdentifier, ComponentState>();
     private final List<SelectorState> selectors = Lists.newLinkedList();
+    private final VariantNameBuilder variantNameBuilder;
     private ComponentState selected;
 
-    ModuleResolveState(IdGenerator<Long> idGenerator, ModuleIdentifier id, ComponentMetaDataResolver metaDataResolver) {
+    ModuleResolveState(IdGenerator<Long> idGenerator, ModuleIdentifier id, ComponentMetaDataResolver metaDataResolver, VariantNameBuilder variantNameBuilder) {
         this.idGenerator = idGenerator;
         this.id = id;
         this.metaDataResolver = metaDataResolver;
+        this.variantNameBuilder = variantNameBuilder;
     }
 
     @Override
@@ -98,6 +101,7 @@ class ModuleResolveState implements CandidateModule {
         for (ComponentState version : versions.values()) {
             version.evict();
         }
+
         selected.select();
     }
 
@@ -119,22 +123,12 @@ class ModuleResolveState implements CandidateModule {
         }
     }
 
-    public void softSelect(ComponentState selected) {
-        assert this.selected == null;
-        this.selected = selected;
-        for (ComponentState version : versions.values()) {
-            version.makeSelectable();
-        }
-        selected.select();
-        doRestart(selected);
-    }
-
     private void doRestart(ComponentState selected) {
         for (ComponentState version : versions.values()) {
             version.restart(selected);
         }
         for (SelectorState selector : selectors) {
-            selector.restart(selected);
+            selector.overrideSelection(selected);
         }
         if (!unattachedDependencies.isEmpty()) {
             restartUnattachedDependencies(selected);
@@ -160,10 +154,10 @@ class ModuleResolveState implements CandidateModule {
         unattachedDependencies.remove(edge);
     }
 
-    public ComponentState getVersion(ModuleVersionIdentifier id) {
+    public ComponentState getVersion(ModuleVersionIdentifier id, ComponentIdentifier componentIdentifier) {
         ComponentState moduleRevision = versions.get(id);
         if (moduleRevision == null) {
-            moduleRevision = new ComponentState(idGenerator.generateId(), this, id, metaDataResolver);
+            moduleRevision = new ComponentState(idGenerator.generateId(), this, id, componentIdentifier, metaDataResolver, variantNameBuilder);
             versions.put(id, moduleRevision);
         }
         return moduleRevision;
