@@ -50,7 +50,7 @@ public class ComponentResultSerializer implements Serializer<ComponentResult> {
         ComponentIdentifier componentId = componentIdSerializer.read(decoder);
         String variantName = decoder.readString();
         AttributeContainer attributes = attributeContainerSerializer.read(decoder);
-        return new DefaultComponentResult(resultId, id, reason, componentId, variantName, attributes);
+        return new DetachedComponentResult(resultId, id, reason, componentId, variantName, attributes);
     }
 
     public void write(Encoder encoder, ComponentResult value) throws IOException {
