@@ -27,7 +27,7 @@ import org.gradle.api.artifacts.repositories.IvyArtifactRepositoryMetaDataProvid
 import org.gradle.api.artifacts.repositories.RepositoryLayout;
 import org.gradle.api.artifacts.repositories.RepositoryResourceAccessor;
 import org.gradle.api.internal.DefaultActionConfiguration;
-import org.gradle.api.internal.ExperimentalFeatures;
+import org.gradle.api.internal.FeaturePreviews;
 import org.gradle.api.internal.InstantiatorFactory;
 import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ModuleVersionPublisher;
@@ -73,6 +73,8 @@ import java.net.URI;
 import java.util.LinkedHashSet;
 import java.util.Set;
 
+import static org.gradle.api.internal.FeaturePreviews.Feature.GRADLE_METADATA;
+
 public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupportedRepository implements IvyArtifactRepository, ResolutionAwareRepository, PublicationAwareRepository {
     private final static Factory<ComponentMetadataSupplier> NO_METADATA_SUPPLIER = new Factory<ComponentMetadataSupplier>() {
         @Override
@@ -112,7 +114,7 @@ public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupporte
                                         InstantiatorFactory instantiatorFactory,
                                         FileResourceRepository fileResourceRepository,
                                         ModuleMetadataParser moduleMetadataParser,
-                                        ExperimentalFeatures experimentalFeatures,
+                                        FeaturePreviews featurePreviews,
                                         IvyMutableModuleMetadataFactory metadataFactory) {
         super(instantiatorFactory.decorate(), authenticationContainer);
         this.fileResolver = fileResolver;
@@ -130,7 +132,7 @@ public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupporte
         this.metaDataProvider = new MetaDataProvider();
         this.instantiator = instantiatorFactory.decorate();
         this.ivyContextManager = ivyContextManager;
-        this.metadataSources.setDefaults(experimentalFeatures);
+        this.metadataSources.setDefaults(featurePreviews);
     }
 
     @Override
@@ -346,9 +348,9 @@ public class DefaultIvyArtifactRepository extends AbstractAuthenticationSupporte
         boolean ivyDescriptor;
         boolean artifact;
 
-        void setDefaults(ExperimentalFeatures experimentalFeatures) {
+        void setDefaults(FeaturePreviews featurePreviews) {
             ivyDescriptor();
-            if (experimentalFeatures.isEnabled()) {
+            if (featurePreviews.isFeatureEnabled(GRADLE_METADATA)) {
                 gradleMetadata();
             } else {
                 artifact();
