@@ -164,7 +164,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
     private void doListModuleVersions(ModuleDependencyMetadata dependency, BuildableModuleVersionListingResolveResult result) {
         ModuleIdentifier module = moduleIdentifierFactory.module(dependency.getSelector().getGroup(), dependency.getSelector().getModule());
 
-        // TODO:DAZ Provide an abstraction for accessing resources within the same module (maven-metadata, directory listing, etc)
+        // TODO: Provide an abstraction for accessing resources within the same module (maven-metadata, directory listing, etc)
         // That way we can avoid passing `ivyPatterns` and `artifactPatterns` around everywhere
         ResourceVersionLister versionLister = new ResourceVersionLister(repository);
         List<ResourcePattern> completeIvyPatterns = filterComplete(this.ivyPatterns, module);
@@ -369,7 +369,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
 
         @Override
         protected final void resolveMetaDataArtifacts(T module, BuildableArtifactSetResolveResult result) {
-            ModuleDescriptorArtifactMetadata artifact = getMetaDataArtifactFor(module.getComponentId());
+            ModuleDescriptorArtifactMetadata artifact = getMetaDataArtifactFor(module.getId());
             result.resolved(Collections.singleton(artifact));
         }
 
@@ -414,7 +414,7 @@ public abstract class ExternalResourceResolver<T extends ModuleComponentResolveM
 
         private void checkArtifactsResolved(ComponentResolveMetadata component, Object context, BuildableTypedResolveResult<?, ? super ArtifactResolveException> result) {
             if (!result.hasResult()) {
-                result.failed(new ArtifactResolveException(component.getComponentId(),
+                result.failed(new ArtifactResolveException(component.getId(),
                     String.format("Cannot locate %s for '%s' in repository '%s'", context, component, name)));
             }
         }
