@@ -20,16 +20,15 @@ import com.google.common.collect.Lists;
 import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.PublishArtifact;
 import org.gradle.api.artifacts.VersionConstraint;
+import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
-import org.gradle.api.internal.artifacts.configurations.OutgoingVariant;
 import org.gradle.api.internal.artifacts.dependencies.DefaultImmutableVersionConstraint;
-import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.internal.Pair;
 import org.gradle.internal.component.external.descriptor.Configuration;
 import org.gradle.internal.component.external.model.DefaultModuleComponentSelector;
 import org.gradle.internal.component.external.model.ModuleComponentArtifactIdentifier;
-import org.gradle.internal.component.local.model.BuildableLocalComponentMetadata;
+import org.gradle.internal.component.local.model.BuildableLocalConfigurationMetadata;
 import org.gradle.internal.component.local.model.LocalFileDependencyMetadata;
 import org.gradle.internal.component.model.DefaultIvyArtifactName;
 import org.gradle.internal.component.model.ExcludeMetadata;
@@ -46,7 +45,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-public class DefaultIvyModulePublishMetadata implements BuildableIvyModulePublishMetadata, BuildableLocalComponentMetadata {
+public class DefaultIvyModulePublishMetadata implements IvyModulePublishMetadata {
     private static final Transformer<String, String> VERSION_TRANSFORMER = new IvyVersionTransformer();
     private final ModuleComponentIdentifier id;
     private final String status;
@@ -60,11 +59,6 @@ public class DefaultIvyModulePublishMetadata implements BuildableIvyModulePublis
         this.status = status;
     }
 
-    public DefaultIvyModulePublishMetadata(IvyModulePublishMetadata metadata) {
-        this.id = metadata.getComponentId();
-        this.status = metadata.getStatus();
-    }
-
     public ModuleComponentIdentifier getComponentId() {
         return id;
     }
@@ -89,22 +83,12 @@ public class DefaultIvyModulePublishMetadata implements BuildableIvyModulePublis
         return excludes;
     }
 
-    @Override
-    public void addConfiguration(String name, String description, Set<String> extendsFrom, Set<String> hierarchy, boolean visible, boolean transitive, ImmutableAttributes attributes, boolean canBeConsumed, boolean canBeResolved) {
+    public BuildableLocalConfigurationMetadata addConfiguration(String name, Set<String> extendsFrom, boolean visible, boolean transitive) {
         List<String> sortedExtends = Lists.newArrayList(extendsFrom);
         Collections.sort(sortedExtends);
         Configuration configuration = new Configuration(name, transitive, visible, sortedExtends);
         configurations.put(name, configuration);
-    }
-
-    @Override
-    public void addExclude(String configuration, ExcludeMetadata exclude) {
-        excludes.add(Pair.of(exclude, configuration));
-    }
-
-    @Override
-    public void addDependency(LocalOriginDependencyMetadata dependency) {
-        dependencies.add(normalizeVersionForIvy(dependency));
+        return new ConfigurationMetadata(name);
     }
 
     /**
@@ -124,21 +108,6 @@ public class DefaultIvyModulePublishMetadata implements BuildableIvyModulePublis
         return dependency;
     }
 
-    @Override
-    public void addArtifacts(String configuration, Iterable<? extends PublishArtifact> artifacts) {
-        for (PublishArtifact artifact : artifacts) {
-            DefaultIvyArtifactName ivyName = DefaultIvyArtifactName.forPublishArtifact(artifact);
-            DefaultIvyModuleArtifactPublishMetadata ivyArtifact = getOrCreate(ivyName);
-            ivyArtifact.setFile(artifact.getFile());
-            ivyArtifact.addConfiguration(configuration);
-        }
-    }
-
-    @Override
-    public void addVariant(String configuration, OutgoingVariant variant) {
-        // Ignore
-    }
-
     public void addArtifact(IvyArtifactName artifact, File file) {
         DefaultIvyModuleArtifactPublishMetadata publishMetadata = new DefaultIvyModuleArtifactPublishMetadata(id, artifact);
         publishMetadata.setFile(file);
@@ -164,9 +133,40 @@ public class DefaultIvyModulePublishMetadata implements BuildableIvyModulePublis
         return artifactsById.values();
     }
 
-    @Override
-    public void addFiles(String configuration, LocalFileDependencyMetadata files) {
-        // Ignore
+    public void addArtifact(String configuration, PublishArtifact artifact) {
+        DefaultIvyArtifactName ivyName = DefaultIvyArtifactName.forPublishArtifact(artifact);
+        DefaultIvyModuleArtifactPublishMetadata ivyArtifact = getOrCreate(ivyName);
+        ivyArtifact.setFile(artifact.getFile());
+        ivyArtifact.addConfiguration(configuration);
+    }
+
+    private class ConfigurationMetadata implements BuildableLocalConfigurationMetadata {
+        private final String configurationName;
+
+        private ConfigurationMetadata(String configurationName) {
+            this.configurationName = configurationName;
+        }
+
+        @Override
+        public ComponentIdentifier getComponentId() {
+            return id;
+        }
+
+        @Override
+        public void addDependency(LocalOriginDependencyMetadata dependency) {
+            assert dependency.getModuleConfiguration().equals(configurationName);
+            dependencies.add(normalizeVersionForIvy(dependency));
+        }
+
+        @Override
+        public void addExclude(ExcludeMetadata exclude) {
+            excludes.add(Pair.of(exclude, configurationName));
+        }
+
+        @Override
+        public void addFiles(LocalFileDependencyMetadata files) {
+            // Ignore files
+        }
     }
 
     private static class IvyVersionTransformer implements Transformer<String, String> {
