@@ -17,31 +17,60 @@
 package org.gradle.ide.visualstudio.internal;
 
 import org.gradle.api.Action;
+import org.gradle.api.Transformer;
+import org.gradle.api.file.ProjectLayout;
+import org.gradle.api.file.RegularFile;
 import org.gradle.api.internal.tasks.DefaultTaskDependency;
+import org.gradle.api.model.ObjectFactory;
+import org.gradle.api.provider.Provider;
+import org.gradle.api.provider.ProviderFactory;
+import org.gradle.api.tasks.Input;
+import org.gradle.api.tasks.Internal;
+import org.gradle.api.tasks.Nested;
 import org.gradle.api.tasks.TaskDependency;
 import org.gradle.ide.visualstudio.TextConfigFile;
 import org.gradle.ide.visualstudio.TextProvider;
-import org.gradle.internal.component.local.model.LocalComponentArtifactMetadata;
 import org.gradle.internal.file.PathToFileResolver;
-import org.gradle.internal.reflect.Instantiator;
 import org.gradle.plugins.ide.internal.IdeArtifactRegistry;
+import org.gradle.util.CollectionUtils;
 
+import javax.inject.Inject;
 import java.io.File;
 import java.util.ArrayList;
+import java.util.LinkedHashSet;
 import java.util.List;
+import java.util.Set;
+import java.util.concurrent.Callable;
 
 public class DefaultVisualStudioSolution implements VisualStudioSolutionInternal {
     private final String name;
     private final SolutionFile solutionFile;
     private final DefaultTaskDependency buildDependencies = new DefaultTaskDependency();
     private final IdeArtifactRegistry ideArtifactRegistry;
+    private final Provider<RegularFile> location;
 
-    public DefaultVisualStudioSolution(String name, PathToFileResolver fileResolver, Instantiator instantiator, IdeArtifactRegistry ideArtifactRegistry) {
+    @Inject
+    public DefaultVisualStudioSolution(String name, ObjectFactory objectFactory, IdeArtifactRegistry ideArtifactRegistry, ProviderFactory providers, ProjectLayout projectLayout) {
         this.name = name;
-        this.solutionFile = instantiator.newInstance(SolutionFile.class, fileResolver, getName() + ".sln");
+        this.solutionFile = objectFactory.newInstance(SolutionFile.class, getName() + ".sln");
+        this.location = projectLayout.file(providers.provider(new Callable<File>() {
+            @Override
+            public File call() {
+                return solutionFile.getLocation();
+            }
+        }));
         this.ideArtifactRegistry = ideArtifactRegistry;
-        builtBy(ideArtifactRegistry.getIdeArtifacts(VisualStudioProjectInternal.ARTIFACT_TYPE));
-        builtBy(ideArtifactRegistry.getIdeArtifacts(VisualStudioProjectConfiguration.ARTIFACT_TYPE));
+        builtBy(ideArtifactRegistry.getIdeArtifacts(VisualStudioProjectMetadata.class));
+    }
+
+    @Override
+    public String getDisplayName() {
+        return "Visual Studio solution";
+    }
+
+    @Override
+    public Provider<RegularFile> getLocation() {
+        return location;
     }
 
     @Override
@@ -49,18 +78,44 @@ public class DefaultVisualStudioSolution implements VisualStudioSolutionInternal
         return name;
     }
 
+    @Override
     public SolutionFile getSolutionFile() {
         return solutionFile;
     }
 
     @Override
-    public List<LocalComponentArtifactMetadata> getProjectArtifacts() {
-        return ideArtifactRegistry.getIdeArtifactMetadata(VisualStudioProjectInternal.ARTIFACT_TYPE);
+    @Internal
+    public List<VisualStudioProjectMetadata> getProjects() {
+        return CollectionUtils.collect(ideArtifactRegistry.getIdeArtifactMetadata(VisualStudioProjectMetadata.class), new Transformer<VisualStudioProjectMetadata, IdeArtifactRegistry.Reference<VisualStudioProjectMetadata>>() {
+            @Override
+            public VisualStudioProjectMetadata transform(IdeArtifactRegistry.Reference<VisualStudioProjectMetadata> reference) {
+                return reference.get();
+            }
+        });
     }
 
-    @Override
-    public List<LocalComponentArtifactMetadata> getProjectConfigurationArtifacts() {
-        return ideArtifactRegistry.getIdeArtifactMetadata(VisualStudioProjectConfiguration.ARTIFACT_TYPE);
+    @Input
+    public List<String> getProjectFilePaths() {
+        return CollectionUtils.collect(getProjects(), new Transformer<String, VisualStudioProjectMetadata>() {
+            @Override
+            public String transform(VisualStudioProjectMetadata metadata) {
+                return metadata.getFile().getAbsolutePath();
+            }
+        });
+    }
+
+    @Input
+    public Set<String> getConfigurationNames() {
+        Set<String> configurations = new LinkedHashSet<String>();
+        for (VisualStudioProjectMetadata projectMetadata : getProjects()) {
+            configurations.addAll(projectMetadata.getConfigurations());
+        }
+        return configurations;
+    }
+
+    @Nested
+    public List<Action<? super TextProvider>> getSolutionFileActions() {
+        return solutionFile.getTextActions();
     }
 
     @Override
@@ -78,11 +133,13 @@ public class DefaultVisualStudioSolution implements VisualStudioSolutionInternal
         private final PathToFileResolver fileResolver;
         private Object location;
 
+        @Inject
         public SolutionFile(PathToFileResolver fileResolver, String defaultLocation) {
             this.fileResolver = fileResolver;
             this.location = defaultLocation;
         }
 
+        @Internal
         public File getLocation() {
             return fileResolver.resolve(location);
         }
@@ -95,6 +152,7 @@ public class DefaultVisualStudioSolution implements VisualStudioSolutionInternal
             actions.add(action);
         }
 
+        @Nested
         public List<Action<? super TextProvider>> getTextActions() {
             return actions;
         }
