@@ -23,6 +23,7 @@ import org.gradle.plugins.ide.eclipse.model.Container;
 import org.gradle.plugins.ide.eclipse.model.EclipseClasspath;
 import org.gradle.plugins.ide.eclipse.model.Output;
 import org.gradle.plugins.ide.eclipse.model.SourceFolder;
+import org.gradle.plugins.ide.internal.IdeArtifactRegistry;
 
 import java.util.Collections;
 import java.util.List;
@@ -32,9 +33,9 @@ public class ClasspathFactory {
     private final EclipseClasspath classpath;
     private final EclipseDependenciesCreator dependenciesCreator;
 
-    public ClasspathFactory(EclipseClasspath classpath) {
+    public ClasspathFactory(EclipseClasspath classpath, IdeArtifactRegistry ideArtifactRegistry) {
         this.classpath = classpath;
-        this.dependenciesCreator = new EclipseDependenciesCreator(classpath);
+        this.dependenciesCreator = new EclipseDependenciesCreator(classpath, ideArtifactRegistry);
     }
 
     public List<ClasspathEntry> createEntries() {
