@@ -29,6 +29,7 @@ import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.language.scala.ScalaPlatform;
 import org.gradle.plugins.ide.idea.model.internal.IdeaDependenciesProvider;
+import org.gradle.plugins.ide.internal.IdeArtifactRegistry;
 
 import java.io.File;
 import java.util.Collection;
@@ -69,6 +70,12 @@ import static org.gradle.util.ConfigureUtil.configure;
  *     //and some extra test source dirs
  *     testSourceDirs += file('some-extra-test-dir')
  *
+ *     //and some extra resource dirs
+ *     resourceDirs += file('some-extra-resource-dir')
+ *
+ *     //and some extra test resource dirs
+ *     testResourceDirs += file('some-extra-test-resource-dir')
+ *
  *     //and hint to mark some of existing source dirs as generated sources
  *     generatedSourceDirs += file('some-extra-source-folder')
  *
@@ -149,6 +156,8 @@ public class IdeaModule {
     private String name;
     private Set<File> sourceDirs;
     private Set<File> generatedSourceDirs = Sets.newLinkedHashSet();
+    private Set<File> resourceDirs = Sets.newLinkedHashSet();
+    private Set<File> testResourceDirs = Sets.newLinkedHashSet();
     private Map<String, Map<String, Collection<Configuration>>> scopes = Maps.newLinkedHashMap();
     private boolean downloadSources = true;
     private boolean downloadJavadoc;
@@ -210,6 +219,9 @@ public class IdeaModule {
     /**
      * The directories containing the production sources.
      * <p>
+     * For backward-compatibility it is set of directories containing the production sources from <tt>project.sourceSets.main.allSource</tt>.
+     * <p>
+     * Starting with Gradle 5.0 it is set of directories containing the production sources from <tt>project.sourceSets.main.allJava</tt>.
      * For example see docs for {@link IdeaModule}
      */
     public Set<File> getSourceDirs() {
@@ -302,8 +314,14 @@ public class IdeaModule {
         this.contentRoot = contentRoot;
     }
 
+
     /**
-     * The directories containing the test sources. <p> For example see docs for {@link IdeaModule}
+     * The directories containing the test sources.
+     * <p>
+     * For backward-compatibility it is set of directories containing the test sources from <tt>project.sourceSets.test.allSource</tt>.
+     * <p>
+     * Starting with Gradle 5.0 it is set of directories containing the test sources from <tt>project.sourceSets.test.allJava</tt>.
+     * For example see docs for {@link IdeaModule}
      */
     public Set<File> getTestSourceDirs() {
         return testSourceDirs;
@@ -313,6 +331,41 @@ public class IdeaModule {
         this.testSourceDirs = testSourceDirs;
     }
 
+    /**
+     * The directories containing resources. <p> For example see docs for {@link IdeaModule}
+     * @since 4.7
+     */
+    @Incubating
+    public Set<File> getResourceDirs() {
+        return resourceDirs;
+    }
+
+    /**
+     * Sets the directories containing resources. <p> For example see docs for {@link IdeaModule}
+     * @since 4.7
+     */
+    @Incubating
+    public void setResourceDirs(Set<File> resourceDirs) {
+        this.resourceDirs = resourceDirs;
+    }
+
+    /**
+     * The directories containing the test resources. <p> For example see docs for {@link IdeaModule}
+     * @since 4.7
+     */
+    @Incubating
+    public Set<File> getTestResourceDirs() {
+        return testResourceDirs;
+    }
+
+    /**
+     * Sets the directories containing the test resources. <p> For example see docs for {@link IdeaModule}
+     * @since 4.7
+     */
+    @Incubating
+    public void setTestResourceDirs(Set<File> testResourceDirs) {
+        this.testResourceDirs = testResourceDirs;
+    }
     /**
      * Directories to be excluded. <p> For example see docs for {@link IdeaModule}
      */
@@ -529,7 +582,7 @@ public class IdeaModule {
      */
     public Set<Dependency> resolveDependencies() {
         ProjectInternal projectInternal = (ProjectInternal) project;
-        IdeaDependenciesProvider ideaDependenciesProvider = new IdeaDependenciesProvider(projectInternal.getServices());
+        IdeaDependenciesProvider ideaDependenciesProvider = new IdeaDependenciesProvider(projectInternal.getServices().get(IdeArtifactRegistry.class));
         return ideaDependenciesProvider.provide(this);
     }
 
@@ -540,6 +593,8 @@ public class IdeaModule {
         Set<Path> sourceFolders = pathsOf(existing(getSourceDirs()));
         Set<Path> generatedSourceFolders = pathsOf(existing(getGeneratedSourceDirs()));
         Set<Path> testSourceFolders = pathsOf(existing(getTestSourceDirs()));
+        Set<Path> resourceFolders = pathsOf(existing(getResourceDirs()));
+        Set<Path> testResourceFolders = pathsOf(existing(getTestResourceDirs()));
         Set<Path> excludeFolders = pathsOf(getExcludeDirs());
         Path outputDir = getOutputDir() != null ? getPathFactory().path(getOutputDir()) : null;
         Path testOutputDir = getTestOutputDir() != null ? getPathFactory().path(getTestOutputDir()) : null;
@@ -548,7 +603,10 @@ public class IdeaModule {
 
         xmlModule.configure(
             contentRoot,
-            sourceFolders, testSourceFolders, generatedSourceFolders, excludeFolders,
+            sourceFolders, testSourceFolders,
+            resourceFolders, testResourceFolders,
+            generatedSourceFolders,
+            excludeFolders,
             getInheritOutputDirs(), outputDir, testOutputDir,
             dependencies,
             getJdkName(), level
@@ -574,4 +632,5 @@ public class IdeaModule {
             }
         }));
     }
+
 }
