@@ -44,12 +44,11 @@ import org.gradle.internal.logging.LoggingManagerInternal;
 import org.gradle.internal.logging.services.DefaultLoggingManagerFactory;
 import org.gradle.internal.logging.services.LoggingServiceRegistry;
 import org.gradle.internal.logging.sink.ConsoleStateUtil;
-import org.gradle.internal.logging.sink.OutputEventRenderer;
 import org.gradle.internal.nativeintegration.services.NativeServices;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.ServiceRegistryBuilder;
 import org.gradle.internal.service.scopes.GlobalScopeServices;
-import org.gradle.internal.time.Clock;
+import org.gradle.launcher.cli.CommandLineActionFactory;
 import org.gradle.launcher.daemon.configuration.DaemonBuildOptions;
 import org.gradle.process.internal.streams.SafeStreams;
 import org.gradle.test.fixtures.file.TestDirectoryProvider;
@@ -62,7 +61,6 @@ import org.gradle.util.GradleVersion;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.OutputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 import java.nio.charset.Charset;
@@ -77,7 +75,10 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 
-import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.*;
+import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.DAEMON;
+import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.FOREGROUND;
+import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.NOT_DEFINED;
+import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.NO_DAEMON;
 import static org.gradle.integtests.fixtures.executer.OutputScrapingExecutionResult.STACK_TRACE_ELEMENT;
 import static org.gradle.internal.service.scopes.DefaultGradleUserHomeScopeServiceRegistry.REUSE_USER_HOME_SERVICES;
 import static org.gradle.util.CollectionUtils.collect;
@@ -156,6 +157,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private ConsoleOutput consoleType;
     protected WarningMode warningMode = WarningMode.All;
     private boolean showStacktrace = true;
+    private boolean renderWelcomeMessage;
 
     private int expectedDeprecationWarnings;
     private boolean eagerClassLoaderCreationChecksOn = true;
@@ -228,6 +230,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         useOnlyRequestedJvmOpts = false;
         expectedDeprecationWarnings = 0;
         stackTraceChecksOn = true;
+        renderWelcomeMessage = false;
         debug = Boolean.getBoolean(DEBUG_SYSPROP);
         debugLauncher = Boolean.getBoolean(LAUNCHER_DEBUG_SYSPROP);
         profiler = System.getProperty(PROFILE_SYSPROP, "");
@@ -391,6 +394,11 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (!showStacktrace) {
             executer.withStacktraceDisabled();
         }
+
+        if (renderWelcomeMessage) {
+            executer.withWelcomeMessageEnabled();
+        }
+
         return executer;
     }
 
@@ -757,6 +765,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
+    @Override
+    public GradleExecuter withWelcomeMessageEnabled() {
+        renderWelcomeMessage = true;
+        return this;
+    }
+
     /**
      * Performs cleanup at completion of the test.
      */
@@ -945,6 +959,8 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
             }
         }
 
+        properties.put(CommandLineActionFactory.WELCOME_MESSAGE_ENABLED_SYSTEM_PROPERTY, Boolean.toString(renderWelcomeMessage));
+
         return properties;
     }
 
@@ -1113,7 +1129,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                 boolean executionFailure = isExecutionFailure(executionResult);
 
                 // for tests using rich console standard out and error are combined in output of execution result
-                if (executionFailure && isErrorOutEmpty(error)) {
+                if (executionFailure) {
                     normalizedOutput = removeExceptionStackTraceForFailedExecution(normalizedOutput);
                 }
 
@@ -1355,49 +1371,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     }
 
     private static LoggingServiceRegistry newCommandLineProcessLogging() {
-        LoggingServiceRegistry loggingServices = new LoggingServiceRegistry() {
-            @Override
-            protected OutputEventRenderer createOutputEventRenderer(Clock clock) {
-                return new VerboseAwareOutputEventRenderer(clock);
-            }
-        };
+        LoggingServiceRegistry loggingServices = LoggingServiceRegistry.newEmbeddableLogging();
         LoggingManagerInternal rootLoggingManager = loggingServices.get(DefaultLoggingManagerFactory.class).getRoot();
-        rootLoggingManager.captureSystemSources();
+//        rootLoggingManager.captureSystemSources();
         rootLoggingManager.attachSystemOutAndErr();
         return loggingServices;
     }
-
-    private static class VerboseAwareOutputEventRenderer extends OutputEventRenderer {
-        VerboseAwareOutputEventRenderer(Clock clock) {
-            super(clock);
-        }
-
-        @Override
-        public void attachAnsiConsole(OutputStream outputStream) {
-            if (outputStream instanceof VerboseAwareAnsiOutputStream) {
-                attachAnsiConsole(outputStream, VerboseAwareAnsiOutputStream.class.cast(outputStream).isVerbose());
-            } else {
-                super.attachAnsiConsole(outputStream);
-            }
-        }
-    }
-
-    protected static class VerboseAwareAnsiOutputStream extends OutputStream {
-        private final OutputStream delegate;
-        private boolean verbose;
-
-        VerboseAwareAnsiOutputStream(OutputStream delegate, boolean verbose) {
-            this.delegate = delegate;
-            this.verbose = verbose;
-        }
-
-        public boolean isVerbose() {
-            return verbose;
-        }
-
-        @Override
-        public void write(int b) throws IOException {
-            delegate.write(b);
-        }
-    }
 }
