@@ -145,7 +145,9 @@ public class DefaultGradleDistribution implements GradleDistribution {
     }
 
     public VersionNumber getArtifactCacheLayoutVersion() {
-        if (isSameOrNewer("4.6-rc-1")) {
+        if (isSameOrNewer("4.7-rc-1")) {
+            return VersionNumber.parse("2.56");
+        } else if (isSameOrNewer("4.6-rc-1")) {
             return VersionNumber.parse("2.53");
         } else if (isSameOrNewer("4.5.1-rc-1")) {
             return VersionNumber.parse("2.51");
