@@ -16,6 +16,8 @@
 
 package org.gradle.integtests.fixtures.executer;
 
+import junit.framework.AssertionFailedError;
+import org.apache.commons.io.output.TeeOutputStream;
 import org.gradle.BuildResult;
 import org.gradle.StartParameter;
 import org.gradle.api.GradleException;
@@ -26,7 +28,6 @@ import org.gradle.api.execution.TaskExecutionListener;
 import org.gradle.api.internal.StartParameterInternal;
 import org.gradle.api.internal.classpath.ModuleRegistry;
 import org.gradle.api.internal.file.TestFiles;
-import org.gradle.api.logging.StandardOutputListener;
 import org.gradle.api.logging.configuration.ConsoleOutput;
 import org.gradle.api.tasks.TaskState;
 import org.gradle.cli.CommandLineParser;
@@ -46,15 +47,13 @@ import org.gradle.internal.classpath.ClassPath;
 import org.gradle.internal.event.ListenerManager;
 import org.gradle.internal.exceptions.LocationAwareException;
 import org.gradle.internal.invocation.BuildAction;
-import org.gradle.internal.io.LineBufferingOutputStream;
-import org.gradle.internal.io.TextStream;
 import org.gradle.internal.jvm.Jvm;
 import org.gradle.internal.logging.LoggingManagerInternal;
 import org.gradle.internal.nativeintegration.ProcessEnvironment;
 import org.gradle.launcher.Main;
-import org.gradle.launcher.cli.ExecuteBuildAction;
 import org.gradle.launcher.cli.Parameters;
 import org.gradle.launcher.cli.ParametersConverter;
+import org.gradle.launcher.cli.action.ExecuteBuildAction;
 import org.gradle.launcher.exec.BuildActionExecuter;
 import org.gradle.launcher.exec.BuildActionParameters;
 import org.gradle.launcher.exec.DefaultBuildActionParameters;
@@ -68,10 +67,10 @@ import org.gradle.util.SetSystemProperties;
 import org.hamcrest.Matcher;
 import org.hamcrest.Matchers;
 
-import javax.annotation.Nullable;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.InputStream;
-import java.io.StringWriter;
+import java.io.OutputStream;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -83,14 +82,28 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
+import java.util.TreeSet;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CopyOnWriteArraySet;
 import java.util.regex.Pattern;
 
 import static org.gradle.integtests.fixtures.executer.OutputScrapingExecutionResult.flattenTaskPaths;
-import static org.gradle.util.Matchers.*;
-import static org.hamcrest.Matchers.*;
-import static org.junit.Assert.*;
+import static org.gradle.util.Matchers.hasMessage;
+import static org.gradle.util.Matchers.isEmpty;
+import static org.gradle.util.Matchers.normalizedLineSeparators;
+import static org.hamcrest.Matchers.containsInAnyOrder;
+import static org.hamcrest.Matchers.containsString;
+import static org.hamcrest.Matchers.equalTo;
+import static org.hamcrest.Matchers.hasItem;
+import static org.hamcrest.Matchers.hasItems;
+import static org.hamcrest.Matchers.not;
+import static org.hamcrest.Matchers.nullValue;
+import static org.hamcrest.Matchers.startsWith;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertThat;
+import static org.junit.Assert.assertTrue;
 
 public class InProcessGradleExecuter extends AbstractGradleExecuter {
     private final ProcessEnvironment processEnvironment = GLOBAL_SERVICES.get(ProcessEnvironment.class);
@@ -122,17 +135,17 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             return createGradleHandle().waitForFinish();
         }
 
-        StandardOutputListener outputListener = new OutputListenerImpl();
-        StandardOutputListener errorListener = new OutputListenerImpl();
+        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
+        ByteArrayOutputStream errorStream = new ByteArrayOutputStream();
         BuildListenerImpl buildListener = new BuildListenerImpl();
-        BuildResult result = doRun(outputListener, errorListener, buildListener);
+        BuildResult result = doRun(outputStream, errorStream, buildListener);
         try {
             result.rethrowFailure();
         } catch (Exception e) {
             throw new UnexpectedBuildFailure(e);
         }
         return assertResult(new InProcessExecutionResult(buildListener.executedTasks, buildListener.skippedTasks,
-            new OutputScrapingExecutionResult(outputListener.toString(), errorListener.toString())));
+            OutputScrapingExecutionResult.from(outputStream.toString(), errorStream.toString())));
     }
 
     @Override
@@ -141,15 +154,15 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             return createGradleHandle().waitForFailure();
         }
 
-        StandardOutputListener outputListener = new OutputListenerImpl();
-        StandardOutputListener errorListener = new OutputListenerImpl();
+        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
+        ByteArrayOutputStream errorStream = new ByteArrayOutputStream();
         BuildListenerImpl buildListener = new BuildListenerImpl();
         try {
-            doRun(outputListener, errorListener, buildListener).rethrowFailure();
+            doRun(outputStream, errorStream, buildListener).rethrowFailure();
             throw new AssertionError("expected build to fail but it did not.");
         } catch (GradleException e) {
             return assertResult(new InProcessExecutionFailure(buildListener.executedTasks, buildListener.skippedTasks,
-                new OutputScrapingExecutionFailure(outputListener.toString(), errorListener.toString()), e));
+                OutputScrapingExecutionFailure.from(outputStream.toString(), errorStream.toString()), e));
         }
     }
 
@@ -211,7 +224,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         return result;
     }
 
-    private BuildResult doRun(StandardOutputListener outputListener, StandardOutputListener errorListener, BuildListenerImpl listener) {
+    private BuildResult doRun(OutputStream outputStream, OutputStream errorStream, BuildListenerImpl listener) {
         // Capture the current state of things that we will change during execution
         InputStream originalStdIn = System.in;
         Properties originalSysProperties = new Properties();
@@ -223,7 +236,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         Set<String> changedEnvVars = new HashSet<String>(invocation.environmentVars.keySet());
 
         try {
-            return executeBuild(invocation, outputListener, errorListener, listener);
+            return executeBuild(invocation, outputStream, errorStream, listener);
         } finally {
             // Restore the environment
             System.setProperties(originalSysProperties);
@@ -246,32 +259,22 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         SetSystemProperties.resetTempDirLocation();
     }
 
-    private LoggingManagerInternal createLoggingManager(StartParameter startParameter, final StandardOutputListener outputListener, final StandardOutputListener errorListener) {
-        LoggingManagerInternal loggingManager =
-            GLOBAL_SERVICES.getFactory(LoggingManagerInternal.class).create();
+    private LoggingManagerInternal createLoggingManager(StartParameter startParameter, OutputStream outputStream, OutputStream errorStream) {
+        LoggingManagerInternal loggingManager = GLOBAL_SERVICES.getFactory(LoggingManagerInternal.class).create();
+        loggingManager.captureSystemSources();
 
-        if (startParameter.getConsoleOutput() == ConsoleOutput.Rich || startParameter.getConsoleOutput() == ConsoleOutput.Verbose) {
-            // Explicitly enabling rich console forces everything into the stdout listener
-            loggingManager.attachAnsiConsole(new VerboseAwareAnsiOutputStream(new LineBufferingOutputStream(new TextStream() {
-                @Override
-                public void text(String text) {
-                    outputListener.onOutput(text);
-                }
-
-                @Override
-                public void endOfStream(@Nullable Throwable failure) {
-
-                }
-            }), startParameter.getConsoleOutput() == ConsoleOutput.Verbose));
-        } else {
-            loggingManager.addStandardOutputListener(outputListener);
-            loggingManager.addStandardErrorListener(errorListener);
+        ConsoleOutput consoleOutput = startParameter.getConsoleOutput();
+        if (consoleOutput == ConsoleOutput.Auto) {
+            // IDEA runs tests attached to a console, use plain so test can assume never attached to a console
+            // Should really run all tests against a plain and a rich console to make these assumptions explicit
+            consoleOutput = ConsoleOutput.Plain;
         }
+        loggingManager.attachConsole(new TeeOutputStream(System.out, outputStream), new TeeOutputStream(System.err, errorStream), consoleOutput);
 
         return loggingManager;
     }
 
-    private BuildResult executeBuild(GradleInvocation invocation, final StandardOutputListener outputListener, final StandardOutputListener errorListener, BuildListenerImpl listener) {
+    private BuildResult executeBuild(GradleInvocation invocation, OutputStream outputStream, OutputStream errorStream, BuildListenerImpl listener) {
         // Augment the environment for the execution
         System.setIn(connectStdIn());
         processEnvironment.maybeSetProcessDir(getWorkingDir());
@@ -283,7 +286,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         resetTempDirLocation();
 
         // TODO: Fix tests that rely on this being set before we process arguments like this...
-        StartParameter startParameter = new StartParameterInternal();
+        StartParameterInternal startParameter = new StartParameterInternal();
         startParameter.setCurrentDir(getWorkingDir());
 
         // TODO: Reuse more of CommandlineActionFactory
@@ -305,7 +308,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             BuildActionParameters buildActionParameters = createBuildActionParameters(startParameter);
             BuildRequestContext buildRequestContext = createBuildRequestContext();
 
-            LoggingManagerInternal loggingManager = createLoggingManager(startParameter, outputListener, errorListener);
+            LoggingManagerInternal loggingManager = createLoggingManager(startParameter, outputStream, errorStream);
             loggingManager.start();
 
             try {
@@ -378,19 +381,6 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         }
     }
 
-    private static class OutputListenerImpl implements StandardOutputListener {
-        private StringWriter writer = new StringWriter();
-
-        @Override
-        public String toString() {
-            return writer.toString();
-        }
-
-        public void onOutput(CharSequence output) {
-            writer.append(output);
-        }
-    }
-
     private static class TaskListenerImpl implements TaskExecutionListener {
         private final List<Task> planned;
         private final List<String> executedTasks;
@@ -433,7 +423,6 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         private final List<String> plannedTasks;
         private final Set<String> skippedTasks;
         private final OutputScrapingExecutionResult outputResult;
-        private GroupedOutputFixture groupedOutputFixture;
 
         public InProcessExecutionResult(List<String> plannedTasks, Set<String> skippedTasks, OutputScrapingExecutionResult outputResult) {
             this.plannedTasks = plannedTasks;
@@ -452,10 +441,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
 
         @Override
         public GroupedOutputFixture getGroupedOutput() {
-            if (groupedOutputFixture == null) {
-                this.groupedOutputFixture = new GroupedOutputFixture(getOutput());
-            }
-            return groupedOutputFixture;
+            return outputResult.getGroupedOutput();
         }
 
         public ExecutionResult assertOutputEquals(String expectedOutput, boolean ignoreExtraLines, boolean ignoreLineOrder) {
@@ -463,12 +449,35 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             return this;
         }
 
+        @Override
+        public ExecutionResult assertNotOutput(String expectedOutput) {
+            outputResult.assertNotOutput(expectedOutput);
+            return this;
+        }
+
         @Override
         public ExecutionResult assertOutputContains(String expectedOutput) {
             outputResult.assertOutputContains(expectedOutput);
             return this;
         }
 
+        @Override
+        public ExecutionResult assertHasPostBuildOutput(String expectedOutput) {
+            outputResult.assertHasPostBuildOutput(expectedOutput);
+            return this;
+        }
+
+        @Override
+        public boolean hasErrorOutput(String expectedOutput) {
+            return outputResult.hasErrorOutput(expectedOutput);
+        }
+
+        @Override
+        public ExecutionResult assertHasErrorOutput(String expectedOutput) {
+            outputResult.assertHasErrorOutput(expectedOutput);
+            return this;
+        }
+
         public String getError() {
             return outputResult.getError();
         }
@@ -485,12 +494,26 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         }
 
         public ExecutionResult assertTasksExecuted(Object... taskPaths) {
-            List<String> flattenedTasks = flattenTaskPaths(taskPaths);
+            Set<String> flattenedTasks = new TreeSet<String>(flattenTaskPaths(taskPaths));
             assertThat(plannedTasks, containsInAnyOrder(flattenedTasks.toArray()));
             outputResult.assertTasksExecuted(flattenedTasks);
             return this;
         }
 
+        @Override
+        public ExecutionResult assertTaskExecuted(String taskPath) {
+            assertThat(plannedTasks, hasItem(taskPath));
+            outputResult.assertTaskExecuted(taskPath);
+            return this;
+        }
+
+        @Override
+        public ExecutionResult assertTaskNotExecuted(String taskPath) {
+            assertThat(plannedTasks, not(hasItem(taskPath)));
+            outputResult.assertTaskNotExecuted(taskPath);
+            return this;
+        }
+
         @Override
         public ExecutionResult assertTaskOrder(Object... taskPaths) {
             Set<String> expected = TaskOrderSpecs.exact(taskPaths).getTasks();
@@ -500,24 +523,18 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         }
 
         public Set<String> getSkippedTasks() {
-            return new HashSet<String>(skippedTasks);
+            return new TreeSet<String>(skippedTasks);
         }
 
         @Override
         public ExecutionResult assertTasksSkipped(Object... taskPaths) {
-            if (GradleContextualExecuter.isParallel()) {
-                return this;
-            }
-            Set<String> expected = new HashSet<String>(flattenTaskPaths(taskPaths));
+            Set<String> expected = new TreeSet<String>(flattenTaskPaths(taskPaths));
             assertThat(skippedTasks, equalTo(expected));
             outputResult.assertTasksSkipped(expected);
             return this;
         }
 
         public ExecutionResult assertTaskSkipped(String taskPath) {
-            if (GradleContextualExecuter.isParallel()) {
-                return this;
-            }
             assertThat(skippedTasks, hasItem(taskPath));
             outputResult.assertTaskSkipped(taskPath);
             return this;
@@ -525,10 +542,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
 
         @Override
         public ExecutionResult assertTasksNotSkipped(Object... taskPaths) {
-            if (GradleContextualExecuter.isParallel()) {
-                return this;
-            }
-            Set<String> expected = new HashSet<String>(flattenTaskPaths(taskPaths));
+            Set<String> expected = new TreeSet<String>(flattenTaskPaths(taskPaths));
             Set<String> notSkipped = getNotSkippedTasks();
             assertThat(notSkipped, equalTo(expected));
             outputResult.assertTasksNotSkipped(expected);
@@ -536,16 +550,13 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         }
 
         public ExecutionResult assertTaskNotSkipped(String taskPath) {
-            if (GradleContextualExecuter.isParallel()) {
-                return this;
-            }
             assertThat(getNotSkippedTasks(), hasItem(taskPath));
             outputResult.assertTaskNotSkipped(taskPath);
             return this;
         }
 
         private Set<String> getNotSkippedTasks() {
-            Set<String> notSkipped = new HashSet<String>(plannedTasks);
+            Set<String> notSkipped = new TreeSet<String>(plannedTasks);
             notSkipped.removeAll(skippedTasks);
             return notSkipped;
         }
@@ -559,8 +570,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         private final String lineNumber;
         private final String description;
 
-        public InProcessExecutionFailure(List<String> tasks, Set<String> skippedTasks, OutputScrapingExecutionFailure outputFailure,
-                                         GradleException failure) {
+        public InProcessExecutionFailure(List<String> tasks, Set<String> skippedTasks, OutputScrapingExecutionFailure outputFailure, GradleException failure) {
             super(tasks, skippedTasks, outputFailure);
             this.outputFailure = outputFailure;
             this.failure = failure;
@@ -603,7 +613,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         public ExecutionFailure assertThatCause(Matcher<String> matcher) {
             List<Throwable> causes = new ArrayList<Throwable>();
             extractCauses(failure, causes);
-            assertThat(causes, Matchers.<Throwable>hasItem(hasMessage(normalizedLineSeparators(matcher))));
+            assertThat(causes, Matchers.hasItem(hasMessage(normalizedLineSeparators(matcher))));
             outputFailure.assertThatCause(matcher);
             return this;
         }
@@ -621,6 +631,20 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             }
         }
 
+        @Override
+        public ExecutionFailure assertHasNoCause(String description) {
+            Matcher<Throwable> matcher = hasMessage(containsString(description));
+            List<Throwable> causes = new ArrayList<Throwable>();
+            extractCauses(failure, causes);
+            for (Throwable cause : causes) {
+                if (matcher.matches(cause)) {
+                    throw new AssertionFailedError(String.format("Expected no failure with description '%s', found: %s", description, cause));
+                }
+            }
+            outputFailure.assertHasNoCause(description);
+            return this;
+        }
+
         public ExecutionFailure assertHasNoCause() {
             if (failure instanceof LocationAwareException) {
                 LocationAwareException exception = (LocationAwareException) failure;
