@@ -16,36 +16,33 @@
 
 package org.gradle.integtests.fixtures.logging;
 
-import org.apache.commons.io.IOUtils;
-import org.fusesource.jansi.AnsiOutputStream;
-import org.gradle.api.UncheckedIOException;
+import org.gradle.integtests.fixtures.executer.LogContent;
 
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
-import java.io.StringReader;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 /**
- * Parses rich console output into its pieces for verification in functional tests
+ * Parses console output into its pieces for verification in functional tests
  */
 public class GroupedOutputFixture {
     /**
      * All tasks will start with > Task, captures everything starting with : and going until a control char
      */
-    private final static String TASK_HEADER = "> Task (:[\\w:]*)\\n?";
+    private final static String TASK_HEADER = "> Task (:[\\w:]*) ?(FAILED|FROM-CACHE|UP-TO-DATE|SKIPPED|NO-SOURCE)?\\n?";
 
+    private final static String EMBEDDED_BUILD_START = "> :\\w* > root project";
     private final static String BUILD_STATUS_FOOTER = "BUILD SUCCESSFUL";
-    private final static String BUILD_FAILED_FOOTER = "FAILURE:";
+    private final static String BUILD_FAILED_FOOTER = "BUILD FAILED";
+    private final static String ACTIONABLE_TASKS = "[0-9]+ actionable tasks:";
 
     /**
      * Various patterns to detect the end of the task output
      */
-    private final static String END_OF_TASK_OUTPUT = TASK_HEADER + "|" + BUILD_STATUS_FOOTER + "|" + BUILD_FAILED_FOOTER;
+    private final static String END_OF_TASK_OUTPUT = TASK_HEADER + "|" + BUILD_STATUS_FOOTER + "|" + BUILD_FAILED_FOOTER + "|" + EMBEDDED_BUILD_START + "|" + ACTIONABLE_TASKS + "|\\z";
 
-    private final static String PROGRESS_BAR_PATTERN = "<[-=]*> \\d+% (INITIALIZ|CONFIGUR|EXECUT)ING \\[((\\d+h )? \\d+m )?\\d+s\\]";
+    private final static String PROGRESS_BAR_PATTERN = "<[-=(\u001b\\[\\d+[a-zA-Z;])]*> \\d+% (INITIALIZ|CONFIGUR|EXECUT)ING \\[((\\d+h )? \\d+m )?\\d+s\\]";
     private final static String WORK_IN_PROGRESS_PATTERN = "\u001b\\[\\d+[a-zA-Z]> (IDLE|[:a-z][\\w\\s\\d:>/\\\\\\.]+)\u001b\\[\\d*[a-zA-Z]";
     private final static String DOWN_MOVEMENT_WITH_NEW_LINE_PATTERN = "\u001b\\[\\d+B\\n";
 
@@ -57,14 +54,13 @@ public class GroupedOutputFixture {
     private static final Pattern TASK_OUTPUT_PATTERN;
 
     static {
-        String precompiledPattern = "(?ms)";
-        precompiledPattern += TASK_HEADER;
+        String pattern = "(?ms)";
+        pattern += TASK_HEADER;
         // Capture all output, lazily up until two new lines and an END_OF_TASK designation
-        precompiledPattern += "((.|\\n)*?(?=[^\\n]*?" + END_OF_TASK_OUTPUT + "))";
-        TASK_OUTPUT_PATTERN = Pattern.compile(precompiledPattern);
+        pattern += "([\\s\\S]*?(?=[^\\n]*?" + END_OF_TASK_OUTPUT + "))";
+        TASK_OUTPUT_PATTERN = Pattern.compile(pattern);
     }
 
-
     private final String originalOutput;
     private final String strippedOutput;
     private Map<String, GroupedTaskFixture> tasks;
@@ -77,19 +73,21 @@ public class GroupedOutputFixture {
     private String parse(String output) {
         tasks = new HashMap<String, GroupedTaskFixture>();
 
-        String strippedOutput = stripAnsiCodes(stripWorkInProgressArea(output));
+        String strippedOutput = LogContent.of(stripWorkInProgressArea(output)).removeAnsiChars().withNormalizedEol();
         Matcher matcher = TASK_OUTPUT_PATTERN.matcher(strippedOutput);
         while (matcher.find()) {
             String taskName = matcher.group(1);
-            String taskOutput = matcher.group(2);
-            taskOutput = taskOutput.trim();
-            if (tasks.containsKey(taskName)) {
-                tasks.get(taskName).addOutput(taskOutput);
-            } else {
-                GroupedTaskFixture task = new GroupedTaskFixture(taskName);
-                task.addOutput(taskOutput);
+            String taskOutcome = matcher.group(2);
+            String taskOutput = matcher.group(3).trim();
+
+            GroupedTaskFixture task = tasks.get(taskName);
+            if (task == null) {
+                task = new GroupedTaskFixture(taskName);
                 tasks.put(taskName, task);
             }
+
+            task.addOutput(taskOutput);
+            task.setOutcome(taskOutcome);
         }
 
         return strippedOutput;
@@ -107,16 +105,6 @@ public class GroupedOutputFixture {
         return "(\u001b\\[0K\\n){" + scroll + "}\u001b\\[" + scroll + "A";
     }
 
-    private String stripAnsiCodes(String output) {
-        try {
-            ByteArrayOutputStream baos = new ByteArrayOutputStream();
-            IOUtils.copy(new StringReader(output), new AnsiOutputStream(baos));
-            return baos.toString();
-        } catch (IOException e) {
-            throw new UncheckedIOException(e);
-        }
-    }
-
     public int getTaskCount() {
         return tasks.size();
     }
@@ -129,7 +117,7 @@ public class GroupedOutputFixture {
         boolean foundTask = hasTask(taskName);
 
         if (!foundTask) {
-            throw new AssertionError(String.format("The grouped output for task '%s' could not be found", taskName));
+            throw new AssertionError(String.format("The grouped output for task '%s' could not be found.%nOutput:%n%s", taskName, originalOutput));
         }
 
         return tasks.get(taskName);
