@@ -31,4 +31,8 @@ public interface TestExecutionResult {
      * Returns the result for the first test class whose name starts with the given string.
      */
     TestClassExecutionResult testClassStartsWith(String testClass);
+
+    int getTotalNumberOfTestClassesExecuted();
+
+    boolean testClassExists(String testClass);
 }
