@@ -17,6 +17,7 @@
 package org.gradle.external.javadoc;
 
 import org.gradle.external.javadoc.internal.JavadocOptionFileWriterContext;
+import org.gradle.internal.HasInternalProtocol;
 
 import java.io.IOException;
 
@@ -25,6 +26,7 @@ import java.io.IOException;
  *
  * @param <T> The type which this option represents.
  */
+@HasInternalProtocol
 public interface OptionLessJavadocOptionFileOption<T> {
     T getValue();
 
