@@ -17,6 +17,7 @@ package org.gradle.language.c.plugins;
 
 import com.google.common.collect.Maps;
 import org.gradle.api.Incubating;
+import org.gradle.api.NonNullApi;
 import org.gradle.api.Plugin;
 import org.gradle.api.Project;
 import org.gradle.internal.service.ServiceRegistry;
@@ -44,6 +45,7 @@ import java.util.Map;
  * Adds core C language support.
  */
 @Incubating
+@NonNullApi
 public class CLangPlugin implements Plugin<Project> {
 
     @Override
