@@ -17,12 +17,15 @@ package org.gradle.language.nativeplatform.tasks;
 
 import org.gradle.api.DefaultTask;
 import org.gradle.api.Incubating;
+import org.gradle.api.Transformer;
 import org.gradle.api.file.ConfigurableFileCollection;
 import org.gradle.api.file.DirectoryProperty;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.TaskFileVarFactory;
+import org.gradle.api.model.ObjectFactory;
 import org.gradle.api.provider.ListProperty;
+import org.gradle.api.provider.Property;
 import org.gradle.api.tasks.Input;
 import org.gradle.api.tasks.InputFiles;
 import org.gradle.api.tasks.Internal;
@@ -41,6 +44,8 @@ import org.gradle.language.nativeplatform.internal.incremental.IncrementalCompil
 import org.gradle.nativeplatform.internal.BuildOperationLoggingCompilerDecorator;
 import org.gradle.nativeplatform.platform.NativePlatform;
 import org.gradle.nativeplatform.platform.internal.NativePlatformInternal;
+import org.gradle.nativeplatform.toolchain.Clang;
+import org.gradle.nativeplatform.toolchain.Gcc;
 import org.gradle.nativeplatform.toolchain.NativeToolChain;
 import org.gradle.nativeplatform.toolchain.internal.NativeCompileSpec;
 import org.gradle.nativeplatform.toolchain.internal.NativeToolChainInternal;
@@ -55,8 +60,8 @@ import java.util.Map;
  */
 @Incubating
 public abstract class AbstractNativeCompileTask extends DefaultTask {
-    private NativeToolChainInternal toolChain;
-    private NativePlatformInternal targetPlatform;
+    private final Property<NativePlatform> targetPlatform;
+    private final Property<NativeToolChain> toolChain;
     private boolean positionIndependentCode;
     private boolean debug;
     private boolean optimize;
@@ -68,12 +73,21 @@ public abstract class AbstractNativeCompileTask extends DefaultTask {
     private final IncrementalCompilerBuilder.IncrementalCompiler incrementalCompiler;
 
     public AbstractNativeCompileTask() {
-        includes = getProject().files();
-        source = getTaskFileVarFactory().newInputFileCollection(this);
-        objectFileDir = newOutputDirectory();
-        compilerArgs = getProject().getObjects().listProperty(String.class);
-        incrementalCompiler = getIncrementalCompilerBuilder().newCompiler(this, source, includes);
+        ObjectFactory objectFactory = getProject().getObjects();
+        this.includes = getProject().files();
         dependsOn(includes);
+
+        this.source = getTaskFileVarFactory().newInputFileCollection(this);
+        this.objectFileDir = newOutputDirectory();
+        this.compilerArgs = getProject().getObjects().listProperty(String.class);
+        this.targetPlatform = objectFactory.property(NativePlatform.class);
+        this.toolChain = objectFactory.property(NativeToolChain.class);
+        this.incrementalCompiler = getIncrementalCompilerBuilder().newCompiler(this, source, includes, toolChain.map(new Transformer<Boolean, NativeToolChain>() {
+            @Override
+            public Boolean transform(NativeToolChain nativeToolChain) {
+                return nativeToolChain instanceof Gcc || nativeToolChain instanceof Clang;
+            }
+        }));
     }
 
     @Inject
@@ -100,7 +114,7 @@ public abstract class AbstractNativeCompileTask extends DefaultTask {
     public void compile(IncrementalTaskInputs inputs) {
         BuildOperationLogger operationLogger = getOperationLoggerFactory().newOperationLogger(getName(), getTemporaryDir());
         NativeCompileSpec spec = createCompileSpec();
-        spec.setTargetPlatform(targetPlatform);
+        spec.setTargetPlatform(targetPlatform.get());
         spec.setTempDir(getTemporaryDir());
         spec.setObjectFileDir(objectFileDir.get().getAsFile());
         spec.include(includes);
@@ -115,7 +129,9 @@ public abstract class AbstractNativeCompileTask extends DefaultTask {
 
         configureSpec(spec);
 
-        PlatformToolProvider platformToolProvider = toolChain.select(targetPlatform);
+        NativeToolChainInternal nativeToolChain = (NativeToolChainInternal) toolChain.get();
+        NativePlatformInternal nativePlatform = (NativePlatformInternal) targetPlatform.get();
+        PlatformToolProvider platformToolProvider = nativeToolChain.select(nativePlatform);
         setDidWork(doCompile(spec, platformToolProvider).getDidWork());
     }
 
@@ -134,29 +150,24 @@ public abstract class AbstractNativeCompileTask extends DefaultTask {
 
     /**
      * The tool chain used for compilation.
+     *
+     * @since 4.7
      */
     @Internal
-    public NativeToolChain getToolChain() {
+    public Property<NativeToolChain> getToolChain() {
         return toolChain;
     }
 
-    public void setToolChain(NativeToolChain toolChain) {
-        this.toolChain = (NativeToolChainInternal) toolChain;
-        incrementalCompiler.setToolChain(this.toolChain);
-    }
-
     /**
-     * The platform being targeted.
+     * The platform being compiled for.
+     *
+     * @since 4.7
      */
     @Nested
-    public NativePlatform getTargetPlatform() {
+    public Property<NativePlatform> getTargetPlatform() {
         return targetPlatform;
     }
 
-    public void setTargetPlatform(NativePlatform targetPlatform) {
-        this.targetPlatform = (NativePlatformInternal) targetPlatform;
-    }
-
     /**
      * Should the compiler generate position independent code?
      */
