@@ -39,8 +39,8 @@ import org.gradle.language.swift.SwiftVersion;
 import org.gradle.language.swift.internal.DefaultSwiftBinary;
 import org.gradle.language.swift.internal.DefaultSwiftComponent;
 import org.gradle.language.swift.tasks.SwiftCompile;
-import org.gradle.nativeplatform.platform.NativePlatform;
-import org.gradle.nativeplatform.toolchain.internal.NativeToolChainInternal;
+import org.gradle.nativeplatform.toolchain.internal.ToolType;
+import org.gradle.nativeplatform.toolchain.internal.xcode.MacOSSdkPathLocator;
 import org.gradle.nativeplatform.toolchain.plugins.SwiftCompilerPlugin;
 import org.gradle.swiftpm.internal.SwiftPmTarget;
 import org.gradle.util.VersionNumber;
@@ -56,10 +56,12 @@ import java.util.concurrent.Callable;
 @Incubating
 public class SwiftBasePlugin implements Plugin<ProjectInternal> {
     private final ProjectPublicationRegistry publicationRegistry;
+    private final MacOSSdkPathLocator locator;
 
     @Inject
-    public SwiftBasePlugin(ProjectPublicationRegistry publicationRegistry) {
+    public SwiftBasePlugin(ProjectPublicationRegistry publicationRegistry, MacOSSdkPathLocator locator) {
         this.publicationRegistry = publicationRegistry;
+        this.locator = locator;
     }
 
     @Override
@@ -80,15 +82,15 @@ public class SwiftBasePlugin implements Plugin<ProjectInternal> {
                 SwiftCompile compile = tasks.create(names.getCompileTaskName("swift"), SwiftCompile.class);
                 compile.getModules().from(binary.getCompileModules());
                 compile.getSource().from(binary.getSwiftSource());
-                if (binary.isDebuggable()) {
-                    compile.setDebuggable(true);
-                }
-                if (binary.isOptimized()) {
-                    compile.setOptimized(true);
-                }
+                compile.getDebuggable().set(binary.isDebuggable());
+                compile.getOptimized().set(binary.isOptimized());
                 if (binary.isTestable()) {
                     compile.getCompilerArgs().add("-enable-testing");
                 }
+                if (binary.getTargetPlatform().getOperatingSystem().isMacOsX()) {
+                    compile.getCompilerArgs().add("-sdk");
+                    compile.getCompilerArgs().add(locator.find().getAbsolutePath());
+                }
                 compile.getModuleName().set(binary.getModule());
                 compile.getObjectFileDir().set(buildDirectory.dir("obj/" + names.getDirName()));
                 compile.getModuleFile().set(buildDirectory.file(providers.provider(new Callable<String>() {
@@ -100,12 +102,10 @@ public class SwiftBasePlugin implements Plugin<ProjectInternal> {
                 compile.getSourceCompatibility().set(binary.getSourceCompatibility());
                 binary.getModuleFile().set(compile.getModuleFile());
 
-                NativePlatform currentPlatform = binary.getTargetPlatform();
-                compile.setTargetPlatform(currentPlatform);
+                compile.getTargetPlatform().set(binary.getTargetPlatform());
 
                 // TODO - make this lazy
-                NativeToolChainInternal toolChain = binary.getToolChain();
-                compile.setToolChain(toolChain);
+                compile.getToolChain().set(binary.getToolChain());
 
                 binary.getCompileTask().set(compile);
                 binary.getObjectsDir().set(compile.getObjectFileDir());
@@ -143,7 +143,7 @@ public class SwiftBasePlugin implements Plugin<ProjectInternal> {
                             public SwiftVersion call() throws Exception {
                                 SwiftVersion swiftSourceCompatibility = component.getSourceCompatibility().getOrNull();
                                 if (swiftSourceCompatibility == null) {
-                                    return toSwiftVersion(binary.getPlatformToolProvider().getCompilerMetadata().getVersion());
+                                    return toSwiftVersion(binary.getPlatformToolProvider().getCompilerMetadata(ToolType.SWIFT_COMPILER).getVersion());
                                 }
                                 return swiftSourceCompatibility;
                             }
