@@ -52,6 +52,7 @@ import org.gradle.launcher.exec.BuildActionParameters;
 
 import java.io.InputStream;
 import java.util.List;
+import java.util.UUID;
 
 /**
  * The client piece of the build daemon.
@@ -96,13 +97,13 @@ public class DaemonClient implements BuildActionExecuter<BuildActionParameters>
     private final ExplainingSpec<DaemonContext> compatibilitySpec;
     private final InputStream buildStandardInput;
     private final ExecutorFactory executorFactory;
-    private final IdGenerator<?> idGenerator;
+    private final IdGenerator<UUID> idGenerator;
     private final ProcessEnvironment processEnvironment;
 
     //TODO - outputEventListener and buildStandardInput are per-build settings
     //so down the road we should refactor the code accordingly and potentially attach them to BuildActionParameters
     public DaemonClient(DaemonConnector connector, OutputEventListener outputEventListener, ExplainingSpec<DaemonContext> compatibilitySpec,
-                        InputStream buildStandardInput, ExecutorFactory executorFactory, IdGenerator<?> idGenerator, ProcessEnvironment processEnvironment) {
+                        InputStream buildStandardInput, ExecutorFactory executorFactory, IdGenerator<UUID> idGenerator, ProcessEnvironment processEnvironment) {
         this.connector = connector;
         this.outputEventListener = outputEventListener;
         this.compatibilitySpec = compatibilitySpec;
@@ -112,7 +113,7 @@ public class DaemonClient implements BuildActionExecuter<BuildActionParameters>
         this.processEnvironment = processEnvironment;
     }
 
-    protected IdGenerator<?> getIdGenerator() {
+    protected IdGenerator<UUID> getIdGenerator() {
         return idGenerator;
     }
 
@@ -127,7 +128,7 @@ public class DaemonClient implements BuildActionExecuter<BuildActionParameters>
      * @throws org.gradle.initialization.ReportedException On failure, when the failure has already been logged/reported.
      */
     public Object execute(BuildAction action, BuildRequestContext requestContext, BuildActionParameters parameters, ServiceRegistry contextServices) {
-        Object buildId = idGenerator.generateId();
+        UUID buildId = idGenerator.generateId();
         List<DaemonInitialConnectException> accumulatedExceptions = Lists.newArrayList();
 
         LOGGER.debug("Executing build " + buildId + " in daemon client {pid=" + processEnvironment.maybeGetPid() + "}");
@@ -137,7 +138,7 @@ public class DaemonClient implements BuildActionExecuter<BuildActionParameters>
         for (int i = 1; i < saneNumberOfAttempts; i++) {
             final DaemonClientConnection connection = connector.connect(compatibilitySpec);
             try {
-                Build build = new Build(buildId, connection.getDaemon().getToken(), action, requestContext.getClient(), requestContext.getBuildTimeClock().getStartTime(), parameters);
+                Build build = new Build(buildId, connection.getDaemon().getToken(), action, requestContext.getClient(), requestContext.getStartTime(), parameters);
                 return executeBuild(build, connection, requestContext.getCancellationToken(), requestContext.getEventConsumer());
             } catch (DaemonInitialConnectException e) {
                 // this exception means that we want to try again.
@@ -149,8 +150,8 @@ public class DaemonClient implements BuildActionExecuter<BuildActionParameters>
         }
 
         throw new NoUsableDaemonFoundException("Unable to find a usable idle daemon. I have connected to "
-                + saneNumberOfAttempts + " different daemons but I could not use any of them to run the build. BuildActionParameters were "
-                + parameters + ".", accumulatedExceptions);
+            + saneNumberOfAttempts + " different daemons but I could not use any of them to run the build. BuildActionParameters were "
+            + parameters + ".", accumulatedExceptions);
     }
 
     protected Object executeBuild(Build build, DaemonClientConnection connection, BuildCancellationToken cancellationToken, BuildEventConsumer buildEventConsumer) throws DaemonInitialConnectException {
@@ -208,14 +209,17 @@ public class DaemonClient implements BuildActionExecuter<BuildActionParameters>
 
             while (true) {
                 Message object = connection.receive();
-                LOGGER.trace("Received object #{}, type: {}", objectsReceived++, object == null ? null : object.getClass().getName());
+                objectsReceived++;
+                if (LOGGER.isTraceEnabled()) {
+                    LOGGER.trace("Received object #{}, type: {}", objectsReceived++, object == null ? null : object.getClass().getName());
+                }
 
                 if (object == null) {
                     return handleDaemonDisappearance(build, diagnostics);
                 } else if (object instanceof OutputMessage) {
                     outputEventListener.onOutput(((OutputMessage) object).getEvent());
                 } else if (object instanceof BuildEvent) {
-                    buildEventConsumer.dispatch(((BuildEvent)object).getPayload());
+                    buildEventConsumer.dispatch(((BuildEvent) object).getPayload());
                 } else {
                     return object;
                 }
@@ -231,17 +235,17 @@ public class DaemonClient implements BuildActionExecuter<BuildActionParameters>
         //if he's really dead we should deregister it if it is not already deregistered.
         //if the daemon is not dead we might continue receiving from him (and try to find the bug in messaging infrastructure)
         LOGGER.error("The message received from the daemon indicates that the daemon has disappeared."
-                     + "\nBuild request sent: {}"
-                     + "\nAttempting to read last messages from the daemon log...",  build);
+            + "\nBuild request sent: {}"
+            + "\nAttempting to read last messages from the daemon log...", build);
 
         LOGGER.error(diagnostics.describe());
         throw new DaemonDisappearedException();
     }
 
     private IllegalStateException invalidResponse(Object response, Build command, DaemonDiagnostics diagnostics) {
-        String diagnosticsMessage = diagnostics==null ? "No diagnostics available." : diagnostics.describe();
+        String diagnosticsMessage = diagnostics == null ? "No diagnostics available." : diagnostics.describe();
         return new IllegalStateException(String.format(
-                "Received invalid response from the daemon: '%s' is a result of a type we don't have a strategy to handle. "
+            "Received invalid response from the daemon: '%s' is a result of a type we don't have a strategy to handle. "
                 + "Earlier, '%s' request was sent to the daemon. Diagnostics:\n%s", response, command, diagnosticsMessage));
     }
 }
