@@ -16,13 +16,11 @@
 package org.gradle.launcher.daemon.client;
 
 import org.gradle.api.internal.DocumentationRegistry;
-import org.gradle.internal.time.TrueTimeProvider;
 import org.gradle.internal.concurrent.ExecutorFactory;
 import org.gradle.internal.event.ListenerManager;
-import org.gradle.internal.id.CompositeIdGenerator;
 import org.gradle.internal.id.IdGenerator;
-import org.gradle.internal.id.LongIdGenerator;
 import org.gradle.internal.id.UUIDGenerator;
+import org.gradle.internal.invocation.BuildAction;
 import org.gradle.internal.logging.events.OutputEventListener;
 import org.gradle.internal.logging.progress.DefaultProgressLoggerFactory;
 import org.gradle.internal.logging.progress.ProgressLoggerFactory;
@@ -30,14 +28,19 @@ import org.gradle.internal.logging.services.ProgressLoggingBridge;
 import org.gradle.internal.nativeintegration.ProcessEnvironment;
 import org.gradle.internal.remote.internal.OutgoingConnector;
 import org.gradle.internal.remote.internal.inet.TcpOutgoingConnector;
+import org.gradle.internal.serialize.Serializer;
 import org.gradle.internal.service.DefaultServiceRegistry;
 import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.internal.time.Clock;
+import org.gradle.internal.time.Time;
 import org.gradle.launcher.daemon.context.DaemonCompatibilitySpec;
 import org.gradle.launcher.daemon.context.DaemonContext;
 import org.gradle.launcher.daemon.context.DaemonContextBuilder;
+import org.gradle.launcher.daemon.protocol.DaemonMessageSerializer;
 import org.gradle.launcher.daemon.registry.DaemonRegistry;
 
 import java.io.InputStream;
+import java.util.UUID;
 
 /**
  * Some support wiring for daemon clients.
@@ -60,11 +63,11 @@ public abstract class DaemonClientServicesSupport extends DefaultServiceRegistry
         return new DaemonStopClient(connector, idGenerator);
     }
 
-    ReportDaemonStatusClient createReportDaemonStatusClient(DaemonRegistry registry, DaemonConnector connector, IdGenerator idGenerator, DocumentationRegistry documentationRegistry) {
+    ReportDaemonStatusClient createReportDaemonStatusClient(DaemonRegistry registry, DaemonConnector connector, IdGenerator<UUID> idGenerator, DocumentationRegistry documentationRegistry) {
         return new ReportDaemonStatusClient(registry, connector, idGenerator, documentationRegistry);
     }
 
-    protected DaemonClient createDaemonClient() {
+    protected DaemonClient createDaemonClient(IdGenerator<UUID> idGenerator) {
         DaemonCompatibilitySpec matchingContextSpec = new DaemonCompatibilitySpec(get(DaemonContext.class));
         return new DaemonClient(
                 get(DaemonConnector.class),
@@ -72,7 +75,7 @@ public abstract class DaemonClientServicesSupport extends DefaultServiceRegistry
                 matchingContextSpec,
                 buildStandardInput,
                 get(ExecutorFactory.class),
-                get(IdGenerator.class),
+                idGenerator,
                 get(ProcessEnvironment.class));
     }
 
@@ -87,19 +90,23 @@ public abstract class DaemonClientServicesSupport extends DefaultServiceRegistry
 
     }
 
-    IdGenerator<?> createIdGenerator() {
-        return new CompositeIdGenerator(new UUIDGenerator().generateId(), new LongIdGenerator());
+    IdGenerator<UUID> createIdGenerator() {
+        return new UUIDGenerator();
     }
 
     OutgoingConnector createOutgoingConnector() {
         return new TcpOutgoingConnector();
     }
 
-    ProgressLoggerFactory createProgressLoggerFactory() {
-        return new DefaultProgressLoggerFactory(new ProgressLoggingBridge(get(OutputEventListener.class)), new TrueTimeProvider());
+    Clock createClock() {
+        return Time.clock();
     }
 
-    DaemonConnector createDaemonConnector(DaemonRegistry daemonRegistry, OutgoingConnector outgoingConnector, DaemonStarter daemonStarter, ListenerManager listenerManager, ProgressLoggerFactory progressLoggerFactory) {
-        return new DefaultDaemonConnector(daemonRegistry, outgoingConnector, daemonStarter, listenerManager.getBroadcaster(DaemonStartListener.class), progressLoggerFactory);
+    ProgressLoggerFactory createProgressLoggerFactory(Clock clock) {
+        return new DefaultProgressLoggerFactory(new ProgressLoggingBridge(get(OutputEventListener.class)), clock);
+    }
+
+    DaemonConnector createDaemonConnector(DaemonRegistry daemonRegistry, OutgoingConnector outgoingConnector, DaemonStarter daemonStarter, ListenerManager listenerManager, ProgressLoggerFactory progressLoggerFactory, Serializer<BuildAction> buildActionSerializer) {
+        return new DefaultDaemonConnector(daemonRegistry, outgoingConnector, daemonStarter, listenerManager.getBroadcaster(DaemonStartListener.class), progressLoggerFactory, DaemonMessageSerializer.create(buildActionSerializer));
     }
 }
