@@ -16,6 +16,7 @@
 
 package org.gradle.launcher.exec;
 
+import org.gradle.api.internal.StartParameterInternal;
 import org.gradle.initialization.BuildRequestContext;
 import org.gradle.initialization.GradleLauncher;
 import org.gradle.initialization.GradleLauncherFactory;
@@ -23,31 +24,40 @@ import org.gradle.initialization.RootBuildLifecycleListener;
 import org.gradle.internal.event.ListenerManager;
 import org.gradle.internal.invocation.BuildAction;
 import org.gradle.internal.invocation.BuildActionRunner;
+import org.gradle.internal.invocation.GradleBuildController;
+import org.gradle.internal.jvm.UnsupportedJavaRuntimeException;
 import org.gradle.internal.service.ServiceRegistry;
 
 public class InProcessBuildActionExecuter implements BuildActionExecuter<BuildActionParameters> {
-    private final GradleLauncherFactory gradleLauncherFactory;
     private final BuildActionRunner buildActionRunner;
 
-    public InProcessBuildActionExecuter(GradleLauncherFactory gradleLauncherFactory, BuildActionRunner buildActionRunner) {
-        this.gradleLauncherFactory = gradleLauncherFactory;
+    public InProcessBuildActionExecuter(BuildActionRunner buildActionRunner) {
         this.buildActionRunner = buildActionRunner;
     }
 
     public Object execute(BuildAction action, BuildRequestContext buildRequestContext, BuildActionParameters actionParameters, ServiceRegistry contextServices) {
+        GradleLauncherFactory gradleLauncherFactory = contextServices.get(GradleLauncherFactory.class);
         GradleLauncher gradleLauncher = gradleLauncherFactory.newInstance(action.getStartParameter(), buildRequestContext, contextServices);
+        GradleBuildController buildController = new GradleBuildController(gradleLauncher);
+        checkDeprecations(action.getStartParameter());
         try {
             RootBuildLifecycleListener buildLifecycleListener = contextServices.get(ListenerManager.class).getBroadcaster(RootBuildLifecycleListener.class);
             buildLifecycleListener.afterStart();
             try {
-                GradleBuildController buildController = new GradleBuildController(gradleLauncher);
                 buildActionRunner.run(action, buildController);
                 return buildController.getResult();
             } finally {
                 buildLifecycleListener.beforeComplete();
             }
         } finally {
-            gradleLauncher.stop();
+            buildController.stop();
         }
     }
+
+    private void checkDeprecations(StartParameterInternal startParameter) {
+        UnsupportedJavaRuntimeException.javaDeprecationWarning();
+
+        // This must be done here because DeprecationLogger needs to be initialized properly
+        startParameter.checkDeprecation();
+    }
 }
