@@ -37,12 +37,14 @@ import org.gradle.internal.logging.console.UserInputConsoleRenderer;
 import org.gradle.internal.logging.console.UserInputStandardOutputRenderer;
 import org.gradle.internal.logging.console.WorkInProgressRenderer;
 import org.gradle.internal.logging.events.EndOutputEvent;
+import org.gradle.internal.logging.events.FlushOutputEvent;
 import org.gradle.internal.logging.events.LogLevelChangeEvent;
 import org.gradle.internal.logging.events.OutputEvent;
 import org.gradle.internal.logging.events.OutputEventListener;
 import org.gradle.internal.logging.events.ProgressCompleteEvent;
 import org.gradle.internal.logging.events.ProgressEvent;
 import org.gradle.internal.logging.events.ProgressStartEvent;
+import org.gradle.internal.logging.events.RenderableOutputEvent;
 import org.gradle.internal.logging.format.PrettyPrefixedLogHeaderFormatter;
 import org.gradle.internal.logging.text.StreamBackedStandardOutputListener;
 import org.gradle.internal.logging.text.StreamingStyledTextOutput;
@@ -64,32 +66,19 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     private final AtomicReference<LogLevel> logLevel = new AtomicReference<LogLevel>(LogLevel.LIFECYCLE);
     private final Clock clock;
     private final ListenerBroadcast<OutputEventListener> formatters = new ListenerBroadcast<OutputEventListener>(OutputEventListener.class);
-    private final ListenerBroadcast<StandardOutputListener> stdoutListeners = new ListenerBroadcast<StandardOutputListener>(StandardOutputListener.class);
-    private final ListenerBroadcast<StandardOutputListener> stderrListeners = new ListenerBroadcast<StandardOutputListener>(StandardOutputListener.class);
 
     private ColorMap colourMap;
     private OutputStream originalStdOut;
     private OutputStream originalStdErr;
-    private StreamBackedStandardOutputListener stdOutListener;
-    private StreamBackedStandardOutputListener stdErrListener;
+    private OutputEventListener stdOutListener;
+    private OutputEventListener stdErrListener;
     private OutputEventListener console;
+    private OutputEventListener userListenerChain;
+    private ListenerBroadcast<StandardOutputListener> userStdoutListeners;
+    private ListenerBroadcast<StandardOutputListener> userStderrListeners;
 
     public OutputEventRenderer(final Clock clock) {
         this.clock = clock;
-        OutputEventListener stdOutChain = new LazyListener(new Factory<OutputEventListener>() {
-            @Override
-            public OutputEventListener create() {
-                return onNonError(new UserInputStandardOutputRenderer(new BuildLogLevelFilterRenderer(new ProgressLogEventGenerator(new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(stdoutListeners.getSource())), false)), clock));
-            }
-        });
-        formatters.add(stdOutChain);
-        OutputEventListener stdErrChain = new LazyListener(new Factory<OutputEventListener>() {
-            @Override
-            public OutputEventListener create() {
-                return onError(new BuildLogLevelFilterRenderer(new ProgressLogEventGenerator(new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(stderrListeners.getSource())), false)));
-            }
-        });
-        formatters.add(stdErrChain);
     }
 
     @Override
@@ -111,8 +100,7 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
             // TODO - also close console when it is replaced
             if (snapshot.console != console) {
                 if (snapshot.console == null) {
-                    formatters.remove(console);
-                    console.onOutput(new EndOutputEvent());
+                    removeChain(console);
                     console = null;
                 } else {
                     throw new UnsupportedOperationException("Cannot restore previous console. This is not implemented yet.");
@@ -121,6 +109,16 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
         }
     }
 
+    private void addChain(OutputEventListener listener) {
+        listener.onOutput(new LogLevelChangeEvent(logLevel.get()));
+        formatters.add(listener);
+    }
+
+    private void removeChain(OutputEventListener listener) {
+        formatters.remove(listener);
+        listener.onOutput(new EndOutputEvent());
+    }
+
     public ColorMap getColourMap() {
         synchronized (lock) {
             if (colourMap == null) {
@@ -130,6 +128,11 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
         return colourMap;
     }
 
+    @Override
+    public void flush() {
+        onOutput(new FlushOutputEvent());
+    }
+
     public OutputStream getOriginalStdOut() {
         return originalStdOut;
     }
@@ -145,59 +148,69 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     }
 
     @Override
-    public void attachAnsiConsole(OutputStream outputStream) {
-        attachAnsiConsole(outputStream, false);
-    }
-
-    protected void attachAnsiConsole(OutputStream outputStream, boolean verbose) {
+    public void attachConsole(OutputStream outputStream, OutputStream errorStream, ConsoleOutput consoleOutput) {
         synchronized (lock) {
-            ConsoleMetaData consoleMetaData = FallbackConsoleMetaData.INSTANCE;
-            OutputStreamWriter writer = new OutputStreamWriter(outputStream);
-            Console console = new AnsiConsole(writer, writer, getColourMap(), consoleMetaData, true);
-            addConsole(console, true, true, consoleMetaData, verbose);
+            if (consoleOutput == ConsoleOutput.Plain) {
+                addPlainConsole(new StreamBackedStandardOutputListener(outputStream), new StreamBackedStandardOutputListener(errorStream));
+            } else {
+                ConsoleMetaData consoleMetaData = FallbackConsoleMetaData.INSTANCE;
+                OutputStreamWriter writer = new OutputStreamWriter(outputStream);
+                Console console = new AnsiConsole(writer, writer, getColourMap(), consoleMetaData, true);
+                addRichConsole(console, true, true, consoleMetaData, consoleOutput == ConsoleOutput.Verbose);
+            }
         }
     }
 
     public void attachSystemOutAndErr() {
-        addStandardOutputListener();
-        addStandardErrorListener();
+        addSystemOutAsLoggingDestination();
+        addSystemErrAsLoggingDestination();
     }
 
-    private void addStandardOutputListener() {
+    private void addSystemOutAsLoggingDestination() {
         synchronized (lock) {
             originalStdOut = System.out;
             if (stdOutListener != null) {
-                stdoutListeners.remove(stdOutListener);
+                removeChain(stdOutListener);
             }
-            stdOutListener = new StreamBackedStandardOutputListener((Appendable) System.out);
-            addStandardOutputListener(stdOutListener);
+            stdOutListener = new LazyListener(new Factory<OutputEventListener>() {
+                @Override
+                public OutputEventListener create() {
+                    return onNonError(new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(new StreamBackedStandardOutputListener((Appendable) originalStdOut))));
+                }
+            });
+            addChain(stdOutListener);
         }
     }
 
-    private void addStandardErrorListener() {
+    private void addSystemErrAsLoggingDestination() {
         synchronized (lock) {
             originalStdErr = System.err;
             if (stdErrListener != null) {
-                stderrListeners.remove(stdErrListener);
+                removeChain(stdErrListener);
             }
-            stdErrListener = new StreamBackedStandardOutputListener((Appendable) System.err);
-            addStandardErrorListener(stdErrListener);
+            stdErrListener = new LazyListener(new Factory<OutputEventListener>() {
+                @Override
+                public OutputEventListener create() {
+                    return onError(new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(new StreamBackedStandardOutputListener((Appendable) originalStdErr))));
+                }
+            });
+            addChain(stdErrListener);
         }
     }
 
-    private void removeStandardOutputListener() {
+    private void removeSystemOutAsLoggingDestination() {
         synchronized (lock) {
             if (stdOutListener != null) {
-                stdoutListeners.remove(stdOutListener);
+                removeChain(stdOutListener);
                 stdOutListener = null;
             }
         }
     }
 
-    private void removeStandardErrorListener() {
+    private void removeSystemErrAsLoggingDestination() {
         synchronized (lock) {
             if (stdErrListener != null) {
-                stderrListeners.remove(stdErrListener);
+                removeChain(stdErrListener);
                 stdErrListener = null;
             }
         }
@@ -205,78 +218,129 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
 
     public void addOutputEventListener(OutputEventListener listener) {
         synchronized (lock) {
-            formatters.add(listener);
+            addChain(listener);
         }
     }
 
     public void removeOutputEventListener(OutputEventListener listener) {
         synchronized (lock) {
-            formatters.remove(listener);
+            removeChain(listener);
         }
     }
 
-    public OutputEventRenderer addConsole(Console console, boolean stdout, boolean stderr, ConsoleMetaData consoleMetaData) {
-        return addConsole(console, stdout, stderr, consoleMetaData, false);
+    public OutputEventRenderer addRichConsole(Console console, boolean stdout, boolean stderr, ConsoleMetaData consoleMetaData) {
+        return addRichConsole(console, stdout, stderr, consoleMetaData, false);
     }
 
-    public OutputEventRenderer addConsole(Console console, boolean stdout, boolean stderr, ConsoleMetaData consoleMetaData, boolean verbose) {
+    public OutputEventRenderer addRichConsole(Console console, boolean stdout, boolean stderr, ConsoleMetaData consoleMetaData, boolean verbose) {
         final OutputEventListener consoleChain = new ThrottlingOutputEventListener(
             new UserInputConsoleRenderer(
                 new BuildStatusRenderer(
                     new WorkInProgressRenderer(
                         new BuildLogLevelFilterRenderer(
-                            new GroupingProgressLogEventGenerator(new StyledTextOutputBackedRenderer(console.getBuildOutputArea()), clock, new PrettyPrefixedLogHeaderFormatter(!verbose), verbose)),
+                            new GroupingProgressLogEventGenerator(new StyledTextOutputBackedRenderer(console.getBuildOutputArea()), new PrettyPrefixedLogHeaderFormatter(), verbose)),
                         console.getBuildProgressArea(), new DefaultWorkInProgressFormatter(consoleMetaData), new ConsoleLayoutCalculator(consoleMetaData)),
-                    console.getStatusBar(), console, consoleMetaData, clock),
+                    console.getStatusBar(), console, consoleMetaData),
                 console),
-                clock);
+            clock);
+        return addConsoleChain(consoleChain, stdout, stderr);
+    }
+
+    public OutputEventRenderer addPlainConsole() {
+        return addPlainConsole(new StreamBackedStandardOutputListener((Appendable) originalStdOut), new StreamBackedStandardOutputListener((Appendable)originalStdErr));
+    }
+
+    private OutputEventRenderer addPlainConsole(StandardOutputListener outputListener, StandardOutputListener errorListener) {
+        final OutputEventListener stdoutChain = new UserInputStandardOutputRenderer(new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(outputListener)), clock);
+        final OutputEventListener stderrChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(errorListener));
+        OutputEventListener consoleChain = new ThrottlingOutputEventListener(
+            new BuildLogLevelFilterRenderer(
+                new GroupingProgressLogEventGenerator(new PlainConsoleDispatchingListener(stderrChain, stdoutChain), new PrettyPrefixedLogHeaderFormatter(), true)
+            ),
+            clock
+        );
+
+        return addConsoleChain(consoleChain, true, true);
+    }
+
+    private OutputEventRenderer addConsoleChain(OutputEventListener consoleChain, boolean stdout, boolean stderr) {
         synchronized (lock) {
             if (stdout && stderr) {
                 this.console = consoleChain;
-                removeStandardOutputListener();
-                removeStandardErrorListener();
+                removeSystemOutAsLoggingDestination();
+                removeSystemErrAsLoggingDestination();
             } else if (stdout) {
                 this.console = onNonError(consoleChain);
-                removeStandardOutputListener();
-            } else {
+                removeSystemOutAsLoggingDestination();
+            } else if (stderr) {
                 this.console = onError(consoleChain);
-                removeStandardErrorListener();
+                removeSystemErrAsLoggingDestination();
+            } else {
+                this.console = consoleChain;
             }
-            consoleChain.onOutput(new LogLevelChangeEvent(logLevel.get()));
-            formatters.add(this.console);
+            addChain(this.console);
         }
         return this;
     }
 
     private OutputEventListener onError(final OutputEventListener listener) {
-        return new OutputEventListener() {
-            public void onOutput(OutputEvent event) {
-                if (event.getLogLevel() == LogLevel.ERROR || event.getLogLevel() == null) {
-                    listener.onOutput(event);
-                }
-            }
-        };
+        return new LogEventDispatcher(null, listener);
     }
 
     private OutputEventListener onNonError(final OutputEventListener listener) {
-        return new OutputEventListener() {
-            public void onOutput(OutputEvent event) {
-                if (event.getLogLevel() != LogLevel.ERROR || event.getLogLevel() == null) {
-                    listener.onOutput(event);
-                }
+        return new LogEventDispatcher(listener, null);
+    }
+
+    @Override
+    public void enableUserStandardOutputListeners() {
+        // Create all of the pipeline eagerly as soon as this is enabled, to track the state of build operations.
+        // All of the pipelines do this, so should instead have a single stage that tracks this for all pipelines and that can replay the current state to new pipelines
+        // Then, a pipeline can be added for each listener as required
+        synchronized (lock) {
+            if (userStdoutListeners == null) {
+                userStdoutListeners = new ListenerBroadcast<StandardOutputListener>(StandardOutputListener.class);
+                userStderrListeners = new ListenerBroadcast<StandardOutputListener>(StandardOutputListener.class);
+                final OutputEventListener stdOutChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(userStdoutListeners.getSource()));
+                final OutputEventListener stdErrChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(userStderrListeners.getSource()));
+                userListenerChain = new BuildLogLevelFilterRenderer(
+                    new ProgressLogEventGenerator(new OutputEventListener() {
+                        @Override
+                        public void onOutput(OutputEvent event) {
+                            // Do not forward events for rendering when there are no listeners to receive
+                            if (event instanceof LogLevelChangeEvent) {
+                                stdOutChain.onOutput(event);
+                                stdErrChain.onOutput(event);
+                            } else if (event.getLogLevel() == LogLevel.ERROR && !userStderrListeners.isEmpty() && event instanceof RenderableOutputEvent) {
+                                stdErrChain.onOutput(event);
+                            } else if (event.getLogLevel() != LogLevel.ERROR && !userStdoutListeners.isEmpty() && event instanceof RenderableOutputEvent) {
+                                stdOutChain.onOutput(event);
+                            }
+                        }
+                    })
+                );
+                addChain(userListenerChain);
             }
-        };
+        }
+    }
+
+    private void assertUserListenersEnabled() {
+        if (userListenerChain == null) {
+            throw new IllegalStateException("Custom standard output listeners not enabled.");
+        }
+        userListenerChain.onOutput(new FlushOutputEvent());
     }
 
     public void addStandardErrorListener(StandardOutputListener listener) {
         synchronized (lock) {
-            stderrListeners.add(listener);
+            assertUserListenersEnabled();
+            userStderrListeners.add(listener);
         }
     }
 
     public void addStandardOutputListener(StandardOutputListener listener) {
         synchronized (lock) {
-            stdoutListeners.add(listener);
+            assertUserListenersEnabled();
+            userStdoutListeners.add(listener);
         }
     }
 
@@ -288,16 +352,17 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
         addStandardErrorListener(new StreamBackedStandardOutputListener(outputStream));
     }
 
-
     public void removeStandardOutputListener(StandardOutputListener listener) {
         synchronized (lock) {
-            stdoutListeners.remove(listener);
+            assertUserListenersEnabled();
+            userStdoutListeners.remove(listener);
         }
     }
 
     public void removeStandardErrorListener(StandardOutputListener listener) {
         synchronized (lock) {
-            stderrListeners.remove(listener);
+            assertUserListenersEnabled();
+            userStderrListeners.remove(listener);
         }
     }
 
@@ -340,6 +405,7 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     private static class LazyListener implements OutputEventListener {
         private Factory<OutputEventListener> factory;
         private OutputEventListener delegate;
+        private LogLevelChangeEvent pendingLogLevel;
 
         private LazyListener(Factory<OutputEventListener> factory) {
             this.factory = factory;
@@ -348,10 +414,24 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
         @Override
         public void onOutput(OutputEvent event) {
             if (delegate == null) {
+                if (event instanceof EndOutputEvent || event instanceof FlushOutputEvent) {
+                    // Ignore
+                    return;
+                }
+                if (event instanceof LogLevelChangeEvent) {
+                    // Keep until the listener is created
+                    pendingLogLevel = (LogLevelChangeEvent) event;
+                    return;
+                }
                 delegate = factory.create();
                 factory = null;
+                if (pendingLogLevel != null) {
+                    delegate.onOutput(pendingLogLevel);
+                    pendingLogLevel = null;
+                }
             }
             delegate.onOutput(event);
         }
     }
+
 }
