@@ -18,8 +18,6 @@ package org.gradle.internal.logging.sink;
 
 import org.gradle.api.logging.LogLevel;
 import org.gradle.internal.SystemProperties;
-import org.gradle.internal.logging.events.BatchOutputEventListener;
-import org.gradle.internal.logging.events.OperationIdentifier;
 import org.gradle.internal.logging.events.OutputEvent;
 import org.gradle.internal.logging.events.OutputEventListener;
 import org.gradle.internal.logging.events.ProgressCompleteEvent;
@@ -27,6 +25,7 @@ import org.gradle.internal.logging.events.ProgressEvent;
 import org.gradle.internal.logging.events.ProgressStartEvent;
 import org.gradle.internal.logging.events.RenderableOutputEvent;
 import org.gradle.internal.logging.events.StyledTextOutputEvent;
+import org.gradle.internal.operations.OperationIdentifier;
 import org.gradle.util.GUtil;
 
 import java.util.Arrays;
@@ -41,16 +40,14 @@ import static org.gradle.internal.logging.text.StyledTextOutput.Style;
  * An {@code org.gradle.logging.internal.OutputEventListener} implementation which generates output events to log the
  * progress of operations.
  */
-public class ProgressLogEventGenerator extends BatchOutputEventListener {
+public class ProgressLogEventGenerator implements OutputEventListener {
     private static final String EOL = SystemProperties.getInstance().getLineSeparator();
 
     private final OutputEventListener listener;
-    private final boolean deferHeader;
     private final Map<OperationIdentifier, Operation> operations = new LinkedHashMap<OperationIdentifier, Operation>();
 
-    public ProgressLogEventGenerator(OutputEventListener listener, boolean deferHeader) {
+    public ProgressLogEventGenerator(OutputEventListener listener) {
         this.listener = listener;
-        this.deferHeader = deferHeader;
     }
 
     public void onOutput(OutputEvent event) {
@@ -101,7 +98,7 @@ public class ProgressLogEventGenerator extends BatchOutputEventListener {
         Operation operation = new Operation(progressStartEvent.getCategory(), progressStartEvent.getLoggingHeader(), progressStartEvent.getTimestamp(), progressStartEvent.getBuildOperationId());
         operations.put(progressStartEvent.getProgressOperationId(), operation);
 
-        if (!deferHeader || !(progressStartEvent.getLoggingHeader() != null && progressStartEvent.getLoggingHeader().equals(progressStartEvent.getShortDescription()))) {
+        if (!(progressStartEvent.getLoggingHeader() != null && progressStartEvent.getLoggingHeader().equals(progressStartEvent.getShortDescription()))) {
             operation.startHeader();
         }
     }
@@ -109,7 +106,7 @@ public class ProgressLogEventGenerator extends BatchOutputEventListener {
     enum State {None, HeaderStarted, HeaderCompleted, Completed}
 
     private class Operation {
-        private final Object buildOperationIdentifier;
+        private final OperationIdentifier buildOperationIdentifier;
         private final String category;
         private final String loggingHeader;
         private final long startTime;
@@ -118,7 +115,7 @@ public class ProgressLogEventGenerator extends BatchOutputEventListener {
         private State state = State.None;
         private long completeTime;
 
-        private Operation(String category, String loggingHeader, long startTime, Object buildOperationIdentifier) {
+        private Operation(String category, String loggingHeader, long startTime, OperationIdentifier buildOperationIdentifier) {
             this.category = category;
             this.loggingHeader = loggingHeader;
             this.startTime = startTime;
