@@ -16,6 +16,7 @@
 package org.gradle.api.publication.maven.internal.pom;
 
 import groovy.lang.Closure;
+import groovy.lang.GroovyObject;
 import org.apache.maven.model.Dependency;
 import org.apache.maven.model.Model;
 import org.apache.maven.model.io.xpp3.MavenXpp3Writer;
@@ -133,6 +134,16 @@ public class DefaultMavenPom implements MavenPom {
         return this;
     }
 
+    @Override
+    public DefaultMavenPom project(final Action<? super GroovyObject> action) {
+        return project(new Closure(this, this) {
+            @SuppressWarnings("unused")
+            public void doCall() {
+                action.execute((GroovyObject) getDelegate());
+            }
+        });
+    }
+
     public Model getModel() {
         return model;
     }
