@@ -16,42 +16,50 @@
 
 package org.gradle.jvm.internal;
 
-import org.gradle.api.Transformer;
-import org.gradle.api.artifacts.ResolvedArtifact;
+import org.gradle.api.artifacts.ResolutionStrategy;
+import org.gradle.api.artifacts.ResolveException;
 import org.gradle.api.artifacts.component.ComponentArtifactIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.artifacts.type.ArtifactTypeContainer;
 import org.gradle.api.attributes.AttributeContainer;
-import org.gradle.api.attributes.AttributesSchema;
 import org.gradle.api.internal.artifacts.ArtifactDependencyResolver;
 import org.gradle.api.internal.artifacts.GlobalDependencyResolutionRules;
-import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 import org.gradle.api.internal.artifacts.ResolveContext;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ArtifactSet;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ArtifactVisitor;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.BuildDependenciesVisitor;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.DefaultResolvedArtifactsBuilder;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.DependencyArtifactsVisitor;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ParallelResolveArtifactSet;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvableArtifact;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvedArtifactSet;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvedVariant;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvedVariantSet;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.SelectedArtifactResults;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphEdge;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphNode;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphVisitor;
 import org.gradle.api.internal.artifacts.repositories.ResolutionAwareRepository;
+import org.gradle.api.internal.artifacts.transform.VariantSelector;
+import org.gradle.api.internal.artifacts.type.ArtifactTypeRegistry;
+import org.gradle.api.internal.attributes.AttributesSchemaInternal;
+import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.api.internal.file.AbstractFileCollection;
 import org.gradle.api.internal.tasks.TaskDependencies;
 import org.gradle.api.specs.Specs;
 import org.gradle.api.tasks.TaskDependency;
+import org.gradle.initialization.BuildIdentity;
 import org.gradle.internal.UncheckedException;
+import org.gradle.internal.component.local.model.LocalFileDependencyMetadata;
 import org.gradle.internal.component.model.DependencyMetadata;
+import org.gradle.internal.component.model.VariantResolveMetadata;
+import org.gradle.internal.operations.BuildOperationExecutor;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
 import org.gradle.language.base.internal.resolve.LibraryResolveException;
 import org.gradle.platform.base.internal.BinarySpecInternal;
 
 import java.io.File;
 import java.util.ArrayList;
-import java.util.Collection;
 import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
@@ -63,9 +71,9 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
     private final BinarySpecInternal binary;
     private final ArtifactDependencyResolver dependencyResolver;
     private final ResolveContext resolveContext;
-    private final AttributesSchema attributesSchema;
-    private final ImmutableModuleIdentifierFactory moduleIdentifierFactory;
-    private final ModuleExclusions moduleExclusions;
+    private final AttributesSchemaInternal attributesSchema;
+    private final BuildOperationExecutor buildOperationExecutor;
+    private final BuildIdentity thisBuild;
 
     private final String descriptor;
     private ResolveResult resolveResult;
@@ -76,15 +84,17 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
         ArtifactDependencyResolver dependencyResolver,
         List<ResolutionAwareRepository> remoteRepositories,
         ResolveContext resolveContext,
-        AttributesSchema attributesSchema, ImmutableModuleIdentifierFactory moduleIdentifierFactory, ModuleExclusions moduleExclusions) {
+        AttributesSchemaInternal attributesSchema,
+        BuildOperationExecutor buildOperationExecutor,
+        BuildIdentity thisBuild) {
         this.binary = binarySpec;
         this.descriptor = descriptor;
         this.dependencyResolver = dependencyResolver;
         this.remoteRepositories = remoteRepositories;
         this.resolveContext = resolveContext;
         this.attributesSchema = attributesSchema;
-        this.moduleIdentifierFactory = moduleIdentifierFactory;
-        this.moduleExclusions = moduleExclusions;
+        this.buildOperationExecutor = buildOperationExecutor;
+        this.thisBuild = thisBuild;
     }
 
     @Override
@@ -96,9 +106,10 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
     public Set<File> getFiles() {
         ensureResolved(true);
         final Set<File> result = new LinkedHashSet<File>();
-        resolveResult.artifactsResults.getArtifacts().visit(new ArtifactVisitor() {
+        ParallelResolveArtifactSet artifacts = ParallelResolveArtifactSet.wrap(resolveResult.artifactsResults.getArtifacts(), buildOperationExecutor);
+        artifacts.visit(new ArtifactVisitor() {
             @Override
-            public void visitArtifact(AttributeContainer variant, ResolvedArtifact artifact) {
+            public void visitArtifact(String variantName, AttributeContainer variantAttributes, ResolvableArtifact artifact) {
                 result.add(artifact.getFile());
             }
 
@@ -113,7 +124,12 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
             }
 
             @Override
-            public void visitFile(ComponentArtifactIdentifier artifactIdentifier, AttributeContainer variant, File file) {
+            public boolean requireArtifactFiles() {
+                return true;
+            }
+
+            @Override
+            public void visitFile(ComponentArtifactIdentifier artifactIdentifier, String variantName, AttributeContainer variantAttributes, File file) {
                 result.add(file);
             }
         });
@@ -124,8 +140,22 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
     public TaskDependency getBuildDependencies() {
         ensureResolved(false);
 
-        List<TaskDependency> taskDependencies = new ArrayList<TaskDependency>();
-        resolveResult.artifactsResults.getArtifacts().collectBuildDependencies(taskDependencies);
+        final List<Object> taskDependencies = new ArrayList<Object>();
+        final List<Throwable> failures = new ArrayList<Throwable>();
+        resolveResult.artifactsResults.getArtifacts().collectBuildDependencies(new BuildDependenciesVisitor() {
+            @Override
+            public void visitDependency(Object dep) {
+                taskDependencies.add(dep);
+            }
+
+            @Override
+            public void visitFailure(Throwable failure) {
+                failures.add(failure);
+            }
+        });
+        if (!failures.isEmpty()) {
+            throw new ResolveException(getDisplayName(), failures);
+        }
         return TaskDependencies.of(taskDependencies);
     }
 
@@ -140,7 +170,22 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
 
     private ResolveResult resolve() {
         ResolveResult result = new ResolveResult();
-        dependencyResolver.resolve(resolveContext, remoteRepositories, globalRules, Specs.<DependencyMetadata>satisfyAll(), result, result, attributesSchema, moduleIdentifierFactory, moduleExclusions);
+        dependencyResolver.resolve(resolveContext, remoteRepositories, globalRules, Specs.<DependencyMetadata>satisfyAll(), result, result, attributesSchema, new ArtifactTypeRegistry() {
+            @Override
+            public ImmutableAttributes mapAttributesFor(File file) {
+                return ImmutableAttributes.EMPTY;
+            }
+
+            @Override
+            public ImmutableAttributes mapAttributesFor(VariantResolveMetadata variant) {
+                return variant.getAttributes().asImmutable();
+            }
+
+            @Override
+            public ArtifactTypeContainer create() {
+                throw new UnsupportedOperationException();
+            }
+        });
         return result;
     }
 
@@ -152,7 +197,7 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
 
     class ResolveResult implements DependencyGraphVisitor, DependencyArtifactsVisitor {
         public final List<Throwable> notFound = new LinkedList<Throwable>();
-        public DefaultResolvedArtifactsBuilder artifactsBuilder = new DefaultResolvedArtifactsBuilder(true);
+        public DefaultResolvedArtifactsBuilder artifactsBuilder = new DefaultResolvedArtifactsBuilder(thisBuild.getCurrentBuild(), true, ResolutionStrategy.SortOrder.DEFAULT);
         public SelectedArtifactResults artifactsResults;
 
         @Override
@@ -160,13 +205,14 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
         }
 
         @Override
-        public void visitNode(DependencyGraphNode resolvedConfiguration) {
-            for (DependencyGraphEdge dependency : resolvedConfiguration.getOutgoingEdges()) {
+        public void visitNode(DependencyGraphNode node) {
+            for (DependencyGraphEdge dependency : node.getOutgoingEdges()) {
                 ModuleVersionResolveException failure = dependency.getFailure();
                 if (failure != null) {
                     notFound.add(failure);
                 }
             }
+            artifactsBuilder.visitNode(node);
         }
 
         @Override
@@ -174,7 +220,7 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
         }
 
         @Override
-        public void visitEdges(DependencyGraphNode resolvedConfiguration) {
+        public void visitEdges(DependencyGraphNode node) {
         }
 
         @Override
@@ -186,17 +232,22 @@ public class DependencyResolvingClasspath extends AbstractFileCollection {
         }
 
         @Override
-        public void visitArtifacts(DependencyGraphNode from, DependencyGraphNode to, ArtifactSet artifacts) {
-            artifactsBuilder.visitArtifacts(from, to, artifacts);
+        public void visitArtifacts(DependencyGraphNode from, LocalFileDependencyMetadata fileDependency, int artifactSetId, ArtifactSet artifactSet) {
+            artifactsBuilder.visitArtifacts(from, fileDependency, artifactSetId, artifactSet);
+        }
+
+        @Override
+        public void visitArtifacts(DependencyGraphNode from, DependencyGraphNode to, int artifactSetId, ArtifactSet artifacts) {
+            artifactsBuilder.visitArtifacts(from, to, artifactSetId, artifacts);
         }
 
         @Override
         public void finishArtifacts() {
-            artifactsResults = artifactsBuilder.complete().select(Specs.<ComponentIdentifier>satisfyAll(), new Transformer<ResolvedArtifactSet, Collection<? extends ResolvedVariant>>() {
+            artifactsResults = artifactsBuilder.complete().select(Specs.<ComponentIdentifier>satisfyAll(), new VariantSelector() {
                 @Override
-                public ResolvedArtifactSet transform(Collection<? extends ResolvedVariant> variants) {
+                public ResolvedArtifactSet select(ResolvedVariantSet variants) {
                     // Select the first variant
-                    return variants.iterator().next().getArtifacts();
+                    return variants.getVariants().iterator().next().getArtifacts();
                 }
             });
         }
