@@ -15,11 +15,12 @@
  */
 package org.gradle.jvm.internal.resolve;
 
-import org.gradle.api.attributes.AttributesSchema;
 import org.gradle.api.internal.artifacts.ArtifactDependencyResolver;
 import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.api.internal.artifacts.repositories.ResolutionAwareRepository;
+import org.gradle.api.internal.attributes.AttributesSchemaInternal;
+import org.gradle.initialization.BuildIdentity;
+import org.gradle.internal.operations.BuildOperationExecutor;
 import org.gradle.jvm.internal.DependencyResolvingClasspath;
 import org.gradle.language.base.LanguageSourceSet;
 import org.gradle.platform.base.DependencySpec;
@@ -36,14 +37,18 @@ public class SourceSetDependencyResolvingClasspath extends DependencyResolvingCl
         ArtifactDependencyResolver dependencyResolver,
         VariantsMetaData binaryVariants,
         List<ResolutionAwareRepository> remoteRepositories,
-        AttributesSchema attributesSchema,
+        AttributesSchemaInternal attributesSchema,
         ImmutableModuleIdentifierFactory moduleIdentifierFactory,
-        ModuleExclusions moduleExclusions) {
+        BuildOperationExecutor buildOperationExecutor,
+        BuildIdentity thisBuild) {
         super(binarySpec,
             "source set '" + sourceSet.getDisplayName() + "'",
             dependencyResolver,
             remoteRepositories,
-            new JvmLibraryResolveContext(binarySpec.getId(), binaryVariants, dependencies, UsageKind.API, sourceSet.getDisplayName(), moduleIdentifierFactory), attributesSchema, moduleIdentifierFactory, moduleExclusions);
+            new JvmLibraryResolveContext(binarySpec.getId(), binaryVariants, dependencies, UsageKind.API, sourceSet.getDisplayName(), moduleIdentifierFactory),
+            attributesSchema,
+            buildOperationExecutor,
+            thisBuild);
     }
 
 }
