@@ -19,14 +19,14 @@ package org.gradle.jvm.internal.services;
 import org.gradle.api.internal.artifacts.ResolveContext;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.ComponentResolvers;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.ResolverProviderFactory;
-import org.gradle.api.internal.project.taskfactory.ClasspathPropertyAnnotationHandler;
-import org.gradle.api.internal.project.taskfactory.CompileClasspathPropertyAnnotationHandler;
 import org.gradle.api.internal.resolve.DefaultLocalLibraryResolver;
 import org.gradle.api.internal.resolve.LocalLibraryDependencyResolver;
 import org.gradle.api.internal.resolve.ProjectModelResolver;
 import org.gradle.api.internal.resolve.VariantBinarySelector;
+import org.gradle.api.internal.tasks.properties.annotations.ClasspathPropertyAnnotationHandler;
+import org.gradle.api.internal.tasks.properties.annotations.CompileClasspathPropertyAnnotationHandler;
 import org.gradle.internal.service.ServiceRegistration;
-import org.gradle.internal.service.scopes.PluginServiceRegistry;
+import org.gradle.internal.service.scopes.AbstractPluginServiceRegistry;
 import org.gradle.jvm.JvmBinarySpec;
 import org.gradle.jvm.internal.JarBinaryRenderer;
 import org.gradle.jvm.internal.resolve.DefaultJavaPlatformVariantAxisCompatibility;
@@ -44,7 +44,7 @@ import org.gradle.process.internal.ExecActionFactory;
 
 import java.util.List;
 
-public class PlatformJvmServices implements PluginServiceRegistry {
+public class PlatformJvmServices extends AbstractPluginServiceRegistry {
     @Override
     public void registerGlobalServices(ServiceRegistration registration) {
         registration.add(JarBinaryRenderer.class);
@@ -53,23 +53,11 @@ public class PlatformJvmServices implements PluginServiceRegistry {
         registration.add(CompileClasspathPropertyAnnotationHandler.class);
     }
 
-    @Override
-    public void registerBuildSessionServices(ServiceRegistration registration) {
-    }
-
     @Override
     public void registerBuildServices(ServiceRegistration registration) {
         registration.addProvider(new BuildScopeServices());
     }
 
-    @Override
-    public void registerGradleServices(ServiceRegistration registration) {
-    }
-
-    @Override
-    public void registerProjectServices(ServiceRegistration registration) {
-    }
-
     private class BuildScopeServices {
         LocalLibraryDependencyResolverFactory createResolverProviderFactory(ProjectModelResolver projectModelResolver, ModelSchemaStore schemaStore, List<VariantAxisCompatibilityFactory> factories) {
             return new LocalLibraryDependencyResolverFactory(projectModelResolver, schemaStore, factories);
