@@ -24,7 +24,6 @@ import org.gradle.api.tasks.OutputFile;
 import org.gradle.api.tasks.SourceTask;
 import org.gradle.api.tasks.TaskAction;
 import org.gradle.internal.ErroringAction;
-import org.gradle.util.internal.PatchedClassReader;
 import org.objectweb.asm.ClassReader;
 
 import java.io.BufferedOutputStream;
@@ -119,7 +118,7 @@ public class ApiJar extends SourceTask {
                         if (!isClassFile(sourceFile)) {
                             continue;
                         }
-                        ClassReader classReader = new PatchedClassReader(readFileToByteArray(sourceFile));
+                        ClassReader classReader = new ClassReader(readFileToByteArray(sourceFile));
                         if (!apiClassExtractor.shouldExtractApiClassFrom(classReader)) {
                             continue;
                         }
