@@ -18,6 +18,7 @@ package org.gradle.api.publish.internal;
 
 import com.google.common.base.Strings;
 import com.google.gson.stream.JsonWriter;
+import org.apache.commons.lang.StringUtils;
 import org.gradle.api.Named;
 import org.gradle.api.artifacts.Dependency;
 import org.gradle.api.artifacts.DependencyConstraint;
@@ -31,11 +32,14 @@ import org.gradle.api.artifacts.PublishArtifact;
 import org.gradle.api.artifacts.VersionConstraint;
 import org.gradle.api.attributes.Attribute;
 import org.gradle.api.attributes.AttributeContainer;
+import org.gradle.api.capabilities.Capability;
+import org.gradle.api.component.ComponentWithCoordinates;
 import org.gradle.api.component.ComponentWithVariants;
 import org.gradle.api.component.SoftwareComponent;
 import org.gradle.api.internal.artifacts.DefaultExcludeRule;
 import org.gradle.api.internal.artifacts.dependencies.DefaultImmutableVersionConstraint;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.parser.ModuleMetadataParser;
+import org.gradle.api.internal.artifacts.ivyservice.projectmodule.ProjectDependencyPublicationResolver;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.api.internal.component.SoftwareComponentInternal;
 import org.gradle.api.internal.component.UsageContext;
@@ -191,11 +195,15 @@ public class ModuleMetadataFileGenerator {
         }
         if (component instanceof ComponentWithVariants) {
             for (SoftwareComponent childComponent : ((ComponentWithVariants) component).getVariants()) {
-                ComponentData componentData = componentCoordinates.get(childComponent);
-                ModuleVersionIdentifier childCoordinates = componentData == null ? null : componentData.coordinates;
-                if (childCoordinates == null) {
-                    continue;
+                ModuleVersionIdentifier childCoordinates;
+                if (childComponent instanceof ComponentWithCoordinates) {
+                    childCoordinates = ((ComponentWithCoordinates)childComponent).getCoordinates();
+                } else {
+                    ComponentData componentData = componentCoordinates.get(childComponent);
+                    childCoordinates = componentData == null ? null : componentData.coordinates;
                 }
+
+                assert childCoordinates != null;
                 if (childComponent instanceof SoftwareComponentInternal) {
                     for (UsageContext usageContext : ((SoftwareComponentInternal) childComponent).getUsages()) {
                         if (!started) {
@@ -277,6 +285,7 @@ public class ModuleMetadataFileGenerator {
         writeDependencies(variant, jsonWriter);
         writeDependencyConstraints(variant, jsonWriter);
         writeArtifacts(publication, variant, jsonWriter);
+        writeCapabilities(variant, jsonWriter);
 
         jsonWriter.endObject();
     }
@@ -356,23 +365,11 @@ public class ModuleMetadataFileGenerator {
         jsonWriter.endArray();
     }
 
-    private void writeDependencyConstraints(UsageContext variant, JsonWriter jsonWriter) throws IOException {
-        if (variant.getDependencyConstraints().isEmpty()) {
-            return;
-        }
-        jsonWriter.name("dependencyConstraints");
-        jsonWriter.beginArray();
-        for (DependencyConstraint dependencyConstraint : variant.getDependencyConstraints()) {
-            writeDependency(dependencyConstraint, jsonWriter);
-        }
-        jsonWriter.endArray();
-    }
-
     private void writeDependency(Dependency dependency, JsonWriter jsonWriter) throws IOException {
         jsonWriter.beginObject();
         if (dependency instanceof ProjectDependency) {
             ProjectDependency projectDependency = (ProjectDependency) dependency;
-            ModuleVersionIdentifier identifier = projectDependencyResolver.resolve(projectDependency);
+            ModuleVersionIdentifier identifier = projectDependencyResolver.resolve(ModuleVersionIdentifier.class, projectDependency);
             jsonWriter.name("group");
             jsonWriter.value(identifier.getGroup());
             jsonWriter.name("module");
@@ -386,8 +383,6 @@ public class ModuleMetadataFileGenerator {
             VersionConstraint vc;
             if (dependency instanceof ModuleVersionSelector) {
                 vc = ((ExternalDependency) dependency).getVersionConstraint();
-            } else if (dependency instanceof DependencyConstraint) {
-                vc = ((DependencyConstraint) dependency).getVersionConstraint();
             } else {
                 vc = DefaultImmutableVersionConstraint.of(Strings.nullToEmpty(dependency.getVersion()));
             }
@@ -396,6 +391,38 @@ public class ModuleMetadataFileGenerator {
         if (dependency instanceof ModuleDependency) {
             writeExcludes((ModuleDependency) dependency, jsonWriter);
         }
+        String reason = dependency.getReason();
+        if (StringUtils.isNotEmpty(reason)) {
+            jsonWriter.name("reason");
+            jsonWriter.value(reason);
+        }
+        jsonWriter.endObject();
+    }
+
+    private void writeDependencyConstraints(UsageContext variant, JsonWriter jsonWriter) throws IOException {
+        if (variant.getDependencyConstraints().isEmpty()) {
+            return;
+        }
+        jsonWriter.name("dependencyConstraints");
+        jsonWriter.beginArray();
+        for (DependencyConstraint dependencyConstraint : variant.getDependencyConstraints()) {
+            writeDependencyConstraint(dependencyConstraint, jsonWriter);
+        }
+        jsonWriter.endArray();
+    }
+
+    private void writeDependencyConstraint(DependencyConstraint dependencyConstraint, JsonWriter jsonWriter) throws IOException {
+        jsonWriter.beginObject();
+        jsonWriter.name("group");
+        jsonWriter.value(dependencyConstraint.getGroup());
+        jsonWriter.name("module");
+        jsonWriter.value(dependencyConstraint.getName());
+        writeVersionConstraint(dependencyConstraint.getVersionConstraint(), jsonWriter);
+        String reason = dependencyConstraint.getReason();
+        if (StringUtils.isNotEmpty(reason)) {
+            jsonWriter.name("reason");
+            jsonWriter.value(reason);
+        }
         jsonWriter.endObject();
     }
 
@@ -426,6 +453,22 @@ public class ModuleMetadataFileGenerator {
         jsonWriter.endArray();
     }
 
+    private void writeCapabilities(UsageContext variant, JsonWriter jsonWriter) throws IOException {
+        Set<? extends Capability> capabilities = variant.getCapabilities();
+        if (!capabilities.isEmpty()) {
+            jsonWriter.name("capabilities");
+            jsonWriter.beginArray();
+            for (Capability capability : capabilities) {
+                jsonWriter.beginObject();
+                jsonWriter.name("group").value(capability.getGroup());
+                jsonWriter.name("name").value(capability.getName());
+                jsonWriter.name("version").value(capability.getVersion());
+                jsonWriter.endObject();
+            }
+            jsonWriter.endArray();
+        }
+    }
+
     private static class ComponentData {
         private final ModuleVersionIdentifier coordinates;
         private final ImmutableAttributes attributes;
