@@ -17,6 +17,7 @@ package org.gradle.plugins.signing;
 
 import groovy.lang.Closure;
 import org.gradle.api.Action;
+import org.gradle.api.Incubating;
 import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.Project;
 import org.gradle.api.Task;
@@ -30,6 +31,7 @@ import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.plugins.signing.signatory.Signatory;
 import org.gradle.plugins.signing.signatory.SignatoryProvider;
+import org.gradle.plugins.signing.signatory.internal.gnupg.GnupgSignatoryProvider;
 import org.gradle.plugins.signing.signatory.pgp.PgpSignatoryProvider;
 import org.gradle.plugins.signing.type.DefaultSignatureTypeProvider;
 import org.gradle.plugins.signing.type.SignatureType;
@@ -213,6 +215,15 @@ public class SigningExtension {
         this.configuration = configuration;
     }
 
+    /**
+     * Use GnuPG agent to perform signing work.
+     * @since 4.5
+     */
+    @Incubating
+    public void useGpgCmd() {
+        setSignatories(new GnupgSignatoryProvider());
+    }
+
     /**
      * The configuration that signature artifacts are added to.
      */
@@ -383,11 +394,11 @@ public class SigningExtension {
      * Signs the POM artifact for the given Maven deployment.
      *
      * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p>
-     * <pre autoTested=''>
+     * <pre class='autoTested'>
      * uploadArchives {
      *   repositories {
      *     mavenDeployer {
-     *       beforeDeployment { MavenDeployment deployment ->
+     *       beforeDeployment { MavenDeployment deployment -&gt;
      *         signing.signPom(deployment)
      *       }
      *     }
@@ -431,11 +442,11 @@ public class SigningExtension {
      * Signs the POM artifact for the given Maven deployment.
      *
      * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p>
-     * <pre autoTested=''>
+     * <pre class='autoTested'>
      * uploadArchives {
      *   repositories {
      *     mavenDeployer {
-     *       beforeDeployment { MavenDeployment deployment ->
+     *       beforeDeployment { MavenDeployment deployment -&gt;
      *         signing.signPom(deployment)
      *       }
      *     }
