@@ -16,7 +16,7 @@
 
 package org.gradle.api.internal.tasks.testing;
 
-import org.gradle.api.Nullable;
+import javax.annotation.Nullable;
 
 public abstract class AbstractTestDescriptor implements TestDescriptorInternal {
     private final Object id;
@@ -52,4 +52,14 @@ public abstract class AbstractTestDescriptor implements TestDescriptorInternal {
     public Object getOwnerBuildOperationId() {
         return null;
     }
+
+    @Override
+    public String getDisplayName() {
+        return getName();
+    }
+
+    @Override
+    public String getClassDisplayName() {
+        return getClassName();
+    }
 }
