@@ -16,9 +16,12 @@
 
 package org.gradle.api.internal.tasks.testing;
 
-import org.gradle.api.Nullable;
 import org.gradle.api.tasks.testing.TestDescriptor;
+import org.gradle.internal.scan.UsedByScanPlugin;
 
+import javax.annotation.Nullable;
+
+@UsedByScanPlugin
 public interface TestDescriptorInternal extends TestDescriptor {
     @Nullable
     @Override
@@ -32,4 +35,17 @@ public interface TestDescriptorInternal extends TestDescriptor {
      */
     @Nullable
     Object getOwnerBuildOperationId();
+
+    /**
+     * The name for display. It may be the same as or different from {@link #getName()}
+     *
+     * @return the name for display.
+     */
+    String getDisplayName();
+
+    /**
+     * The class name for display. It may be the same as or different from {@link #getClassName()}
+     * @return the class name for display.
+     */
+    String getClassDisplayName();
 }
