@@ -16,7 +16,7 @@
 
 package org.gradle.workers.internal;
 
-import org.gradle.internal.progress.BuildOperationState;
+import org.gradle.internal.operations.BuildOperationRef;
 import org.gradle.internal.work.WorkerLeaseRegistry.WorkerLease;
 
 /**
@@ -24,5 +24,5 @@ import org.gradle.internal.work.WorkerLeaseRegistry.WorkerLease;
  */
 public interface Worker {
     DefaultWorkResult execute(ActionExecutionSpec spec);
-    DefaultWorkResult execute(ActionExecutionSpec spec, WorkerLease parentWorkerWorkerLease, final BuildOperationState parentBuildOperation);
+    DefaultWorkResult execute(ActionExecutionSpec spec, WorkerLease parentWorkerWorkerLease, final BuildOperationRef parentBuildOperation);
 }
